package FI2.Dictionary;
/**{p}
 * -- FI2.Dictionary --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Dizionari
 */

/**{f}
 * OrderedArraySeq.java
 * dizionario realizzato in forma di sequenza
 * ordinata
 *
 * @version 2.00 00/09/25
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Exceptions.InvalidKeyException;
import FI2.Linear.Sequence;
import FI2.Linear.ArraySequence;
import FI2.Set.InspectableContainer;
import FI2.Set.InspectableKeyBasedContainer;
import FI2.Set.Container;
import FI2.Set.Item;
import FI2.Set.Position;
import FI2.Set.ObjectIterator;
import FI2.Set.PositionIterator;
import FI2.Util.Comparator;
import FI2.Util.FI2Sys;
import FI2.Util.VectorIter;

/**{c}
 * sequenza su array ordinata - si appoggia su un
 * Sequence in cui e` imposto l'ordinamento delle
 * chiavi mediante ricerca binaria, utilizzata sia
 * per le ricerche che per l'inserimento -
 * gli elementi sono Item chiave-oggetto -
 * la comparazione delle chiavi avviene 
 * mediante un comparatore fornito all'atto della costruzione
 * @see OrderedDictionary
 */
public class OrderedArraySeq 
  implements OrderedDictionary  
{
    private Comparator C;  // comparatore
    private Sequence S;  // sequenza

    /**[c]
     * costruttore base, sequenza vuota
     * @param c  comparatore per i confronti di chiave
     */
    public OrderedArraySeq(Comparator c)  
    {
        C = c;
        S = (Sequence) new ArraySequence();
    } //[c] OrderedArraySeq

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua l'ordinamento degli elementi -
     * gli oggetti non vengono clonati 
     * @param key  array di chiavi da copiare
     * @param val  array di dati da copiare
     * @param c  comparatore per i confronti di chiave
     */
    public OrderedArraySeq(Object key[], 
      Object val[], Comparator c)
    {
        C = c;
        S = (Sequence) new ArraySequence();
        for (int i=0; i<key.length; i++)
            insertItem(key[i], val[i]);
    }  //[c] OrderedArraySeq


    // metodi ausiliari

    /**[m]
     * chiave associata ad una posizione
     * @param position  posizionatore
     * @return la chiave 'posizionata'
     */
    protected Object key(Position position)  
    {
        return ((Item) position.element()).key();
    } //[m] key

    /**[m]
     * elemento associato ad una posizione
     * @param position  posizionatore
     * @return l'elemento 'posizionato'
     */
    protected Object element(Position position)  
    { 
        return ((Item) position.element()).element(); 
    } //[m] element

    /**[m]
     * verifica se la chiave e` legale
     * @param key  chiave da verificare
     * @exception InvalidKeyException
     * se chiave illegale
     */
    protected void checkKey(Object key) 
      throws InvalidKeyException 
    {
        if (!C.isComparable(key))
          throw new InvalidKeyException(
          "La chiave <"+key+"> non e` confrontabile");
    } //[m] checkKey

    /**[m]
     * ricerca binaria (a bisezione) di una chiave -
     * in caso di chiavi multiple effettua una ricerca
     * lineare (locale) per trovare prima e ultima -
     * si assume che il numero di ripetizioni di chiave
     * sia trascurabile rispetto alla dimensione del
     * dizionario
     * @param key  chiave da ricercare
     * @return int[0] la posizione (0..size-1) della prima
     *         istanza, se gia` presente, -1 altrimenti
     *         int[1] la posizione (0..size-1) dell'ultima
     *         istanza, se gia` presente;
     *         la posizione di inserimento (0..size) 
     *         se non trovato
     */
    protected int[] findPosition(Object key) 
    {
        int min = 0, med, max = S.size()-1;
        boolean foundFl=false, greaterFl = false;

        int ret[] = new int[2];
        if (max < 0)
        {
            // caso sequenza vuota
            ret[0] = -1;
            ret[1] = 0;
            return ret;
        }
        do {
            med = (min+max)/2;
            if (greaterFl = 
              C.isGreaterThan(key(S.atRank(med)), key))
                // l'elemento e` nella prima meta`
                max = med -1;
            else
            {
                // l'elemento e` nella seconda meta`
                foundFl = C.isEqualTo(key(S.atRank(med)), 
                  key);
                min = med + 1;
            } //else
        } while (min <= max && !foundFl);
            // esce se ricerca esaurita oppure 
            // se l'oggetto e` stato trovato
        if (foundFl)
        {
            // ricerca la prima istanza
            for (min = med; 
              min >= 1 && 
              C.isEqualTo(key(S.atRank(min-1)), key); min--);
            ret [0] = min;
            // ricerca l'ultima istanza
            for (max = med; 
              max <= S.size()-2 &&
              C.isEqualTo(key(S.atRank(max+1)), key); max++);
            ret [1] = max;
        }
        else 
        {
            // non trovato
            ret[0] = -1;
            ret[1] = (greaterFl) ? med : med+1;
        }
        return ret;
    } //[m] findPosition

    // metodi dell'interfaccia Dictionary

    /**[m]
     * @see InspectableContainer
     */
    public int size()  { return S.size(); }

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty() { return S.isEmpty(); }

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere i valori -
         * si appoggia sull'iteratore di sequenza
         */
        class ElemsIter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore sulla sequenza

            /**[c]
             * costruttore di default
             */
            ElemsIter()
            {
                posIt = S.positions();
            } //[c] ElemsIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return posIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!posIt.hasNext())
                    throw new NoSuchElementException(
"OrderedArraySeq.ElemsIter.nextObject(): fine elementi"
                  );
                  return element(
                    posIt.nextPosition());
            } //[m] nextObject

        } //{c}{l} ElemsIter

        return new ElemsIter();
    } //[m] elements

   /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new OrderedArraySeq(C);
    } //[m] newContainer

    /**[m]
     * @see InspectableKeyBasedContainer
     */
    public ObjectIterator keys()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere le chiavi -
         * si appoggia sull'iteratore di sequenza -
         * l'iteratore e` sempre posizionato sulla chiave
         * successiva (al limite alla fine della sequenza) -
         * NO_SUCH_KEY viene usato per marcare la fine delle
         * chiavi
         */
        class KeysIter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore sulla sequenza
            private Object nextKey;
              // prossima chiave

            /**[c]
             * costruttore di default
             */
            KeysIter()
            {
                posIt = S.positions();
                nextKey = posIt.hasNext() ?
                  key(posIt.nextPosition()) :
                  NO_SUCH_KEY;
            } //[c] KeysIter

             /**[m]
             * @see ObjectIteratore
             */
            public boolean hasNext()
            {
                return nextKey != NO_SUCH_KEY;
            } //[m] hasNext

            /**[m]
             * @see ObjectIteratore
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"OrderedArraySeq.KeysIter.nextObject(): fine elementi"
                  );
                Object keyRet = nextKey, key = nextKey;
                  // ritorna la chiave memorizzata

                while(posIt.hasNext() && 
                  C.isEqualTo(key = 
                  key(posIt.nextPosition()), 
                  nextKey));
                  // cerca la chiave successiva
                if (C.isEqualTo(key, nextKey))
                  // chiavi esaurite
                    nextKey = NO_SUCH_KEY;
                else
                    nextKey = key;
                return keyRet;
            } //[m] nextObject

        } //{c}{l} KeysIter

        return new KeysIter();
    } //[m] keys

    /**[m]
     * @see InspectableDictionary
     */
    public Object findElement(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
        if (curPos[0] != -1)
            // trovata la chiave
            return element(S.atRank(curPos[0]));
        else
            // non trovata la chiave
            return NO_SUCH_KEY;
    } //[m] findElement

    /**[m]
     * @see InspectableDictionary
     */
    public ObjectIterator findAllElements(Object key) 
      throws InvalidKeyException  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per una certa chiave
         */
        class ElemIter implements ObjectIterator
        {
            private int[] curPos;
              // indici delle occorrenze trovate
            private int curRank;
              // indice corrente

            /**[c]
             * costruttore di default
             */
            ElemIter(Object key)
            {
                curPos = findPosition(key);
                curRank = curPos[0];
            } //[c] ElemIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return curRank != -1;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"OrderedArraySeq.ElemIter.nextObject(): fine elementi"
                  );
                Object toReturn = element(
                  S.atRank(curRank++));
                if (curRank > curPos[1])
                    // fine occorrenze
                    curRank = -1;
                return toReturn;
            } //[m] nextObject

        } //{c}{l} ElemIter

        checkKey(key);
          // puo` sollevare InvalidKeyException
        return new ElemIter(key);
    } //[m] findAllElements

    /**[m]
     * @see Dictionary
     */
    public void insertItem(Object key, Object element)
      throws InvalidKeyException  
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);

        if (curPos[0] != -1)
            // chiave gia` presente: inserisce
            // nella posizione che segue immediatamente
            // l'ultimo elemento con la chiave cercata
            curPos[1]++;
        S.insertAtRank(curPos[1], new Item(key, element));
    } //[m] insertItem

    /**[m]
     * @see Dictionary
     */
    public Object replaceElement(Object key, Object element) 
      throws InvalidKeyException  
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
          // posizione di rimozione
        if(curPos[0] == -1) 
            // chiave non trovata
            return NO_SUCH_KEY;
        Object toReturn = element(S.atRank(curPos[0]));
          // elemento da restituire
        S.replaceAtRank(curPos[0], 
          new Item(key, element));
        return toReturn;
    } //[m] replaceElement

    /**[m]
     * @see Dictionary
     */
    public Object removeElement(Object key) 
      throws InvalidKeyException  
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
          // posizione di rimozione
        if(curPos[0] == -1) 
            // chiave non trovata
            return NO_SUCH_KEY;
        return ((Item)S.removeAtRank(
          curPos[0])).element();
          // elemento da restituire
    } //[m] removeElement

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator removeAllElements(Object key) 
      throws InvalidKeyException  
    {
        VectorIter elems = new VectorIter();
        Object curElement;
        
        while((curElement = removeElement(key)) != NO_SUCH_KEY)
            elems.insertLast(curElement);
        
        return elems;
    } //[m] removeAllElements

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestKeyBefore(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
          // posizione di ricerca
        if (curPos[0] != -1) 
            // chiave trovata
            return key;
        if (curPos[1] == 0)
          // e` minore del minimo
            return NO_SUCH_KEY;
        return key(S.atRank(curPos[1]-1));
          // elemento da restituire
    } //[m] closestKeyBefore

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestElemBefore(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
          // posizione di ricerca
        if (curPos[0] != -1) 
            // chiave trovata, restituisce la prima occorrenza
            return element(S.atRank(curPos[0]));
        if (curPos[1] == 0)
          // e` minore del minimo
            return NO_SUCH_KEY;
        return element(S.atRank(curPos[1]-1));
          // elemento da restituire
    } //[m] closestElemBefore

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestKeyAfter(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
          // posizione di ricerca
        if (curPos[0] != -1) 
            // chiave trovata
            return key;
        if (curPos[1] >= S.size())
          // e` maggiore del massimo
            return NO_SUCH_KEY;
        return key(S.atRank(curPos[1]));
          // elemento da restituire
    } //[m] closestKeyAfter

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestElemAfter(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        int[] curPos = findPosition(key);
          // posizione di ricerca
        if (curPos[0] != -1) 
            // chiave trovata, restituisce l'ultima occorrenza
            return element(S.atRank(curPos[1]));
        if (curPos[1] >= S.size())
          // e` maggiore del massimo
            return NO_SUCH_KEY;
        return element(S.atRank(curPos[1]));
          // elemento da restituire
    } //[m] closestElemAfter

    /**[m]
     * conversione a stringa -
     * produce il contenuto dell'albero
     * @return stringa contenuto del dizionario 
     */
    public String toString()
    {
        return "Dizionario con ArraySequence\r\n" +
            S.toString();
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
 "Dizionario: Battere codice 6 cifre + stringa (fine=Ctrl-Z):"
              );
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));

        OrderedArraySeq myDictio = 
          new OrderedArraySeq(new AKeyComp());

        try
        {
            String str;
            while((str = re.readLine()) != null)
                myDictio.insertItem(str.substring(0,6), 
                  str.substring(6));
                  // la chiave sono i primi 6 caratteri
                  // il resto i dati
        }
        catch (IOException e)
        {
            // termina il ciclo
        } //catch(EOF)

        FI2Sys.out.println(myDictio);

        FI2Sys.out.println("numEl=" + myDictio.size());
        FI2Sys.out.println("I primi 3 elementi sono:");
        ObjectIterator it = myDictio.elements();

        int pos1 = 1;
        while (pos1++ <= 3 && it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());

        Object tmp;
        FI2Sys.out.println("La stringa con chiave 123456 " + (
          (tmp = myDictio.findElement("123456BOH")) == 
           NO_SUCH_KEY ?
          "non e` presente" :
          ("e` presente e vale ->" + tmp + "<-")
          )
          );

        it = myDictio.findAllElements("123456BOH");
        FI2Sys.out.println(
          "Le stringhe con chiave 123456 sono:");
        while (it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());
        // l'enumerazione deve tener conto che
        // le chiavi possono essere multiple
        ObjectIterator itk = myDictio.keys();
        FI2Sys.out.println("Dizionario completo:");
        VectorIter vi = new VectorIter();
        while (itk.hasNext())
        {
            Object key = itk.nextObject();
            it = myDictio.findAllElements(key);
            
            // la prima viene copiata
            if (it.hasNext())
            {
                Object val;
                vi.insertLast(
                  new Item(key, val = it.nextObject()));
                FI2Sys.out.println("key=" + key + 
                  " val=" + val);
            }
            while (it.hasNext())
                FI2Sys.out.println("key=" + key + 
                  " val=" + it.nextObject());
        } // while enk

        Object tmpkey[] = new
          Object[vi.size()];
        Object tmpval[] = new
          Object[vi.size()];
        int i = 0;
        while(vi.hasNext())
        {
            Item ie = (Item)vi.nextObject();
            tmpkey[i] = ie.key();
            tmpval[i++] = ie.element();
        }
        
        FI2Sys.out.println(
  "La maggiore chiave <= di 666666 e` " + 
          myDictio.closestKeyBefore("666666") +
          " di valore " +
          myDictio.closestElemBefore("666666"));
        FI2Sys.out.println(
  "La minor chiave >= di 666666 e` " + 
          myDictio.closestKeyAfter("666666") +
          " di valore " +
          myDictio.closestElemAfter("666666"));

        FI2Sys.out.println(
  "La maggiore chiave <= di 555555 e` " + 
          myDictio.closestKeyBefore("555555") +
          " di valore " +
          myDictio.closestElemBefore("555555"));
        FI2Sys.out.println(
  "La minor chiave >= di 555555 e` " + 
          myDictio.closestKeyAfter("555555") +
          " di valore " +
          myDictio.closestElemAfter("555555"));

        Dictionary sndDictio = 
          new OrderedArraySeq(tmpkey, 
          tmpval, new AKeyComp());
        sndDictio.removeElement("654321");
        sndDictio.replaceElement("123456", 
          "------- NUOVO -------");
        FI2Sys.out.println(
  "Il secondo senza la chiave 654321 e con 123456 aggiornato:"
          );
        FI2Sys.out.println(sndDictio);
        FI2Sys.out.println("numEl=" + sndDictio.size());
    } //[m][s] main

} //{c} OrderedArraySeq
