package FI2.Dictionary;
/**{p}
 * -- FI2.Dictionary --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Dizionari
 */

/**{f}
 * UnOrderedSLList.java
 * dizionario in forma di sequenza
 * non ordinata (log file) realizzata con lista
 * concatenata semplice -
 * derivata da UnOrderedSListSeq della v 1.x 
 *
 * @version 2.00 00/09/25
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.ObjectIterator;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.Position;
import FI2.Set.PositionIterator;
import FI2.Set.Item;
import FI2.Exceptions.InvalidKeyException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Util.Equality;
import FI2.Util.FI2Sys;
import FI2.Util.VectorIter;
import FI2.Linear.List;
import FI2.Linear.SLList;

/**{c}
 * sequenza su lista concatenata semplice non ordinata - 
 * gli elementi sono Item chiave-oggetto -
 * la comparazione delle chiavi avviene 
 * mediante un comparatore di eguaglianza fornito 
 * all'atto della costruzione
 * @see Dictionary
 */
public class UnOrderedSLList 
  implements Dictionary  
{
    private Equality E;  // comparatore
    private List S;  // sequenza

    /**[c]
     * costruttore base, sequenza vuota
     * @param e  comparatore per i confronti di chiave
     */
    public UnOrderedSLList(Equality e)  
    {
        if ((E = e) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        S = new SLList();
    } //[c] UnOrderedSLList

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * gli oggetti non vengono clonati 
     * @param key  array di chiavi da copiare
     * @param val  array di dati da copiare
     * @param e  comparatore per i confronti di chiave
     */
    public UnOrderedSLList(Object key[], Object val[], 
      Equality e)
    {
        if ((E = e) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        S = new SLList();
        for (int i=0; i<key.length; i++)
            insertItem(key[i], val[i]);
    }  //[c] UnOrderedSLList


    // metodi ausiliari

    /**[m]
     * chiave associata ad una posizione
     * @param position  posizionatore
     * @return la chiave 'posizionata'
     */
    protected Object key(Position position)  
    {
        return ((Item) position.element()).key();
    } //[m] key

    /**[m]
     * elemento associato ad una posizione
     * @param position  posizionatore
     * @return l'elemento 'posizionato'
     */
    protected Object element(Position position)  
    { 
        return ((Item) position.element()).element(); 
    } //[m] element

    /**[m]
     * verifica se la chiave e` legale
     * @param key  chiave da verificare
     * @exception InvalidKeyException
     * se chiave illegale
     */
    protected void checkKey(Object key) 
      throws InvalidKeyException 
    {
        if (!E.isComparable(key))
          throw new InvalidKeyException(
          "La chiave <"+key+"> non e` confrontabile");
    } //[m] checkKey

    /**[m]
     * ricerca la posizione di una chiave -
     * la ricerca e` lineare a partire dalla
     * posizione fornita -
     * O(n)
     * @param key  chiave da ricercare
     * @param pos  prima posizione da verificare oppure
     *             null se si vuole inziare dal principio
     * @return posizione trovata oppure null
     */
    protected Position findPosition(Object key, Position pos) 
    {
        if (pos == null)
            if (S.isEmpty())
                // vuota
                return null;
            else
                pos = S.first();
        // else ;

        while (!E.isEqualTo(key(pos), key))
            try
            {
                pos = S.after(pos);
            }
            catch (BoundaryViolationException e)
            {
                // non trovata
                return null;
            }
        return pos;
    } //[m] findPosition

    // metodi dell'interfaccia Dictionary

    /**[m]
     * @see InspectableContainer
     */
    public int size()  { return S.size(); }

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty() { return S.isEmpty(); }

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere i valori -
         * si appoggia sull'iteratore di sequenza
         */
        class ElemsIter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore sulla sequenza

            /**[c]
             * costruttore di default
             */
            ElemsIter()
            {
                posIt = S.positions();
            } //[c] ElemsIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return posIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!posIt.hasNext())
                    throw new NoSuchElementException(
"UnOrderedSListSeq.ElemsIter.nextObject(): fine elementi"
                  );
                  return 
                    element(posIt.nextPosition());
            } //[m] nextElement

        } //{c}{l} ElemsIter

        return new ElemsIter();
    } //[m] elements

   /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new UnOrderedSLList(E);
          // usa lo stesso comparatore di this
    } //[m] newContainer

    /**[m]
     * O(n)
     * @see Dictionary
     */
    public Object findElement(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        // ricerca lineare
        Position pos;
        if ((pos = findPosition(key, null)) == null)
            // non trovata
            return NO_SUCH_KEY;
        return element(pos);
    } //[m] findElement

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator findAllElements(Object key) 
      throws InvalidKeyException  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per una certa chiave
         */
        class ElemIter implements ObjectIterator
        {
            private Object curKey;
              // chiave di ricerca
            private Position curPos;
              // posizione corrente

            /**[c]
             * costruttore di default
             */
            ElemIter(Object key)
            {
                curKey = key;
                curPos = findPosition(key, null);
            } //[c] ElemIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return curPos != null;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"UnOrderedSLList.ElemIter.nextObject(): fine elementi"
                  );
                Object toReturn = element(curPos);
                try
                {
                    curPos = 
                      findPosition(curKey, S.after(curPos));
                }
                catch (BoundaryViolationException e)
                {
                    // non ce ne sono piu`
                    curPos = null;
                }
                return toReturn;
            } //[m] nextObject

        } //{c}{l} ElemIter

        checkKey(key);
          // puo` sollevare InvalidKeyException
        return new ElemIter(key);
    } //[m] findAllElements

    /**[m]
     * @see Dictionary
     */
    public void insertItem(Object key, Object element)
      throws InvalidKeyException  
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        // si inserisce alla fine
        S.insertLast(new Item(key, element));
    } //[m] insertItem

    /**[m]
     * @see Dictionary
     */
    public Object replaceElement(Object key, Object element) 
      throws InvalidKeyException  
    {
        Position curPos;
        checkKey(key);
          // puo` sollevare InvalidKeyException
        if ((curPos = findPosition(key, null)) == null)
            // chiave non trovata
            return NO_SUCH_KEY;
        Object toReturn = element(curPos);
          // elemento da restituire
        S.replaceElement(curPos, 
          new Item(key, element));
        return toReturn;
    } //[m] replaceElem

    /**[m]
     * @see Dictionary
     */
    public Object removeElement(Object key) 
      throws InvalidKeyException  
    {
        Position curPos;
        checkKey(key);
          // puo` sollevare InvalidKeyException
        if ((curPos = findPosition(key, null)) == null)
            // chiave non trovata
            return NO_SUCH_KEY;
        return ((Item)S.remove(curPos)).element();
    } //[m] remove

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator removeAllElements(Object key) 
      throws InvalidKeyException  
    {
        VectorIter elems = new VectorIter();
        Object curElement;

        while((curElement = removeElement(key)) != NO_SUCH_KEY)
            elems.insertLast(curElement);

        return elems;
    } //[m] removeAll

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator keys()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere le chiavi -
         * si appoggia sull'iteratore di sequenza -
         * l'iteratore e` sempre posizionato sulla chiave
         * successiva (al limite alla fine della sequenza) -
         * NO_SUCH_KEY viene usato per marcare la fine delle
         * chiavi - per semplicita`, 
         * se ci sono chiavi multiple, a causa
         * del non ordinamento, vengono restituite piu`
         * volte
         */
        class KeysIter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore sulla sequenza
            private Object nextKey;
              // prossima chiave

            /**[c]
             * costruttore di default
             */
            KeysIter()
            {
                posIt = S.positions();
                nextKey = posIt.hasNext() ?
                  key(posIt.nextPosition()) :
                  NO_SUCH_KEY;
            } //[c] KeysIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return nextKey != NO_SUCH_KEY;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"UnOrderedSLList.KeysIter.nextObject(): fine elementi"
                  );
                Object keyRet = nextKey, key = nextKey;
                  // ritorna la chiave memorizzata
                Position curPos;
                // cerca la chiave successiva
                while(posIt.hasNext() && 
                  findPosition
                  (key = key(curPos=
                    posIt.nextPosition()),
                  null) != curPos)
                  ;
                  // salta se si trova una chiave che e` 
                  // presente anche prima
                if (E.isEqualTo(key, nextKey))
                  // chiavi esaurite
                    nextKey = NO_SUCH_KEY;
                else
                    nextKey = key;
                return keyRet;
            } //[m] nextObject

        } //{c}{l} KeysIter

        return new KeysIter();
    } //[m] keys


    /**[m]
     * conversione a stringa -
     * produce il contenuto dell'albero
     * @return stringa contenuto del dizionario 
     */
    public String toString()
    {
        return "Dizionario con SimpleLinkedList\r\n" +
            S.toString() + "\r\n";
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
"Dizionario: Battere codice 6 cifre + stringa (fine=Ctrl-Z):"
              );
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));

        UnOrderedSLList myDictio = 
          new UnOrderedSLList(new AKeyComp());

        try
        {
            String str;
            while((str = re.readLine()) != null)
                myDictio.insertItem(str.substring(0,6), 
                  str.substring(6));
                  // la chiave sono i primi 6 caratteri
                  // il resto i dati
        }
        catch (IOException e)
        {
            // termina il ciclo
        } //catch(EOF)

        FI2Sys.out.println(myDictio);

        FI2Sys.out.println("numEl=" + myDictio.size());
        FI2Sys.out.println("I primi 3 elementi sono:");
        ObjectIterator it = myDictio.elements();

        int pos1 = 1;
        while (pos1++ <= 3 && it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());

        Object tmp;
        FI2Sys.out.println("La stringa con chiave 123456 " + (
          (tmp = myDictio.findElement("123456BOH")) == 
           NO_SUCH_KEY ?
          "non e` presente" :
          ("e` presente e vale ->" + tmp + "<-")
          )
          );

        it = myDictio.findAllElements("123456BOH");
        FI2Sys.out.println(
          "Le stringhe con chiave 123456 sono:");
        while (it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());
        ObjectIterator itk = myDictio.keys();
        FI2Sys.out.println("Dizionario completo:");
        VectorIter vi = new VectorIter();
        while (itk.hasNext())
        {
            Object key = itk.nextObject();
            it = myDictio.findAllElements(key);
            
            // la prima viene copiata
            if (it.hasNext())
            {
                Object val;
                vi.insertLast(
                  new Item(key, val = it.nextObject()));
                FI2Sys.out.println("key=" + key + 
                  " val=" + val);
            }
            while (it.hasNext())
                FI2Sys.out.println("key=" + key + 
                  " val=" + it.nextObject());
        } // while itk

        Object tmpkey[] = new
          Object[vi.size()];
        Object tmpval[] = new
          Object[vi.size()];
        int i = 0;
        while(vi.hasNext())
        {
            Item ie = (Item)vi.nextObject();
            tmpkey[i] = ie.key();
            tmpval[i++] = ie.element();
        }
        
        Dictionary sndDictio = 
          new UnOrderedSLList(tmpkey, tmpval,
          new AKeyComp());
        sndDictio.removeElement("654321");
        sndDictio.replaceElement(
          "123456", "------- NUOVO -------");
        FI2Sys.out.println(
"Il secondo senza la chiave 654321 e con 123456 aggiornato:"
          );
        FI2Sys.out.println(sndDictio);
        FI2Sys.out.println("numEl=" + sndDictio.size());
    } //[m][s] main

} //{c} UnOrderedSLList
