package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi
 */

/**{f}
 * DVect.java
 * realizzazione di una classe derivata da PointE
 *
 * @version 2.00 98/09/24
 * @author M.Moro DEI UPD
 */

/**{c}
 * Vettore orientato: aggiunge a PointE i metodi algebrici
 * @see PointE
 */
public class DVect extends PointE
{
    /**[c]
     * costruttore base in coordinate polari
     * @param rVal  ro (modulo)
     * @param tVal  theta (angolo)
     */
    public DVect(double rVal, double tVal)
    {
        super(true, rVal, tVal);
            // costruttore della superclasse
    } //[c] DVect

    /**[c]
     * costruttore base in coordinate cartesiane
     * @param xVal  ascissa
     * @param yVal  ordinata
     * @param dummy  fittizio
     * (per avere un contratto diverso
     * dal costruttore precedente)
     */
    public DVect(double xVal, double yVal, int dummy)
    {
        super(false, xVal, yVal);
            // costruttore della superclasse
    } //[c] DVect

    /**[c]
     * costruttore di default
     */
     public DVect()
     {
     } //[c] DVect
     
    /**[m][s]
     * Il vettore nullo
     * @return oggetto vettore nullo 
     */
     public static DVect zero()
     {
         return new DVect();
     } //[m] zero

    /**[m]
     * addizione di due vettori (x=this.x+x2 y=this.y+y2)
     * @param SndDv  secondo vettore
     * @return oggetto somma dei due vettori forniti
     */
    public DVect add(DVect sndDv)
    {
        return new DVect(x+sndDv.x, y+sndDv.y, 0);
    } //[m] add

    /**[m]
     * sottrazione di due vettori (x=this.x-x2 y=this.y-y2)
     * @param sndDv  secondo vettore
     * @return oggetto differenza dei due vettori forniti
     */
    public DVect sub(DVect sndDv)
    {
        return new DVect(x-sndDv.x, y-sndDv.y, 0);
    } //[m] sub

    /**[m]
     * vettore opposto (x=-this.x y=-this.y)
     * @return oggetto opposto
     */
    public DVect opp()
    {
        return new DVect(-x, -y, 0);
    } //[m] opp

    /**[m]
     * moltiplicazione di due vettori
     * (ro=this.ro*ro2 th=this.th+th2)
     * @param sndDv  secondo vettore
     * @return oggetto prodotto dei due vettori forniti
     */
    public DVect mul(DVect sndDv)
    {
        return new DVect(rVal()*sndDv.rVal(),
          tVal()+sndDv.tVal());
    } //[m] mul

    /**[m]
     * divisione di due vettori
     * (ro=this.ro/ro2 th=this.th-th2)
     * @param sndDv  secondo vettore
     * @return oggetto quoziente dei due vettori forniti
     */
    public DVect div(DVect sndDv)
    {
        return new DVect(rVal()/sndDv.rVal(),
          tVal()-sndDv.tVal());
    } //[m] div

    /**[m]
     * vettore inverso (ro=1/this.ro th=-this.th)
     * @return oggetto inverso
     */
    public DVect inv()
    {
        return new DVect(1/rVal(), -tVal());
    } //[m] inv

    /**[m]
     * rotazione di angolo fornito
     * rende pubblico il metodo ereditato
     * @param angle  angolo di rotazione
     */
    public void rot(double angle)
    {
        super.rot(angle);
    } //[m] rot

    /**[m]
     * conversione a stringa
     * @return descrizione del contenuto
     */
    public String toString()
    {
        return "{DVect} ro=" + rVal() + " theta=" + tVal();
    }

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  Ve1.ro 
     * @param args[1]  Ve1.theta
     * @param args[2]  Ve2.ro
     * @param args[3]  Ve2.theta
     */
    public static void main(String[] args)
    {
        DVect ve1, ve2;

        if (args.length >= 4) {
            // ci sono i parametri di chiamata
            ve1 = new
              DVect(
              Double.valueOf(args[0]).doubleValue(),
              Double.valueOf(args[1]).doubleValue());
            ve2 = new
              DVect(
              Double.valueOf(args[2]).doubleValue(),
              Double.valueOf(args[3]).doubleValue());
        }
        else {
            // default
            ve1 = new
              DVect(2.0, Math.PI/6);
            ve2 = new
              DVect(3.0, Math.PI*5/4);
        }

        System.out.println("ve1 e` " + ve1);
        System.out.println("ve1 in cartesiane e` x=" +
          ve1.xVal() + " y=" + ve1.yVal());
            // polimorfismo
        System.out.println("ve2 e` " + ve2);
        System.out.println("La somma e` " +
      ve1.add(ve2));
        System.out.println("La differenza e` " +
      ve1.sub(ve2));
        System.out.println("L'opposto di ve1 e` " +
          ve1.opp());
        System.out.println("Il prodotto e` " +
      ve1.mul(ve2));
        System.out.println("Il quoziente e` " +
      ve1.div(ve2));
        System.out.println("L'inverso di ve1 e` " +
      ve1.inv());
        System.out.println("La distanza e` " +
      ve1.dist(ve2));
            // polimorfismo

        ve1.rot(-Math.PI/6);
        System.out.println("ve1 ruotato di -Pi/6 e` " +
      ve1);

        PointE p1 = ve1;
        System.out.println(
      "ve1 ruotato in cartesiane e` x=" +
          p1.xVal() + " y=" + p1.yVal());
            // polimorfismo

        System.out.println(
      "L'opposto di ve1 ruotato e` " +
          ((DVect)p1).opp());
            /* il cast e` necessario e il programmatore 
            assicura che l'oggetto riferito da p1 e` 
            effettivamente di tipo DVect */

        p1 = DVect.zero();
        System.out.println("L'origine come punto e` " +
          p1 + " e come vettore e` " + (DVect)p1);
            // polimorfismo
            /* in realta` il metodo toString chiamato nei due
            casi e` lo stesso ed e` quello della classe
            dell'oggetto correntemente riferito (DVect).
            Questo accade poiche` e` presente con lo stesso
            contratto sia nella classe derivata
            che nella superclasse */
    } //[m] main

} //{c} DVect

