package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * DrawFig.java
 * Esercizio 7.2 -
 * disegno di 4 figure
 *
 * @version 2.00 00/09/29
 * @author M.Moro DEI UPD
 */

import java.awt.*;
import java.awt.event.*;
import FI2.Set.ObjectIterator;
import FI2.Linear.ArraySequence;
import FI2.Linear.Sequence;
import FI2.Util.*;

/**{c}{a}
 * selezione per figura
 */
abstract class SelFigure implements DrawMovable
{
    protected boolean selected = false;
      // selezione

    /**[m]
     * interrogazione
     * @return true se selezionato
     */
    boolean selected() { return selected; }

    /**[m]
     * impostazione selezionato
     */
    void select() { selected = true; }

    /**[m]
     * impostazione non selezionato
     */
    void deselect() { selected = false; }

    /**[m]
     * commutazione selezione
     */
    void changeSelect() { selected = ! selected; }
} //{c}{a} SelFigure 

/**{c}
 * un cerchio
 */
class DCirc extends SelFigure
{
    PointE center;  // centro
    double rad;  // raggio

    /**[c]
     * costruttore base
     * @param cx  x del centro
     * @param cy  y del centro
     * @param r  raggio
     */
    DCirc(double cx, double cy, double r)
    {
        center = new PointE(cx, cy);
        rad = r;
    } //[c] DCirc

    /**[c]
     * @see DrawMovable
     */
    public void draw(Graphics g)
    {
        if (selected)
            g.setColor(java.awt.Color.red);
        else
            g.setColor(java.awt.Color.black);
        g.drawOval((int)(center.xVal()-rad), (int)(center.yVal()-rad), 
          (int)(rad*2), (int)(rad*2));
    } //[m] draw

    /**[c]
     * @see DrawMovable
     */
    public void move(PointE trasl)
    {
        center = new PointE(center.xVal()+trasl.xVal(),
          center.yVal()+trasl.yVal());
    } //[m] move

    /**[c]
     * @see DrawMovable
     */
    public boolean contains(PointE p)
    {
        // appartiene se distanza dal centro <= raggio
        return center.dist(p) <= rad;
    } //[m] contains

    /**[m]
     * conversione a stringa
     * @return stringa descrittiva
     */
    public String toString()
    {
        return "[cerchio cx="+center.xVal()+" cy="+center.yVal()+" raggio="+rad;
    } //[m] toString

} //{c} DCirc

/**{c}
 * un rettangolo
 */
class DRect extends SelFigure implements DrawMovable
{
    PointE left;  // angolo sinistro alto
    PointE right;  // angolo destro basso

    /**[c]
     * costruttore base
     * @param x1  x angolo sinistra alto
     * @param y1  y angolo sinistra alto
     * @param x2  x angolo destra basso
     * @param y3  y angolo destra basso
     */
    DRect(double x1, double y1, double x2, double y2)
    {
        left = new PointE(x1, y1);
        right = new PointE(x2, y2);
    } //[c] DRect

    /**[c]
     * @see DrawMovable
     */
    public void draw(Graphics g)
    {
        if (selected)
            g.setColor(java.awt.Color.red);
        else
            g.setColor(java.awt.Color.black);
        g.drawRect((int)left.xVal(), (int)left.yVal(), 
          (int)(right.xVal()-left.xVal()), (int)(right.yVal()-left.yVal()));
    } //[m] draw

    /**[c]
     * @see DrawMovable
     */
    public void move(PointE trasl)
    {
        left = new PointE(left.xVal()+trasl.xVal(),
          left.yVal()+trasl.yVal());
        right = new PointE(right.xVal()+trasl.xVal(),
          right.yVal()+trasl.yVal());
    } //[m] move

    /**[c]
     * @see DrawMovable
     */
    public boolean contains(PointE p)
    {
        // appartiene se dentro l'area
        return 
          left.xVal() <= p.xVal() &&
          p.xVal() <= right.xVal() &&
          left.yVal() <= p.yVal() &&
          p.yVal() <= right.yVal();
    } //[m] contains

    /**[m]
     * conversione a stringa
     * @return stringa descrittiva
     */
    public String toString()
    {
        return "[rettangolo x1="+left.xVal()+" y1="+left.yVal()+" x2="+right.xVal()+" y2="+right.yVal();
    } //[m] toString

} //{c} DRect

/**{c}
 * un rombo (4 lati eguali)
 */
class DQuad extends SelFigure implements DrawMovable
{
    int xPts[];  // 4 punti
    int yPts[];

    /**[c]
     * costruttore base
     * @param x  x vertice alto
     * @param y  y vertice alto
     * @param e  lato rombo
     * @param a  angolo alto
     */
    DQuad(double x, double y, double e, double a)
    {
        int dx = (int)(e * Math.sin(a/2));
        int dy = (int)(e * Math.cos(a/2));
        xPts = new int[4];
        yPts = new int[4];
        xPts[0] = (int)x;
        yPts[0] = (int)y;
        xPts[1] = xPts[0] - dx;
        yPts[1] = yPts[0] + dy;
        xPts[2] = xPts[0];
        yPts[2] = yPts[0] + dy*2;
        xPts[3] = xPts[0] + dx;
        yPts[3] = yPts[0] + dy;
    } //[c] DQuad

    /**[c]
     * @see DrawMovable
     */
    public void draw(Graphics g)
    {
        if (selected)
            g.setColor(java.awt.Color.red);
        else
            g.setColor(java.awt.Color.black);
        g.drawPolygon(xPts, yPts, 4);
    } //[m] draw

    /**[c]
     * @see DrawMovable
     */
    public void move(PointE trasl)
    {
        for (int i=0; i<4; xPts[i++] += (int)trasl.xVal());
        for (int i=0; i<4; yPts[i++] += (int)trasl.yVal());
    } //[m] move

    /**[c]
     * @see DrawMovable
     */
    public boolean contains(PointE p)
    {
        // appartiene se dentro l'area
        return new Polygon(xPts, yPts, 4).contains((int)p.xVal(),
          (int)p.yVal());
            // si sfrutta il metodo della classe Polygon di AWT
    } //[m] contains

    /**[m]
     * conversione a stringa
     * @return stringa descrittiva
     */
    public String toString()
    {
        return "[rombo vertice x="+xPts[0]+" y="+yPts[0];
    } //[m] toString

} //{c} DQuad

/**{c}
 * un triangolo
 */
class DTria extends SelFigure implements DrawMovable
{
    int xPts[];  // 3 punti
    int yPts[];

    /**[c]
     * costruttore base
     * @param x1  x primo vertice 
     * @param y1  y primo vertice
     * @param x2  x secondo vertice 
     * @param y2  y secondo vertice
     * @param x3  x terzo vertice 
     * @param y3  y terzo vertice
     */
    DTria(double x1, double y1, double x2, double y2, double x3, double y3)
    {
        xPts = new int[3];
        yPts = new int[4];
        xPts[0] = (int)x1;
        yPts[0] = (int)y1;
        xPts[1] = (int)x2;
        yPts[1] = (int)y2;
        xPts[2] = (int)x3;
        yPts[2] = (int)y3;
    }

    /**[c]
     * @see DrawMovable
     */
    public void draw(Graphics g)
    {
        if (selected)
            g.setColor(java.awt.Color.red);
        else
            g.setColor(java.awt.Color.black);
        g.drawPolygon(xPts, yPts, 3);
    } //[m] draw

    /**[c]
     * @see DrawMovable
     */
    public void move(PointE trasl)
    {
        for (int i=0; i<3; xPts[i++] += (int)trasl.xVal());
        for (int i=0; i<3; yPts[i++] += (int)trasl.yVal());
    } //[m] move

    /**[c]
     * @see DrawMovable
     */
    public boolean contains(PointE p)
    {
        // appartiene se dentro l'area
        return new Polygon(xPts, yPts, 3).contains((int)p.xVal(),
          (int)p.yVal());
            // si sfrutta il metodo della classe Polygon di AWT
    } //[m] contains

    /**[m]
     * conversione a stringa
     * @return stringa descrittiva
     */
    public String toString()
    {
        return "[triangolo vertici x1="+xPts[0]+" y1="+yPts[0]+
          " x2="+xPts[1]+" y2="+yPts[1] + " x3="+xPts[2]+" y1="+yPts[2];
    } //[m] toString

} //{c} DTria

/**{c}
 * area di disegno delle figure
 */
class FigArea extends Canvas
{
    // dimensioni area grafica
    private static final int XSIZE=500;
    private static final int YSIZE=500;

    // tipi di figure
    static final int CIRCTYPE = 1;
    static final int RECTTYPE = 2;
    static final int QUADTYPE = 3;
    static final int TRIATYPE = 4;

    // lista oggetti
    Sequence figs;

    // memoria posizione
    int lastX = 0, lastY = 0;

    /**[c]
     * costruttore di default
     */
    FigArea()
    {
        super();
        figs = new ArraySequence();
            // sequenza di supporto

        /**{c}{l}
         * listener per mouse pressed
         */
        class MoListener extends MouseAdapter
        {
            /**[m]
             * @see MouseListener
             */
            public void mousePressed(MouseEvent e)
            {
                lastX = e.getX();
                lastY = e.getY();
                ObjectIterator en = figs.elements();
                boolean changed = false;  // per repaint
                while(en.hasNext())  // scorre gli oggetti
                {
                    SelFigure fig = (SelFigure)en.nextObject();
                    if (fig.contains(new PointE((double)lastX, (double)lastY)))
                    {
                        // entro l'oggetto, selezionare
                        changed = true;
                        fig.changeSelect();
                    }
                }
                if (! changed)
                {
                    // deseleziona tutto
                    en = figs.elements();
                    while(en.hasNext())  // scorre gli oggetti
                        ((SelFigure)en.nextObject()).deselect();
                }
                FigArea.this.repaint();
            } //[m] mousePressed

        } //{c}{l} MoListener

        /**{c}{l}
         * listener per mouse dragged
         */
        class MoMotionListener extends MouseMotionAdapter
        {
            /**[m]
             * @see MouseMotionListener
             */
            public void mouseDragged(MouseEvent e)
            {
                int x = e.getX();
                int y = e.getY();
                ObjectIterator en = figs.elements();
                boolean changed = false;
                while(en.hasNext())  // scorre gli oggetti
                {
                    SelFigure fig = (SelFigure)en.nextObject();
                    if (fig.selected())
                    {
                        // oggetto selezionato, movimento
                        changed = true;
                        fig.move(new PointE((double)(x-lastX), (double)(y-lastY)));
                    }
                }
                if (changed)
                    FigArea.this.repaint();
                lastX = x;  // aggiorna stato
                lastY = y;
            } //[m] mousePressed

        } //{c}{l} MoMotionListener

        setSize(new Dimension(XSIZE, YSIZE));
        setBackground(java.awt.Color.yellow);

        // gestione del mouse
        MoListener moListen = new MoListener();
        MoMotionListener moMotionListen = new MoMotionListener();
        addMouseListener(moListen);
        addMouseMotionListener(moMotionListen);
    } //[c] FigArea

    /**[m]
     * creazione ed inserimento di un oggetto
     * @param figType  tipo della figura (cerchio, ecc.)
     */
    void insert(int figType)
    {
        double xtmp, ytmp;

        // l'inserimento avviene in coda al sequence
        switch (figType)
        {
        case CIRCTYPE:
            figs.insertLast(new DCirc(Math.random()*XSIZE, Math.random()*YSIZE,
              Math.random()*Math.min(XSIZE, YSIZE)*0.3));
            break;
        case RECTTYPE:
            xtmp = Math.random()*XSIZE;
            ytmp = Math.random()*YSIZE;
            figs.insertLast(new DRect(xtmp, ytmp,
              xtmp+Math.random()*(XSIZE-xtmp), ytmp+Math.random()*(YSIZE-ytmp)));
            break;
        case QUADTYPE:
            figs.insertLast(new DQuad(Math.random()*XSIZE, 
              Math.random()*YSIZE, Math.random()*XSIZE*0.3, 
              Math.random()*Math.PI));
            break;
        case TRIATYPE:
            figs.insertLast(new DTria(Math.random()*XSIZE, Math.random()*YSIZE, 
              Math.random()*XSIZE, Math.random()*YSIZE, 
              Math.random()*XSIZE, Math.random()*YSIZE)); 
            break;
        } //switch
    } //[m] insert

    /**[m]
     * @see Canvas
     */
    public void paint(Graphics g)
    {
        ObjectIterator en = figs.elements();
        while(en.hasNext())
            ((DrawMovable)(en.nextObject())).draw(g);
    } //[m] paint

} //{c} FigArea

/**{c}
 * disegno di una di 4 figure selezionabili
 */
public class DrawFig extends Frame 
implements ActionListener, WindowListener 
{
    // bottoni
    Button circButt = new Button("Cerchio");
    Button rectButt = new Button("Rettangolo");
    Button quadButt = new Button("Rombo");
    Button triaButt = new Button("Triangolo");
    FigArea fa = new FigArea();

    /**[c]
     * costruttore base, aggiunge al frame tutti i
     * componenti previsti
     */
    public DrawFig() {
        super("Esercizio 7.2");

        addWindowListener(this);
        setLayout(new FlowLayout(FlowLayout.LEFT));

        circButt.addActionListener(this);
        rectButt.addActionListener(this);
        quadButt.addActionListener(this);
        triaButt.addActionListener(this);

        add(circButt);
        add(rectButt);
        add(quadButt);
        add(triaButt);

        add(fa);

        setSize(new Dimension(700, 700));
        show();
    } //[c] DrawFig

    /**[m]
     * implementazione di ActionListener
     * @see ActionListener
     */
    public void actionPerformed(ActionEvent e) 
    {
        if (e.getSource() == circButt)
        {
            // cerchio
            fa.insert(FigArea.CIRCTYPE);
            fa.repaint();
        }
        else if (e.getSource() == rectButt)
        {
            // cerchio
            fa.insert(FigArea.RECTTYPE);
            fa.repaint();
        }
        else if (e.getSource() == quadButt)
        {
            // rombo
            fa.insert(FigArea.QUADTYPE);
            fa.repaint();
        }
        else
        {
            // triangolo
            fa.insert(FigArea.TRIATYPE);
            fa.repaint();
        }
    } //[m] actionPerformed
    
    /**[m]
     * implementazione di WindowListener
     * @see WindowListener
     */
    public void windowClosing(WindowEvent e) 
    {
        this.dispose();
        System.exit(0);
    } //[m] windowClosing
    
    public void windowActivated(WindowEvent e) { }
    public void windowDeactivated(WindowEvent e) { }
    public void windowDeiconified(WindowEvent e) { }
    public void windowClosed(WindowEvent e) { }
    public void windowIconified(WindowEvent e) { }
    public void windowOpened(WindowEvent e) { }

    /**[m][s]
     * metodo di attivazione
     * @param args[]  non usato
     */
    public static void main(String[] args) 
    {
        DrawFig me = new DrawFig();
    } //[m][s] main

} //{c} DrawFig
