package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * Interp.java
 * Esercizio 4.2 -
 * Interprete di macchina a stack
 *
 * @version 1.00 99/10/19
 * @version 1.01 99/10/29 -
 * aggiunta del comando commento di linea
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.StackFullException;
import FI2.Exceptions.StackEmptyException;
import FI2.Linear.Stack;
import FI2.Linear.ArrayStack;
import FI2.Util.FI2Sys;
import java.io.*;

/**{c}
 * Macchina algebrica a stack con set di variabili
 */
public class Interp
{
    private static final int NVAR = 100;
      // numero variabili di memoria
    private static final int NULCOM = 0;
    private static final int PUSHI = 1;
    private static final int PUSHV = 2;
    private static final int POPV = 3;
    private static final int ADD = 4;
    private static final int SUB = 5;
    private static final int MUL = 6;
    private static final int DIV = 7;
    private static final int PRINT = 8;
    private static final int PRINTS = 9;
    private static final int EXIT = 10;
      // codifiche comandi

    private DoubleStack opstk;  // stack double degli operandi
    private double mem[];  // variabili di memoria

    /**{c}{l}
     * Adattatore di ArrayStack per double
     */
    private class DoubleStack
    {
        private Stack dstk;  // stack degli operandi

        /**[c]
         * costruttore base
         * @param cap  dimensione massima stack
         */
        public DoubleStack(int cap)
        {
            dstk = new ArrayStack(cap);
        } //[c] DoubleStack

        /**[m]
         * @see Stack
         */
        public int size()
        { return dstk.size(); }
    
        /**[m]
         * @see Stack
         */
        public boolean isEmpty()
        {  return dstk.isEmpty(); }
    
        /**[m]
         * il valore in cima allo stack
         * @return il valore in cima, NaN se vuoto
         * @exception StackEmptyException  se stack vuoto
         */
        public double top()
          throws StackEmptyException
        {
            return ((Double)dstk.top()).doubleValue();
        } //[m] top
    
        /**[m]
         * inserimento di un valore
         * @param val  il valore da inserire
         * @return true se inserito, false altrimenti
         * @exception StackFullException  se stack pieno
         */
        public void push(double val)
          throws StackFullException
        {
           	dstk.push(new Double(val));
        } //[m] push
    
        /**[m]
         * il valore estratto dalla cima dello stack
         * @return il valore estratto, NaN se vuoto
         * @exception StackEmptyException  se stack vuoto
         */
        public double pop()
          throws StackEmptyException
        {
            return ((Double)dstk.pop()).doubleValue();
        } //[m] pop

    } //{c} DoubleStack

    /**[c]
     * costruttore base
     * @param cap  dimensione massima stack
     */
    public Interp(int cap)
    {
        opstk = new DoubleStack(cap);
        mem = new double[NVAR];
    } //[c] Interp

    /**[m]
     * codifica comandi
     * @param com  stringa comando
     * @return codifica comando, NULCOM se illegale
     */
    private int toCommand(String com)
    {
        String coms[] = {
        "", "pushi", "pushv", "popv", 
        "add", "sub", "mul", "div",
        "printv", "prints", "exit"
        };

        if (com == null)
            // stringa nulla
            return NULCOM;
        // ricerca lineare
        for (int i=0; i<coms.length; i++)
        {
            if (com.equals(coms[i]))
                return i;
        }
        // non trovato
        return NULCOM;
    } //[m] toCommand

    /**[m]
     * interprete
     * 
     */
    public void intp(StreamTokenizer in)
      throws IOException
    {
        double val;  // valore top

        while(in.nextToken() != StreamTokenizer.TT_EOF)
        {
            try
            {
                String s = in.sval;
                switch(toCommand(s)) 
                {
                  case PUSHI:
                    in.nextToken();
                    if (in.ttype == StreamTokenizer.TT_NUMBER)
                        opstk.push(in.nval);
                    else
                        FI2Sys.out.println(in.sval + 
                          ": valore mancante, ignorato!! **");
                    break;
                  case PUSHV:
                    in.nextToken();
                    if (in.ttype == StreamTokenizer.TT_NUMBER)
                    {
                        if (in.nval >= NVAR || in.nval < 0.0)
                            FI2Sys.out.println(in.nval + 
                              ": indice illegale, ignorato!! **");
                        else 
                            opstk.push(mem[(int)in.nval]);
                    }
                    else
                        FI2Sys.out.println(in.sval + 
                          ": indice mancante, ignorato!! **");
                    break;
                  case POPV:
                    in.nextToken();
                    if (in.ttype == StreamTokenizer.TT_NUMBER)
                    {
                        if (in.nval >= NVAR || in.nval < 0.0)
                            FI2Sys.out.println(in.nval + 
                              ": indice illegale, ignorato!! **");
                        else 
                        {
                            val = opstk.pop();
                            mem[(int)in.nval] = val;
                        }
                    }
                    else
                        FI2Sys.out.println(in.sval + 
                          ": indice mancante, ignorato!! **");
                    break;
                  case ADD:
                    opstk.push(opstk.pop()+opstk.pop());
                    break;
                  case SUB:
                    val = opstk.pop();
                    opstk.push(opstk.pop()-val);
                    break;
                  case MUL:
                    opstk.push(opstk.pop()*opstk.pop());
                    break;
                  case DIV:
                    val = opstk.pop();
                    opstk.push(opstk.pop()/val);
                    break;
                  case PRINT:
                    in.nextToken();
                    if (in.ttype == StreamTokenizer.TT_NUMBER)
                    {
                        if (in.nval >= NVAR || in.nval < 0.0)
                            FI2Sys.out.println(in.nval + 
                              ": indice illegale, ignorato!! **");
                        else
                            FI2Sys.out.println("var["+((int)in.nval)+
                              "]="+ mem[(int)in.nval]);
                    }
                    else
                        FI2Sys.out.println(in.sval + 
                          ": indice mancante, ignorato!! **");
                    break;
                  case PRINTS:
                    FI2Sys.out.println("top="+ opstk.top());
                    break;
                  case EXIT:
                    return;
                  default:
                    FI2Sys.out.println( 
                      ((in.ttype != StreamTokenizer.TT_NUMBER) ?
                      in.sval : Double.toString(in.nval)) + 
                      ": comando illegale, ignorato!! **");
                } // switch
            } //try
            catch (StackEmptyException e)
            {
                FI2Sys.out.println("** stack vuoto!! **");
            } //catch(EOF)
            catch (StackFullException e)
            {
                FI2Sys.out.println("** stack pieno!! **");
            } //catch(EOF)
            catch (EOFException e)
            {
                break;
                // termina il ciclo
            } //catch(EOF)
        } //while
        
    } //[m] intp

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  nome file (opzionale)
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "Interprete algebrico, battere comandi macchina a stack, uno per riga:");
        }
        else
            in = new FileInputStream(args[0]);

        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        sIn.parseNumbers();
        sIn.lowerCaseMode(true);  // riduce a minuscolo
        sIn.commentChar('#'); // v1.01
        Interp inp = new Interp(10);
        inp.intp(sIn);
    } //[m][s] main

} //{c} Interp

