package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi
 */

/**{f}
 * PointE.java
 * realizzazione di un ADT per un punto sul piano cartesiano
 * variante rispetto a Point per renderla estendibile 

 * @version 2.01 99/09/25
 * @author M.Moro DEI UPD
 */

import FI2.Util.SimpleComparable;

/**{c}
 * Realizzazione di punto cartesiano come due campi double
 * protected -
 * implementa l'interfaccia SimpleComparable definendo il 
 * proprio metodo equals()
 */
public class PointE implements SimpleComparable
{

    public static final int SPECX = 0;
        // speculare asse x
    public static final int SPECY = 1;
        // speculare asse y
    public static final int SPECO = 2;
        // speculare origine

    protected double x;  // ascissa
    protected double y;  // ordinata

    /**[c]
     * costruttore base
     * @param xVal  ascissa
     * @param yVal  ordinata
     */
    public PointE(double xVal, double yVal)
    {
        x = xVal;
        y = yVal;
    } //[c] PointE

    /**[c]
     * costruttore di default
     */
    public PointE()
    {
        this(0.0, 0.0);
    } //[c] PointE

    /**[c]
     * costruttore in coordinate polari
     * @param pol  true se polari, false se cartesiane
     * @param val1  ro (modulo) oppure ascissa
     * @param val2  theta (angolo) oppure ordinata
     */
    public PointE(boolean pol, double val1, double val2)
    {
        if (pol) {
            // polari
            x = val1*Math.cos(val2);
            y = val1*Math.sin(val2);
        }
        else {
             // cartesiane
            x = val1;
            y = val2;
        }
    } //[c] PointE

    /**[m]
     * componente ascissa
     * @return ascissa
     */
    public double xVal()
    {
        return x;
    } //[m] xVal

    /**[m]
     * componente ordinata
     * @return ordinata
     */
    public double yVal()
    {
        return y;
    } //[m] yVal

    /**[m]
     * componente ro (modulo)
     * @return modulo
     */
    public double rVal()
    {
        return Math.sqrt(x*x + y*y);
    } //[m] rVal

    /**[m]
     * componente theta
     * @return angolo theta
     */
    public double tVal()
    {
        return Math.atan2(y, x);
    } //[m] tVal

    /**[m]
     * ridefinisce il metodo Object.equals()
     * @param sndP  secondo punto di confronto
     * @return true se le corrispondenti componenti
     * sono eguali
     */
    public boolean equals(Object sndP)
    {
        return x == ((PointE)sndP).x &&
          y == ((PointE)sndP).y;
    } //[m] equals

    /**[m]
     * punto ruotato di angolo fornito
     * @param angle  angolo di rotazione
     */
    protected void rot(double angle)
    {
        double alfa = tVal()+angle;
        double r = rVal();

        x = r * Math.cos(alfa);
        y = r * Math.sin(alfa);
            // punto ruotato
    } //[m] Rot

    /**[m]
     * punto speculare
     * @param spec  tipo di operazione:
     *        - SPECX speculare rispetto all'asse X         
     *        - SPECY speculare rispetto all'asse Y
     *        - SPECO speculare rispetto all'asse O
     *        - altro, nop
     */
    public void pSpec(int spec)
    {
        switch(spec) {
          case SPECX:
            // speculare rispetto all'asse x
            rot(-2.0 * tVal());
                // 2 volte lo theta in senso orario
            break;
          case SPECY:
            // speculare rispetto all'asse y
            rot(Math.PI - 2.0 * tVal());
                // 2*(PI/2 - theta) in senso antiorario
            break;
          case SPECO:
            // speculare rispetto all'origine
            rot(Math.PI);
                // PI in senso antiorario
            break;
          default:
            // specifica non valida, nop
        }
    } //[m] PSpec

    /**[m]
     * distanza tra due punti
     * @param sndP  secondo punto
     * @return distanza tra this e sndP
     */
    public double dist(PointE sndP)
    {
        double diffx = x - sndP.x;
        double diffy = y - sndP.y;

        return Math.sqrt(diffx*diffx + diffy*diffy);

    } //[m] dist

    /**[m]
     * conversione a stringa
     * @return descrizione del contenuto
     */
    public String toString()
    {
        return "{PointE} x=" + xVal() + " y=" + yVal() +
          " ro=" + rVal() + " theta=" + tVal();
    }

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  true se polari, false se cartesiane 
     * @param args[1]  x o ro
     * @param args[2]  y o theta
     */
    public static void main(String[] args)
    {
        PointE p1;

        if (args.length >= 3)
            // ci sono i parametri di chiamata
            p1 = new
              PointE(Boolean.valueOf(args[0]).booleanValue(),
              Double.valueOf(args[1]).doubleValue(),
              Double.valueOf(args[2]).doubleValue());
        else
        {
            System.out.println("Non ci sono parametri di chiamata!");
            p1 = new PointE();
        }

        System.out.println("Il punto p1 e` x=" + p1.xVal() +
          " y=" + p1.yVal() + " ro=" + p1.rVal() +
          " theta=" + p1.tVal());
        System.out.println("Risultato p1: " + p1);

        PointE p2 = new PointE(p1.xVal(), p1.yVal());
        p2.pSpec(SPECX);
        System.out.println("Risultato SPECX: " + p2);
        p2 = new PointE(p1.xVal(), p1.yVal());
        p2.pSpec(SPECY);
        System.out.println("Risultato SPECY: " + p2);
        p2 = new PointE(p1.xVal(), p1.yVal());
        p2.pSpec(SPECO);
        System.out.println("Risultato SPECO: " + p2);
        System.out.println("Distanza p1, SPECO=" +
      p1.dist(p2));
        System.out.println("p1 " +
          (p1.equals(new PointE(1.0, 1.0)) ?
          "coincide" : "non coincide") +
          " con il punto unita` (1.0, 1.0)");
    } //[m][s] main

} //{c} PointE

