package FI2.Graph;
/**{p}
 * -- FI2.Graph --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Grafi
 */

/**{f}
 * DFS.java
 * visita depth first di un grafo -
 * software ricavato dalla libreria JDSL
 *
 * @version 1.00 98/09/24
 * @author M.Moro DEI UPD
 */

/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * @(#)DFS.java  Fri Aug 29 12:58:13 1997
 * Natasha Gelfand
 */

// package jdsl.core.algo.graphtraversals;

// import jdsl.core.api.*;
// import java.util.Hashtable;
import java.util.Enumeration;
import FI2.Set.ObjectIterator;
import FI2.Dictionary.SeparateChainingHashTable;
import FI2.Util.ObjectEqual;
import FI2.Dictionary.UnOrderedSLList;
import FI2.Dictionary.Dictionary;
//import FI2.Exceptions.InvalidPositionException;
//import FI2.Exceptions.InvalidEdgeException;

/**
    * A template method implementation of a depth first 
    * search. A subclass should override various methods 
    * to add functionality to this traversal.
    */
public abstract class DFS {
    /**
        * The graph being traversed.
        */
  protected InspectableGraph graph;
    /**
        * The result of the traversal.
        */
  protected Object visitResult;
    /**
        * A table of marked vertices.
        */
  protected SeparateChainingHashTable markedVerts = 
    new SeparateChainingHashTable(new ObjectEqual(),
    new UnOrderedSLList(new ObjectEqual()));
    /**
        * A table of marked edges.
        */
  protected SeparateChainingHashTable  markedEdges = 
    new SeparateChainingHashTable (new ObjectEqual(),
    new UnOrderedSLList(new ObjectEqual()));

    /**
        * Runs the depth first search algorithm on a graph.
        *
        * @param g An <code>InspectableGraph</code> on which 
        * to run a depth first search.
        * @param start The <code>Vertex</code> at which to 
        * start the depth first search.
        * @param info any extra information that the subclass 
        * might need.
        */
  public Object execute(InspectableGraph g, Vertex start, 
    Object info) {
    graph = g;
    return null;
  }

    /**
        * Performs a recursive depth-first search starting at 
        * <code>v</code>
        */
  // recursive depth-first search starting at vertex v
  protected Object dfsVisit(Vertex v) {
    initResult();
    startVisit(v);
    mark(v);
    // check all incident edges
    for (ObjectIterator inEdges = graph.incidentEdges(v);
     inEdges.hasNext(); ) {
      Edge nextEdge = (Edge)inEdges.nextObject();
      if (!isMarked(nextEdge)) {
    // found an unexplored edge, explore it
    mark(nextEdge);
    Vertex w = graph.opposite(v, nextEdge);
    if (!isMarked(w)) {
      // w is unexplored, this is a discovery edge
      mark(nextEdge);
      traverseDiscovery(nextEdge, v);
      if (!isDone()) {
        visitResult = dfsVisit(w);
      }
    } else {
      // w is explored, this is a back edge
      traverseBack(nextEdge, v);
    }
      }
    }
    finishVisit(v);
    return result();
  }

    /**
        * Called when a vertex needs to be marked as visited.
        */
  protected void mark(Vertex v) {
    markedVerts.insertItem(v, v); 
      // use the vertex as both key and value
  }

    /**
        * Marks an edge as traversed.
        */
  protected void mark(Edge e) {
    markedEdges.insertItem(e, e);
  }

    /**
        * Tests if a vertex has been marked.
        */
  protected boolean isMarked(Vertex v) {
    return markedVerts.findElement(v) != 
      Dictionary.NO_SUCH_KEY;
  }

    /**
        * Tests if an edge has been marked.
        */
  protected boolean isMarked(Edge e) {
    return markedEdges.findElement(e) !=
      Dictionary.NO_SUCH_KEY;
  }

    /**
        * Initializes the result.  Should be overridden by 
        * a subclass.
        */
  // These methods will be defined in the subclasses of 
  // this class
  protected void initResult() {}
    /**
        * Called when a vertex is visited.  Should be 
        * overridden by a subclass.
        */
  protected void startVisit(Vertex v) {}
    /**
        * Called when a discovery edge is traversed.  
        * Should be overridden by a subclass.
        */
  protected void traverseDiscovery(Edge e, Vertex from) {}
    /**
        * Called when a back edge is traversed.  
        * Should be overridden by a subclass.
        */
  protected void traverseBack(Edge e, Vertex from) {}
    /**
        * Tests if the depth first search is done.  
        * Should be overridden by a subclass.
        */
  protected boolean isDone() { return false; }
    /**
        * Called when the search has finished with the vertex.
        * Should be overridden by a subclass.
        */
  protected void finishVisit(Vertex v) {}
    /**
        * Returns the result of the search.
        * Should be overridden by a subclass.
        */
  protected Object result() { return new Object(); }
} //{c}{a} DFS

