package FI2.Graph;
/**{p}
 * -- FI2.Graph --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Grafi
 */

/**{f}
 * SILGraph.java
 * grafo rappresentato come lista di incidenza
 * in forma semplificata -
 * software ricavato dalla libreria JDSL
 *
 * @version 2.00 00/09/28
 * @author M.Moro DEI UPD
 */

/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

// package jdsl.core.ref;

import FI2.Set.Position;
import FI2.Set.ObjectIterator;
import FI2.Set.PositionIterator;
import FI2.Set.Set;
import FI2.Set.ArraySet;
import FI2.Linear.Sequence;
import FI2.Linear.ArraySequence;
import FI2.Set.Container;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.InvalidEdgeException;
import FI2.Util.VectorIter;



/**
 * This is the teach simplification/modification 
 * of the advanced Simple
 * Incidence List Graph.
 *
 * @author Benoit Hudson (bh)
 * @author Ryan Shaun Baker (rsb)
 * @author Mike Boilen (mgb)
 * @version JSDL-Teach release 1.0
 */
public class SILGraph implements Graph {

    //sequence of vertices
    private Sequence verts_ ;
    //sequence of edges
    private Sequence edges_ ;

    /**
     * Constructs a new, empty Graph.
     */
    public SILGraph() {
//      verts_ = new FastEnumerationNodeSequence() ;
//      edges_ = new FastEnumerationNodeSequence() ;
        verts_ = new ArraySequence() ;
        edges_ = new ArraySequence() ;
    }

    /**
     * Constructs a new, empty Graph.
     */
    public Container newContainer() {
        return new SILGraph();
    }

    /**
     * Gets the size of this graph.
     *
     * @return the size, 
     * which is the number of edges plus the number of
     * vertices.
     */
    public int size(){
        return verts_.size()+edges_.size();
    }

    /**
     * Tests if this container is empty.
     *
     * @return <code>true</code> 
     * if this container contains no vertices or
     * edges, <code>false</code> otherwise.
     */
    public boolean isEmpty(){
        return verts_.isEmpty() && edges_.isEmpty();
    } 

    /**
     * Gets the elements in this graph.
     *
     * @return The objects associated 
     * with the vertices in this graph.
     */
    public ObjectIterator elements(){
        ObjectIterator pos = positions();
        Set retval=new ArraySet();
        while(pos.hasNext()) {
            retval.insertElement( 
              ((Position)pos.nextObject()).element() );
        }
        return retval.elements() ;
    }

    /** PositionalContainer methods **/

    /**
     * Gets all the positions in this graph.
     *
     * @return All of the edges and vertices in this graph.
     */
    public ObjectIterator positions(){
        ObjectIterator enum = verts_.elements();
        ObjectIterator addIn = edges_.elements();

        Set vec=new ArraySet();
        while(enum.hasNext())
        {
            vec.insertElement(enum.nextObject());
        }

        while(addIn.hasNext())
        {
            vec.insertElement(addIn.nextObject());
        }
        return vec.elements();
    }


    /**
     * Replaces the object associated with a position.
     *
     * @param p Any position in the graph.
     * @param newElement The new element to associate with the
     * <code>Position</code>
     * @return The old object 
     * associated with <code>p</code>
     * @exception InvalidPositionException 
     * if <code>p</code> is not compatible
     * with this container.
     */
    public Object replaceElement (Position p, 
      Object newElement) throws
        InvalidPositionException{
            //note: only checks that the position 
            // is of the appropriate type,
            //not that it belongs to this container.
            SILPosition pos ;
            if(p instanceof Edge) {
                pos = castToSILEdge(p);
            }
            else if(p instanceof Vertex) {
                pos = castToSILVertex(p);
            }
            else {  
                throw new InvalidPositionException
    ("Told to replace neither an edge nor a vertex");
            }

            Object retval = pos.element();
            pos.setElement(newElement);
            return retval ;
        }


    /**
        * Swaps the elements of two positions. 
        * The <code>Positions</code> stay in place.
        *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
        */
    public void swapElements (Position a, Position b) throws
        InvalidPositionException{
            Object aobj = replaceElement(a, b.element()); 
              // will throw IPEs
            replaceElement(b, aobj);
        }

    /**
     * Gets the number of vertices in this graph.
     */
    public int numVertices() {
        return verts_.size() ;
    }

    /**
     * Gets the number of edges in this graph.
     */
    public int numEdges() {
        return edges_.size() ;
    }

    /**
     * Gets all the vertices in this graph.
     */
    public ObjectIterator vertices() {
        return verts_.elements();
    }

    /**
     * Gets all the edges in this graph.
     */
    public ObjectIterator edges() {
        return edges_.elements();
    }

    /**
     * Gets all the directedEdges in this graph.
     */
    public ObjectIterator directedEdges() {
        Set toReturn=new ArraySet();
        ObjectIterator edges = edges();
        while(edges.hasNext()) {
            Edge e = (Edge)edges.nextObject();
            if(isDirected(e)) {
                toReturn.insertElement(e);
            }
        }
        return toReturn.elements();
    }

    /**
     * Gets all the undirected edges in this graph.
     */
    public ObjectIterator undirectedEdges() {
        Set toReturn = new ArraySet();
        ObjectIterator edges = edges();
        while(edges.hasNext()) {
            Edge e = (Edge)edges.nextObject();
            if(!isDirected(e)) {
                toReturn.insertElement(e);
            }
        }
        return toReturn.elements();
    }

    /**
     * Gets the number of incident edges of a vertex.
     */
    public int degree (Vertex v) {
        SILVertex SILv = castToSILVertex(v) ;
        return SILv.degree();
    }

    /**
     * Gets the number of incoming incident edges of a vertex.
     */
    public int inDegree (Vertex v) {
        ObjectIterator enum = inIncidentEdges(v);
        int retval = 0;
        while(enum.hasNext()) {
            retval++; enum.nextObject();
        }
        return retval ;
    }

    /**
     * Gets the number of outgoing incident edges of a vertex.
     */
    public int outDegree (Vertex v) {
        ObjectIterator enum = outIncidentEdges(v);
        int retval = 0;
        while(enum.hasNext()) {
            retval++; enum.nextObject();
        }
        return retval ;
    }

    /**
     * Gets the adjacent vertices to a particular vertex.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public ObjectIterator adjacentVertices (Vertex v)
        throws InvalidPositionException {
            ObjectIterator enum = incidentEdges (v);
            return adjacentVertices(v, enum);
        }

    /**
     * Gets all the vertices that are the source 
     * of an incoming incident edge
     * to a vertex.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public ObjectIterator inAdjacentVertices (Vertex v)
        throws InvalidPositionException {
            ObjectIterator enum = inIncidentEdges (v);
            return adjacentVertices(v, enum);
        }

    /**
     * Gets all the vertices that are the destination 
     * of an outgoing incident
     * edge to a vertex.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public ObjectIterator outAdjacentVertices (Vertex v)
        throws InvalidPositionException {
            ObjectIterator enum = outIncidentEdges (v);
            return adjacentVertices(v, enum);
        }

    /**
        * Finds all adjacent vertices to a vertex.
        *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
        */
    private ObjectIterator adjacentVertices(Vertex v, 
      ObjectIterator edgelist)
        throws InvalidPositionException {
            Set toReturn = new ArraySet();
            Edge curr;
            while (edgelist.hasNext()) {
                curr = castToSILEdge(edgelist.nextObject());
                toReturn.insertElement (opposite (v,curr) );
            }
            return toReturn.elements();
        }

    /**
     * Returns all edges incident to a particular vertex.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public ObjectIterator incidentEdges (Vertex v)
        throws InvalidPositionException {
            return castToSILVertex(v).incidentEdges();
        }

    /**
     * Returns all edges incident to a particular vertex.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public ObjectIterator inIncidentEdges (Vertex v)
        throws InvalidPositionException {
            ObjectIterator inc = incidentEdges(v);
            Set toReturn = new ArraySet();
            while(inc.hasNext()) {
                Edge e = (Edge)inc.nextObject();
                if(isDirected(e) && (destination(e)==v)) {
                    toReturn.insertElement(e);
                }
            }
            return toReturn.elements();
        }

    /**
     * Gets all edges incident from a particular vertex.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public ObjectIterator outIncidentEdges (Vertex v)
        throws InvalidPositionException {
            ObjectIterator inc = incidentEdges(v);
            VectorIter toReturn = new VectorIter();
            while(inc.hasNext()) {
                Edge e = (Edge)inc.nextObject();
                if(isDirected(e) && (origin(e)==v)) {
                    toReturn.insertLast(e);
                }
            }
            return toReturn;
        }

    /**
     * Gets the endpoints of an edge.
     *
     * @exception InvalidPositionException 
     * if the edge is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Vertex[] endVertices (Edge e)
        throws InvalidPositionException {
            SILEdge SILe = castToSILEdge(e) ;
            return SILe.endpoints();
        }

    /**
     * Gets the end vertex of <code>Edge e</code> that 
     * is different from <code>Vertex v</code>. 
     * If <code>Edge e</code> is a self loop (both
     * oritin and destination are the same vertex), 
     * then <code>v will be returned.
     * @param e Any edge in the graph.
     * @param v A vertex of <code>e</code>
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Vertex opposite (Vertex v, Edge e) 
      throws InvalidEdgeException,
    InvalidPositionException {
        SILEdge SILe = castToSILEdge(e);
        return SILe.opposite(v);
    }

    /**
     * Gets the origin of an edge.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Vertex origin (Edge e) throws InvalidEdgeException,
    InvalidPositionException {
        return castToSILEdge(e).origin();
    }

    /**
     * Gets the destination of an edge.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Vertex destination (Edge e) 
      throws InvalidEdgeException,
    InvalidPositionException {
        return castToSILEdge(e).destination();
    }

    /**
     * Tests if an edge is directed.
     *
     * @return <code>true</code> 
     * if the edge is directed, <code>false</code>
     * otherwise.
     */
    public boolean isDirected (Edge e) 
      throws InvalidEdgeException {
        return castToSILEdge(e).isDirected();
    }

    /**
     * Inserts a vertex into this graph.
     *
     * @param info 
     * Any <code>Object</code> to be associated 
     * with this vertex.
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Vertex insertVertex (Object info) 
      throws InvalidPositionException {
        SILVertex vert = new SILVertex(this,info);
        vert.setPosition(verts_.insertLast((Object)vert));
        return vert ;
    }

    /**
     * Inserts an edge into this graph.
     *
     * @param v1 An endpoint of the edge.
     * @param v2 An endpoint of the edge.
     * @param elt Any <code>Object</code> 
     * to be associated with this edge.
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Edge insertEdge (Vertex v1, Vertex v2, Object elt)
        throws InvalidPositionException  {
            return insertEdge(v1,v2,elt,false);
        }

    /**
     * Inserts a directed edge into this graph.
     *
     * @param v1 The source of edge
     * @param v2 The destination of the ege
     * @param elt Any <code>Object</code> 
     * to be associated with this edge.
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Edge insertDirectedEdge (Vertex v1, 
      Vertex v2, Object elt)
        throws InvalidPositionException  {
            return insertEdge(v1,v2,elt,true);
        }

    /**
     * Inserts an edge into this graph.
     *
     * @param v1 An endpoint of the edge.
     * @param v2 An endpoint of the edge.
     * @param elt Any object 
     * to be associated with this edge.
     * @param directed <code>true</code> 
     * if the edge is directed,
     * <code>false</code> otherwise.
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    private Edge insertEdge(Vertex v1, Vertex v2, 
      Object elt, boolean directed)
        throws InvalidPositionException  {
            SILVertex vert1 = castToSILVertex(v1);
            SILVertex vert2 = castToSILVertex(v2);
            SILEdge edge = new SILEdge(this, elt,
                    v1, v2, directed);
            vert1.addEdge(edge) ;
            vert2.addEdge(edge) ;
            edge.setPosition(this, 
              edges_.insertLast((Object)edge));
            return edge ;
        }

    /**
     * Removes a vertex from this graph.  
     * Removes all edges incident on and
     * incident from <code>v</code>.
     *
     * @param v The vertex to remove.
     * @return The <code>Object</code> 
     * associated with <code>v</code>.
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */

    public Object removeVertex (Vertex v) 
      throws InvalidPositionException{
        ObjectIterator inc = incidentEdges(v) ;
        while(inc.hasNext()) {
            removeEdge( (Edge) inc.nextObject() );
        }
        verts_.remove(castToSILVertex(v).getPosition());
        Object toRet = v.element();
        castToSILVertex(v).invalidate();
        return toRet;
    }

    /**
     * Removes an edge from this graph.
     *
     * @param e The edge to remove.
     * @return the <code>Object</code> 
     * associated with the edge.
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public Object removeEdge (Edge e) 
      throws InvalidPositionException{
        SILEdge edge = castToSILEdge(e);

        // from the endpoints' edge lists
        Vertex[] verts = edge.endpoints() ;
        castToSILVertex(verts[0]) . removeEdge(edge);
        castToSILVertex(verts[1]) . removeEdge(edge);

        // from the graph
        edges_.remove(edge.getPosition(this));
        Object toRet = e.element();
        castToSILEdge(e).invalidate();
        return toRet;
    }


    /**
     * Sets the direction of an edge away from a vertex.
     * 
     * @param e The edge to change.
     * @param newOrigin a vertex of <code>e</code>, 
     * now the new origion of <code>e</code>
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public void setDirectionFrom (Edge e, Vertex newOrigin) 
      throws
        InvalidEdgeException, InvalidPositionException {
            castToSILEdge(e).setDirectionFrom(newOrigin);
        }

    /**
     * Sets the direction of an edge to a vertex.
     *
     * @param e The edge to change.
     * @param newDestination a vertex 
     * of <code>e</code>, now the new
     * destination of <code>e</code>
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     */
    public void setDirectionTo (Edge e, Vertex newDestination) 
      throws
        InvalidEdgeException, InvalidPositionException {
            castToSILEdge(e).setDirectionTo(newDestination);
        }


    /**
     * Makes a directed edge undirected.
     */
    public void makeUndirected (Edge e) 
      throws InvalidEdgeException {
        castToSILEdge(e).setDirected(false);
    }


    /**
     * Reverses the direction of an edge.
     *
     * @exception InvalidPositionException 
     * if the vertex is invalid or
     * incompatible with this <code>Graph</code>
     * @exception InvalidEdgeException if e is not directed
     */
    public void reverseDirection (Edge e) 
      throws InvalidEdgeException,
    InvalidPositionException {
        if(isDirected(e)) castToSILEdge(e).swapPoints();
        else throw new 
          InvalidEdgeException(
          "told to reverse undirected edge");
    }

    /** Casting methods **/


    /**
     * Casts an <code>Object</code> to a vertex.
     */
    private SILVertex castToSILVertex(Object v) {
        if(v==null) 
            throw new InvalidPositionException("Null vertex");
        try
        {
            return (SILVertex)v ;
        }
        catch(ClassCastException e)
        {
            throw new InvalidPositionException(
              "Invalid vertex class");
        }
    }

    /**
     * Casts an object to an edge.
     */
    private SILEdge castToSILEdge(Object e) {
        if(e==null) throw new InvalidPositionException(
          "Null edge");
        try
        {
            return (SILEdge)e ;    

        }
        catch(ClassCastException fe)
        {
            throw new InvalidPositionException(
              "Invalid edge class");
        }
    }
} //{c} SILGraph
