package FI2.Graph;
/**{p}
 * -- FI2.Graph --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Grafi
 */

/**{f}
 * SILVertex.java
 * vertice per SILGraph -
 *
 * @version 2.00 00/09/28
 * @author M.Moro DEI UPD
 * @see SILGraph
 */

/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

// package jdsl.core.ref;

import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Set.ObjectIterator;
import FI2.Linear.Sequence;
import FI2.Linear.ArraySequence;

/**
    * A vertex in the <code>SILGraph</code>.
    *
 * @author Benoit Hudson (bh)
 * @author Mike Boilen (mgb)
    * @version JSDL-Teach release 1.0
    */
public class SILVertex extends SILPosition 
  implements Vertex  {
    private Position pos_ ;
    private Sequence edges_ ;
    
    /** 
     * Create a new isolated vertex for use in a SILGraph.
     * 
     * @param cont  The graph
     * @param elt   The element associated with this position
     */
    SILVertex(Container cont, Object elt) {
    super(cont, elt);
//  edges_ = new NodeSequence() ;
    edges_ = new ArraySequence() ;
    }


    /** 
     * Set the position of the vertex within 
     * the graph's vertex list.
     * 
     * @param pos
     */
    void setPosition(Position pos) { pos_ = pos ; }


    /** 
     * Get the position of this vertex 
     * within the graph's vertex list
     * in constant time.
     * 
     * @return  The position
     */
    Position getPosition() { return pos_ ; }


    /** 
     * Get the degree of this vertex 
     * (the total number of edges)
     * in constant time.
     * 
     * @return  The degree (in, out, and undirected)
     */
    int degree() { return edges_.size(); }


    /** 
     * Get an enumeration of all edges 
     * incident to this vertex.
     * 
     * @return  All incident edges (in, out, and undirected)
     */
    ObjectIterator incidentEdges() { return edges_.elements(); }


    /** 
     * Add an edge to the incidence list, 
     * and notify that edge of its
     * position within the incidence list.
     * 
     * @param e  The edge to add
     */
    void addEdge(SILEdge e) {
    e.setPosition(this, edges_.insertLast((Object)e));
    }

    /** 
     * Remove an edge from the incidence list, 
     * and notify that edge of
     * the removal.
     * 
     * @param e  The edge to remove
     */
    void removeEdge(SILEdge e) {
    edges_.remove(e.getPosition(this));
    e.removePosition(this);
    }

} //{c} SILVertex
