package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArrayList.java
 * Implementazione con vettore di un List
 * (sequenza con posizioni) -
 * derivata da ArrayPositionalSequence della v 1.x 
 *
 * @version 2.00 00/09/20
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import FI2.Set.ObjectIterator;
import FI2.Set.PositionIterator;
import FI2.Set.InspectableContainer;
import FI2.Set.PositionalContainer;
import FI2.Set.InspectablePositionalContainer;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Exceptions.EmptyContainerException;
import FI2.Util.FI2Sys;
import FI2.Set.Position;
import FI2.Set.Container;

/**{c}
 * sequenza generica con posizioni
 * realizzata come vettore a dimensione fissa -
 * si puo` pensare di adottare l'approccio
 * di un adattatore su ArrayVector 
 * essendo i metodi molto simili -
 * risulta concettualmente errato difinire
 * questa classe come estensione di ArrayVector
 * poiche' la specializzazione e` in realta` sugli
 * oggetti che vengono inseriti come elementi (ovvero
 * dei ArrayListPosition) della Sequenza e non 
 * sulla struttura e/o metodi di questa; inoltre 
 * successivamente ArrayList verra` esteso 
 * come ArraySequence e quest'ultima classe dovra` definire i 
 * metodi propri di un Vector inserendo pero` elementi 
 * di tipo ArrayListPosition, e cio` produrrebbe un conflitto
 * (di ridefinizione) con i metodi della classe 
 * ArrayVector di base
 * @see List
 */
public class ArrayList 
  implements List
{
    protected Vector S;

    /**[c]
     * costruttore di default
     */
    public ArrayList()
    { S = new ArrayVector(); }

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua un inserimento nell'ordine degli elementi -
     * gli oggetti non vengono clonati 
     * @param val  array da copiare
     */
    public ArrayList(Object val[])
    { S = new ArrayVector(val); }

    /**[m]
     * @see InspectableContainer
     */
    public int size()
    {
        return S.size();
    } //[m] size

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty()
    {
        return S.size() == 0;
    } //[m] isEmpty

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore
         * interno per gli elementi -
         * sfrutta quello delle posizioni
         */
        class Iter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore di posizioni

            /**[c]
             * costruttore di default
             */
            Iter()
            {
                posIt = positions();
            } //[c] Iter

             /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public boolean hasNext()
            {
                return posIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public Object nextObject()
            {
                return ((ArrayListPosition)posIt.nextPosition())
                  .getElement();
            } //[m] nextObject

        } //{c} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new ArrayList();
    } //[m] newContainer

    /**[m]
     * @see InspectablePositionalContainer
     */
    public PositionIterator positions()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore
         * interno per le posizioni
         */
        class PosIter implements PositionIterator
        {
            private ObjectIterator it;
              // iteratore del vettore di supporto

            /**[c]
             * costruttore di default
             */
            PosIter()
            {
                it = S.elements();
            } //[c] PosIter

             /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public boolean hasNext()
            {
                return it.hasNext();
            } //[m] hasNext

            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public Object nextObject()
            {
                return it.nextObject();
            } //[m] nextObject

            /**[m]
             * @see FI2.Set.PositionIterator
             */
            public Position nextPosition()
            {
                return (Position)it.nextObject();
            } //[m] nextPosition

        } //{c} PosIter

        return new PosIter();
    } //[m] positions

    /**[m]
     * verifica la validita` del posizionatore -
     * O(1)
     * @param p  posizionatore da verificare
     * @return il posizionatore come ArrayListPosition 
     * se legale
     * @exception InvalidPositionException  
     * posizionatore illegale
     */
    protected ArrayListPosition checkPosition(Position p)
      throws InvalidPositionException
    {
        // p.container() verifica se il posizionatore
        // e` legale
        if (p.container() != this)
            throw new InvalidPositionException(
              "Posizionatore riferito ad altro contenitore");
        try
        {
            ArrayListPosition temp = (ArrayListPosition) p;
            return temp;
        }
        catch (ClassCastException e)
        {
            // il posizionatore non e` di tipo 
            // ArrayListPosition
            throw new InvalidPositionException(
"Posizionatore di tipo non corretto per questo contenitore");
        }
    } //[m] checkPosition

    /**[m]
     * O(1)
     * @see PositionalContainer
     */
    public Object replaceElement(Position p, Object newElem)
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        Object oldElem = here.getElement();
        here.setElement(newElem);
        return oldElem;
    } //[m] replaceElement

    /**[m]
     * O(1)
     * @see PositionalContainer
     */
    public void swapElements(Position p, Position q)
      throws InvalidPositionException
    {
        ArrayListPosition first = checkPosition(p);
        ArrayListPosition second = checkPosition(q);
        Object oldFirstElem = first.getElement();
        first.setElement(second.getElement());
        second.setElement(oldFirstElem);
    } //[m] swapElements

    /**[m] !!
     * O(1)
     * scambia le posizioni all'interno della sequenza -
     * differisce da swap perche' le posizioni assumono
     * rank differenti mentre rimane inalterato l'elemento
     * associato
     * @param p  prima posizione
     * @param q  seconda posizione
     * @return l'elemento associato alla prima posizione
     */
    public Object swapPos(Position p, Position q)
      throws InvalidPositionException
    {
        ArrayListPosition first = checkPosition(p);
        ArrayListPosition second = checkPosition(q);
        int pIdx, qIdx;
        S.replaceAtRank(pIdx=first.index(), second);
        S.replaceAtRank(qIdx=second.index(), first);
        first.setIndex(qIdx);
        second.setIndex(pIdx);
        return first.getElement();
    } //[m] swapPos

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position first() throws EmptyContainerException
    {
        if (isEmpty())
            throw new EmptyContainerException(
              "In ArrayList.first");
        return (Position)S.elemAtRank(0);
    } //[m] first

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position last() throws EmptyContainerException
    {
        if (isEmpty())
            throw new EmptyContainerException(
              "In ArrayList.last");
        return (Position)S.elemAtRank(S.size()-1);
    } //[m] last

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public boolean isFirst(Position p) 
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        return (here == (ArrayListPosition)S.elemAtRank(0));
    } //[m] isFirst

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public boolean isLast(Position p) 
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        return (here == 
          (ArrayListPosition)S.elemAtRank(S.size()-1));
    } //[m] isLast

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position before(Position p) 
      throws InvalidPositionException, 
        BoundaryViolationException
    {
        ArrayListPosition here = checkPosition(p);
        if (here == (ArrayListPosition)S.elemAtRank(0))
            throw new BoundaryViolationException(
              "In ArrayList.before: " + 
              "posizionatore in prima posizione");
        return (Position)S.elemAtRank(here.index()-1); 
    } //[m] before

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position after(Position p) 
      throws InvalidPositionException, 
        BoundaryViolationException
    {
        ArrayListPosition here = checkPosition(p);
        if (here == 
          (ArrayListPosition)S.elemAtRank(S.size()-1))
            throw new BoundaryViolationException(
              "In ArrayList.after: " + 
              "posizionatore in ultima posizione");
        return (Position)S.elemAtRank(here.index()+1); 
    } //[m] after

    /**[m]
     * O(n)
     * @see List
     */
    public Position insertFirst(Object e)
    {
        return insertAtRank(0, e);
    } //[m] insertFirst

    /**[m]
     * O(n)
     * @see List
     */
    public Position insertLast(Object e)
    {
        return insertAtRank(S.size(), e);
    } //[m] insertLast

    /**[m]
     * O(n)
     * @see List
     */
    public Position insertBefore(Position p, Object e)
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        return insertAtRank(here.index(), e);
    } //[m] insertBefore

    /**[m]
     * O(n)
     * @see List
     */
    public Position insertAfter(Position p, Object e)
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        return insertAtRank(here.index()+1, e);
    } //[m] insertAfter

    /**[m]
     * O(n)
     * @see List
     */
    public Object remove(Position p)
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        return removeInside(here);
    } //[m] remove

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeBefore(Position p)
      throws InvalidPositionException, 
        BoundaryViolationException
    { return removeInside((ArrayListPosition)before(p)); } 

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeAfter(Position p)
      throws InvalidPositionException, 
        BoundaryViolationException
    { return removeInside((ArrayListPosition)after(p)); } 

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeFirst() throws EmptyContainerException
    { return removeInside((ArrayListPosition)first()); } 

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeLast() throws EmptyContainerException
    { return removeInside((ArrayListPosition)last()); } 

    /**[m]
     * inserisce un nuovo l'elemento all'indice fornito
     * aggiornando successive posizioni -
     * r e` atteso corretto -
     * O(n)
     * @param r  indice di inserimento
     * @param e  nuovo oggetto da inserire
     * @return posizione dell'oggetto inserito
     */
    private Position insertAtRank(int r, Object e)
    {
        Position pos  = new ArrayListPosition(this, e, r);
        S.insertAtRank(r, pos);
        // size e` stato incrementato
        for (int i=r+1; i < S.size(); i++)
            ((ArrayListPosition)S.elemAtRank(i)).setIndex(i);
        return pos;
    } //[m] insertAtRank

    /**[m]
     * rimuove l'elemento del posizionatore interno fornito -
     * spostando i successivi indietro di uno e aggiornando 
     * le successive posizioni -
     * il posizionatore e` atteso corretto
     * O(n)
     * @param p  posizione di rimozione
     * @return oggetto rimosso
     */
    private Object removeInside(ArrayListPosition p)
    {
        int pos;
        S.removeAtRank(pos = p.index());
        // size e` stato decrementato
        for (int i=pos; i < S.size(); i++)
            ((ArrayListPosition)S.elemAtRank(i)).setIndex(i);
        Object elem = p.getElement();
        p.setInvalid();
          // invalida il posizionatore
        return elem;
    } //[m] removeInside

    /**[m] !!
     * metodo di collaudo: verifica la congruenza
     * dei posizionatori
     * @return un posizionatore di debug se sequenza corretta; 
     * il primo posizionatore non corretto, altrimenti
     */
    public Position checkSequence()
    {
        for (int i=0; i<S.size(); i++)
        {
            if (((ArrayListPosition)S.elemAtRank(i)).index() != 
              i)
                return (Position)S.elemAtRank(i);
        } //for
        return new 
          GenericPosition(this, "Check della sequenza Ok");
    } //[m] checkSequence

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        ObjectIterator it = elements();
        String str =
          "List su vettore [size=" + size() +
          "]\n";

output:
        for (int j=0; ; j++)
        {
            int pos = 0;
            for (int i=0; i<6; i++)
            {
                // 6 valori per riga
                pos = j*6+i;
                if (!it.hasNext())
                    // finita la lista
                    break output;
                str += pos + ": " +
                  it.nextObject().toString() + " | ";
            } //for(i)
            if (it.hasNext())
                str += "\n";
        } //for(j)
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "ArrayList: " + 
              "Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArrayList myLis = 
          new ArrayList();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myLis.insertLast(next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (IndexOutOfBoundsException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(IndexOutOfBoundsException)

        FI2Sys.out.println("List creato");
        FI2Sys.out.println(myLis);

        FI2Sys.out.println("size=" + myLis.size());
        FI2Sys.out.println("primo elemento = " + 
          myLis.first().element());
        Position fst = myLis.first();
        FI2Sys.out.println("secondo elemento = " + 
          myLis.after(fst).element());
        Position snd = myLis.after(fst);
        FI2Sys.out.println("terzo elemento = " + 
          myLis.after(snd).element());
        FI2Sys.out.println("ultimo elemento = " + 
          myLis.last().element());
        Position thr = myLis.after(snd);

        Object rem = myLis.remove(snd);
        FI2Sys.out.println("secondo elemento rimosso: " + 
          rem);
        myLis.swapElements(fst, thr);
        FI2Sys.out.println("dopo swap 1^<->3^: " + myLis);
        try
        {
            rem = myLis.replaceElement(snd, new Integer(66));
            FI2Sys.out.println("sostituito in 2:" + rem + 
              " con 66");
        }
        catch (InvalidPositionException e)
        {
            FI2Sys.out.println("2^ non c'e` piu`:" + e);
        }
        Position check = myLis.checkSequence();
        FI2Sys.out.println("Check sequence: " + check);
        ((GenericPosition)check).setInvalid();

        ArrayList myLis2 = 
          new ArrayList();
        Position p2 = myLis2.insertFirst("Stringa 1");
        p2 = myLis2.insertAfter(p2, "Stringa 3");
        myLis2.insertBefore(p2, "Stringa 2");
        FI2Sys.out.println("Secondo List: ");
        ObjectIterator it = myLis2.elements();
        try
        {
            while(true)
                FI2Sys.out.println(
                  it.nextObject().toString());
        }
        catch (NoSuchElementException e)
        {
            FI2Sys.out.println("Fine enumerazione:" + e);
        }

        FI2Sys.out.println("[Ora solleva l'eccezione]: ");
        myLis.insertAfter(p2, "Stringa 4");
          // questa solleva l'eccezione 
          // InvalidPositionException
    } //[m][s] main

} //{c} ArrayList
