package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArraySortedSequencePriorityQueue.java
 * Implementazione con sequenza ordinata su array 
 * di coda a priorita` -
 * derivata da ArrSeqSimplePriorityQueue della v 1.x 
 *
 * @version 2.00 00/09/24
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.Item;
import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.InspectableKeyBasedContainer;
import FI2.Set.ObjectIterator;
import FI2.Util.Comparator;
import FI2.Util.IntegerComparator;
import FI2.Util.FI2Sys;
import FI2.Exceptions.InvalidKeyException;
import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.BoundaryViolationException;

/**{c}
 * coda a priorita` realizzata mediante una 
 * sequenza mantenuta ordinata, a sua volta realizzata
 * con Array
 * @see ArraySequence
 */
public class ArraySortedSequencePriorityQueue 
  implements PriorityQueue 
{
    protected Sequence seq;
      // sequenza di supporto
    protected Comparator comp;
      // comparatore

    // metodi di supporto

    /**[m]
     * chiave associata all'Item indicato dal
     * posizionatore fornito
     * @param pos  posizionatore
     * @return la chiave associata all'item indicato
     */
    protected Object key (Position pos) 
    {
        return ((Item)pos.element()).key();
    } //[m] key

    /**[m]
     * oggetto associato all'Item indicato dal
     * posizionatore fornito
     * @param pos  posizionatore
     * @return l'oggetto associato all'item indicato
     */
    protected Object elem (Position pos) 
    {
        return ((Item)pos.element()).element();
    } //[m] elem

    /**[m]
     * oggetto associato all'Item fornito
     * @param kep  item
     * @return l'oggetto associato all'item 
     */
    protected Object elem (Object kep) 
    {
        return ((Item)kep).element();
    } //[m] elem

    // metodi della classe

    /**[c]
     * costruttore base con comparatore per i confronti
     * @param c  il comparatore
     */
    public ArraySortedSequencePriorityQueue (Comparator c) 
    { 
        seq = new ArraySequence();
        comp = c; 
    } //[c] ArraySortedSequencePriorityQueue

    /**[m]
     * @see InspectableContainer
     */
    public int size () { return seq.size(); }

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty () { return seq.isEmpty(); }

    /**[m]
     * ATTENZIONE: ritorna un iteratore per Item e non
     * per gli elementi associati, che sono ottenibili
     * applicando element()
     * @see InspectableContainer
     */
    public ObjectIterator elements() { return seq.elements(); }

    /**[m]
     * @see Container
     */
    public Container newContainer() 
    { return new ArraySortedSequencePriorityQueue(comp); }
    
    /**[m]
     * @see InspectableKeyBasedContainer
     */
    public ObjectIterator keys()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere le chiavi -
         * si appoggia sull'iteratore di sequenza
         */
        class KeysIter implements ObjectIterator
        {
            private ObjectIterator itemIt;
              // iteratore sulla sequenza

            /**[c]
             * costruttore di default
             */
            KeysIter()
            {
                itemIt = elements();
            } //[c] KeysIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return itemIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"ArraySortedSequencePriorityQueue.KeysIter.nextObject(): fine elementi"
                  );
                return ((Item)itemIt.nextObject()).key();
            } //[m] nextObject

        } //{c}{l} KeysIter

        return new KeysIter();
    } //[m] keys

    /**[m]
     * O(1)
     * @see InspectablePriorityQueue
     */
    public Object minElement () 
      throws EmptyContainerException 
    {
        if (seq.isEmpty())
            throw new EmptyContainerException(
            "La coda e` vuota!");
        return elem(seq.first());
    } //[m] minElement

    /**[m]
     * O(1)
     * @see InspectablePriorityQueue
     */
    public Object minKey () 
      throws EmptyContainerException 
    {
        if (seq.isEmpty())
            throw new EmptyContainerException(
            "La coda e` vuota!");
        return key(seq.first());
    } //[m] minKey

    /**[m]
     * O(n)
     * @see PriorityQueue
     */
     public void insertItem (Object k, Object e) 
       throws InvalidKeyException 
    {
        if (!comp.isComparable(k))
          throw new InvalidKeyException(
            "Chiave illegale:" + k);
        else if (seq.isEmpty())
            // sequenza vuota, inserisce per primo
            seq.insertFirst(new Item(k,e));
        else if (comp.isGreaterThan(
          k, key(seq.last())))
            // > dell'ultimo, inserisce come nuovo ultimo
            seq.insertAfter(seq.last(), new Item(k,e));
        else 
        {
            Position curr = seq.first();
            // cerca la posizione corretta
            while (comp.isGreaterThan(k, key(curr)))
                curr = seq.after(curr);
            seq.insertBefore(curr, new Item(k,e));
        } //else
    } //[m] insertItem

    /**[m]
     * O(1)
     * @see PriorityQueue
     */
    public Object removeMin () 
      throws EmptyContainerException 
    {
        if (seq.isEmpty())
            throw new EmptyContainerException(
            "La coda e` vuota!");
      return elem(seq.remove(seq.first()));
    } //[m] removeMin

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return "ArraySortedSequencePriorityQueue [size=" + size() +
          "]:\n" + seq.toString();
    } //[m] toString()

    /**[m][s] !!
     * ordinamento di una sequenza di elementi-chiave
     * mediante coda a priorita` -
     * gli elementi della coda debbono essere anche
     * interpetabili come chiavi ordinabili
     * usando il comparatore fornito
     * @param s  sequenza da ordinare
     * @param c  comparatore per i confronti
     */
    public static void sort(Sequence s, Comparator c)
    {
        Object e;
        PriorityQueue pq = 
          new ArraySortedSequencePriorityQueue(c);

        while(!s.isEmpty())
        {
            e = s.removeFirst();
            pq.insertItem(e, e);
        } //while

        while(!pq.isEmpty())
        {
            e = pq.removeMin();
            s.insertLast(e);
        } //while
    } //[m][s] sort

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        Comparator comp = new IntegerComparator();

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "ArraySortedSequencePriorityQueue: Battere almeno " +
              "6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArraySortedSequencePriorityQueue myASSPQ = 
          new ArraySortedSequencePriorityQueue(comp);
        Sequence mySeq = new ArraySequence();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;
            String str = "";

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    mySeq.insertLast(next);
                    str = "Valore (" + next.toString() + ")";
                    myASSPQ.insertItem(next, str);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (BoundaryViolationException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(BoundaryViolationException)

        FI2Sys.out.println("Priority Queue creato");
        FI2Sys.out.println(myASSPQ);

        FI2Sys.out.println("Lista chiavi");
        ObjectIterator keyIt = myASSPQ.keys();
        while (keyIt.hasNext())
            FI2Sys.out.println(keyIt.nextObject());
        
        FI2Sys.out.println("size=" + myASSPQ.size());
        while (true)
        {
            try
            {
                FI2Sys.out.println("chiave minima: " +
                  ((Integer)myASSPQ.minKey()).intValue());
                FI2Sys.out.println(
                  "oggetto con chiave minima: " +
                  myASSPQ.minElement());
                FI2Sys.out.println("oggetto rimosso: " +
                  myASSPQ.removeMin());
            }
            catch (EmptyContainerException e)
            {
                FI2Sys.out.println("fine elementi ->" + e);
                break;
            }
        } //while

        FI2Sys.out.println("Sequenza non ordinata");
        FI2Sys.out.println(mySeq);
        ArraySortedSequencePriorityQueue.sort(mySeq, comp);
        FI2Sys.out.println("Sequenza ordinata");
        FI2Sys.out.println(mySeq);
    } //[m][s] main

} //{c} ArraySortedSequencePriorityQueue
