package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArraySortedSequencePriorityQueue2.java
 * Implementazione con sequenza ordinata su array 
 * di coda a priorita` con supporto Locator -
 * derivata da SequenceLocPriorityQueue della v 1.x 
 *
 * @version 2.00 00/09/25
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.Container;
import FI2.Set.LocatorKeyBasedContainer;
import FI2.Set.Position;
import FI2.Set.Locator;
import FI2.Set.ObjectIterator;
import FI2.Set.LocatorIterator;
import FI2.Exceptions.InvalidKeyException;
import FI2.Exceptions.InvalidLocatorException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.UncontainedLocatorException;
import FI2.Util.Comparator;
import FI2.Util.IntegerComparator;
import FI2.Util.FI2Sys;
/*
import java.util.Enumeration;
*/

/**{c}
 * estende ArraySortedSequencePriorityQueue per supportare
 * i riferimenti (Locator)
 * @see ArraySortedSequencePriorityQueue 
 */
public class ArraySortedSequencePriorityQueue2
  extends ArraySortedSequencePriorityQueue 
  implements PriorityQueue2 
{
    // metodi di supporto

    /**[m]
     * inserimento di un Item2 in base alla chiave
     * @param locit  riferimento dell'oggetto da inserire
     * @return il riferimento
     */
    protected Locator locInsert(Item2 locit) 
      throws InvalidKeyException 
    {
        Position p, curr;
        Object k = locit.key();
        if (!comp.isComparable(k))
            // chiave non confrontabile con comp
            throw new InvalidKeyException("Chiave illegale");
        else if (seq.isEmpty())
            // coda vuota
            p = seq.insertFirst(locit);
        else if (comp.isGreaterThan(k,key(seq.last())))
            // e` da inserire in fondo
            p = seq.insertAfter(seq.last(),locit);
        else 
        {
            // cerca la posizione di inserimento
            curr = seq.first();
            while (comp.isGreaterThan(k,key(curr)))
                curr = seq.after(curr);
            p = seq.insertBefore(curr,locit);
        } //else
        locit.setPosition(p);
          // imposta posizionatore
        locit.setContainer(this);
          // imposta contenitore
        return (Locator) locit;
    } //[m] locInsert

    /**[m]
     * rimozione di un Item2
     * @param loc  riferimento dell'oggetto da rimuovere
     * @return il riferimento
     */
    protected Item2 locRemove(Locator loc) {
        checkLocator(loc); // verifica Locator
        seq.remove(((Item2)loc).position());
          // rimozione
        ((Item2) loc).setContainer(null);
          // invalidazione 
        return (Item2) loc;
    } //[m] locRemove

    /**[m]
     * verifica di un Item2
     * @param locit  riferimento dell'oggetto da verifiacre
     * @return il riferimento come Item2
     */
    protected Item2 checkLocator(Object locit)
      throws InvalidLocatorException
    {
        if (locit==null)
          throw new InvalidLocatorException("Locator nullo");
        try
        {
            return (Item2)locit;
        }
        catch(ClassCastException e)
        {
            throw new InvalidLocatorException(
              "Locator non valido in questo contesto");
        }
    } //[m] checkLoc

    // metodi della classe

    /**[c]
     * costruttore base con comparatore per i confronti
     * @param c  il comparatore
     */
    public ArraySortedSequencePriorityQueue2 (Comparator c) 
    { super(c); }

    /**[m]
     * @see Container
     */
    public Container newContainer()
    { return new ArraySortedSequencePriorityQueue2(comp); }

    /**[m]
     * @see InspectablePriorityQueue2
     */
    public Locator min() throws EmptyContainerException
    {
        if (seq.isEmpty())
            throw new EmptyContainerException("Coda vuota!");
        return (Locator)seq.first().element();
    } //[m] min

    /**[m]
     * @see LocatorKeyBasedContainer
     */
    public LocatorIterator locators()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere i riferimenti -
         * si appoggia sull'iteratore di sequenza
         */
        class LocsIter implements LocatorIterator
        {
            private ObjectIterator locIt;
              // iteratore sulla sequenza

            /**[c]
             * costruttore di default
             */
            LocsIter()
            {
                locIt = elements();
            } //[c] LocsIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return locIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return nextLocator();
            }

            /**[m]
             * @see LocatorIterator
             */
            public Locator nextLocator()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"ArraySortedSequencePriorityQueue2.LocsIter.nextLocator(): fine elementi"
                  );
                return (Item2)locIt.nextObject();
            } //[m] nextLocator

        } //{c}{l} LocsIter

        return new LocsIter();
    } //[m] locators

    /**[m]
     * @see LocatorKeyBasedContainer
     */
    public Locator insert(Object k, Object e)
      throws InvalidKeyException
    {
        Item2 locit = new Item2(k, e, null, null);
        return locInsert((Item2) locit);
    } //[m] insert
    
    /**[m]
     * @see LocatorKeyBasedContainer
     */
    public void remove(Locator loc)
      throws InvalidLocatorException, 
      UncontainedLocatorException
    { locRemove(loc); }

    /**[m]
     * @see LocatorKeyBasedContainer
     */
    public Object replaceElement (Locator loc, 
      Object newElement) 
      throws InvalidLocatorException
    {
        checkLocator(loc);
        Object oldElement = ((Item2)loc).element();
        ((Item2)loc).setElement(newElement);
        return oldElement;
    } //[m] replaceElement

    /**[m]
     * @see LocatorKeyBasedContainer
     */
    public Object replaceKey(Locator loc, Object newKey)
      throws InvalidLocatorException,
      InvalidKeyException
    {
        Item2 locit = locRemove(loc);
        Object oldKey = ((Item2)loc).key();
        locit.setKey(newKey);
        locInsert(locit);
        return oldKey;
    } //[m] replaceKey

    /**[m]
     * metodo ridefinito per garantire l'inserimento di un 
     * Item1 invece che un Item
     * @see PriorityQueue
     */
     public void insertItem (Object k, Object e) 
       throws InvalidKeyException 
    { insert(k, e); }

    /**[m]
     * metodo ridefinito per garantire l'invalidazione del 
     * Item2 estratto
     * @see PriorityQueue
     */
     public Object removeMin()
      throws EmptyContainerException
    { 
        Object toReturn = minElement();
        locRemove(min());
        return toReturn;
    } //[m] removeMin
     
    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        Comparator comp = new IntegerComparator();

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "ArraySortedSequencePriorityQueue2: " + 
              "Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArraySortedSequencePriorityQueue2 myASSPQ = 
          new ArraySortedSequencePriorityQueue2(comp);
        Sequence mySeq = new ArraySequence();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;
            String str = "";

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    mySeq.insertLast(next);
                    str = "Valore (" + next.toString() + ")";
                    myASSPQ.insertItem(next, str);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (BoundaryViolationException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(BoundaryViolationException)

        FI2Sys.out.println("Priority Queue creato");
        ObjectIterator it = myASSPQ.elements();
        while (it.hasNext())
            FI2Sys.out.println("---->" + it.nextObject());
        FI2Sys.out.println(myASSPQ);
        FI2Sys.out.println("Lista riferimenti");
        LocatorIterator locIt = myASSPQ.locators();
        while (locIt.hasNext())
            FI2Sys.out.println(locIt.nextLocator());
        FI2Sys.out.println("Lista chiavi");
        ObjectIterator keyIt = myASSPQ.keys();
        while (keyIt.hasNext())
            FI2Sys.out.println(keyIt.nextObject());

        FI2Sys.out.println("size=" + myASSPQ.size());

        Locator loc = myASSPQ.min();
        myASSPQ.replaceElement(loc, "-------- NUOVO -------");
        FI2Sys.out.println(
          "Priority Queue dopo replace di min");
        FI2Sys.out.println(myASSPQ);
        myASSPQ.replaceKey(loc, new Integer(6666));
        FI2Sys.out.println(
          "Priority Queue dopo replace di key");
        FI2Sys.out.println(myASSPQ);
        while (true)
        {
            try
            {
                FI2Sys.out.println("chiave minima: " +
                  ((Integer)myASSPQ.minKey()).intValue());
                FI2Sys.out.println(
                  "oggetto con chiave minima: " +
                  myASSPQ.minElement());
                FI2Sys.out.println("oggetto rimosso: " +
                  myASSPQ.removeMin());
            }
            catch (EmptyContainerException e)
            {
                FI2Sys.out.println("fine elementi ->" + e);
                break;
            }
        } //while

        FI2Sys.out.println("Sequenza non ordinata");
        FI2Sys.out.println(mySeq);
        ArraySortedSequencePriorityQueue2.sort(mySeq, comp);
        FI2Sys.out.println("Sequenza ordinata");
        FI2Sys.out.println(mySeq);
    } //[m][s] main

} //{c} SequenceLocPriorityQueue2
