package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * HeapDistrib.java
 * Calcolo della distribuzione dei nodi in un heap
 *
 * @version 1.00 98/09/22
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.FI2Sys;

/**{c}
 * fornisce la routine heapDistrib che, dato un sottoalbero
 * di n nodi di un heap, fornisce il numero di nodi del
 * sottoalbero di sinistra e di quello di destra della
 * radice
 */
public class HeapDistrib
{
    /**[m][s]
     * calcola la distribuzione dei nodi dei due
     * sottoalberi della radice di in un heap di 
     * n nodi -
     * se 2**k < n+1 <= 2**k + 2**(k-1), allora
     * n(left) = n-1-n(right) = n-2**(k-1)
     * n(right) = 2**(k-1)-1 *
     * last = left --
     * se 2**k + 2**(k-1) < n+1 <= 2**(k+1), allora
     * n(left) = 2**k-1
     * n(right) = n-1-n(left) = n-2**k
     * last = right --
     * la ricerca di k avviene per confronto con 
     * potenze di 2 -
     * O(log n)
     * 
     * @param n  numero nodi albero
     * @return [0] numero nodi sottoalbero sinistro
     *         [1] numero nodi sottoalbero destro
     *         (ret[0] + ret[1] + 1 = n)
     *         [2] 0 se last e` a sinistra
     *         [2] 1 se last e` a destra
     */
    public static int[] eval(int n)
    {
        int toReturn[] = new int[3];
        if (n <= 1)
        {
            toReturn[0] = toReturn[1] = toReturn[2] = 0;
            return toReturn;
        }

        int k, min;
        for (k=1, min=2; k<=31; k++, min <<= 1)
        {
            // min = 2**k
            if (n+1 > (min << 1) )
                // n+1 > 2**(k+1)
                continue;
            if (n+1 > min + (min >> 1))
            {
                // 2**k + 2**(k-1) < n+1 <= 2**(k+1)
                toReturn[0] = min-1;
                toReturn[1] = n-min;
                toReturn[2] = 1;
                return toReturn;
            }
            // 2**k < n+1 <= 2**k + 2**(k-1)
            toReturn[0] = n-(min>>1);
            toReturn[1] = (min>>1)-1;
            toReturn[2] = 0;
            return toReturn;
        } //for
        // fuori range
        return null;
    } //[m][s] eval

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  non usato
     */
    public static void main(String[] args)
    {
        for (int n= 0; n<100; n++)
        {
            int r[] = HeapDistrib.eval(n);
            FI2Sys.out.println("n=" + n + " left=" +
              r[0] + " right=" + r[1] + " last=" +
              (r[2]==0 ? "left" : "right"));
        }
    } //[m][s] main

} //{c} HeapDistrib
