package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * InspectableList.java
 * interfaccia per una sequenza con posizioni
 * read-only -
 * derivata da PositionalSequence della v 1.x 
 *
 * @version 2.00 00/09/24
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Set.Position;
import FI2.Set.InspectablePositionalContainer;

/**{i}
 * generica sequenza con accesso via posizionatore
 * @see InspectablePositionalContainer
 */
public interface InspectableList 
  extends InspectablePositionalContainer
{
    /**[m][a]
     * il posizionatore dell'oggetto in prima posizione
     * @return  posizionatore se esiste l'oggetto 'primo'
     * @exception EmptyContainerException  sequenza vuota
     */
    Position first() throws EmptyContainerException;

    /**[m][a]
     * il posizionatore dell'oggetto in ultima posizione
     * @return  posizionatore se esiste l'oggetto 'ultimo'
     * @exception EmptyContainerException  sequenza vuota
     */
    Position last() throws EmptyContainerException;

    /**[m][a]
     * verifica se il posizionatore corrisponde
     * al primo elemento
     * @param p  posizionatore
     * @return  true se posizionatore in prima posizione
     * @exception InvalidPositionException
     * posizionatore illegale
     */
    boolean isFirst(Position p) 
      throws InvalidPositionException;

    /**[m][a]
     * verifica se il posizionatore corrisponde
     * all'ultimo elemento
     * @param p  posizionatore
     * @return  true se posizionatore in ultima posizione
     * @exception InvalidPositionException
     * posizionatore illegale
     */
    boolean isLast(Position p) 
      throws InvalidPositionException;

    /**[m][a]
     * il posizionatore dell'oggetto precedente quello
     * 'indicato' dal posizionatore fornito
     * @param p  posizionatore
     * @return  posizionatore se esiste l'oggetto precedente
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception BoundaryViolationException
     * posizionatore p in prima posizione
     */
    Position before(Position p) 
      throws InvalidPositionException, 
        BoundaryViolationException;

    /**[m][a]
     * il posizionatore dell'oggetto successivo a quello
     * 'indicato' dal posizionatore fornito
     * @param p  posizionatore
     * @return  posizionatore se esiste l'oggetto successivo
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception BoundaryViolationException
     * posizionatore p in ultima posizione
     */
    Position after(Position p) 
      throws InvalidPositionException, 
        BoundaryViolationException;

    // metodi di aggiornamento

} //{i} InspectableList
