package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * LinkedHeapPriorityQueue.java
 * Implementazione con heap su albero concatenato
 * di coda a priorita` semplificata -
 * derivata da LinkedHeapSPriorityQueue della v 1.x 
 *
 * @version 2.00 00/09/24
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Exceptions.EmptyContainerException;
import FI2.Set.Container;
import FI2.Set.ObjectIterator;
import FI2.Set.Position;
import FI2.Set.Item;
import FI2.Util.Comparator;
import FI2.Util.IntegerComparator;
import FI2.Util.FI2Sys;
import FI2.Tree.LinkedBinaryTree;
import FI2.Tree.LinkedHeapTree;

/**{c}
 * coda a priorita` (non supporto Locator) 
 * realizzata mediante un heap - 
 * lo heap si appoggia su un albero binario
 * realizzato in forma concatenata
 * @see HeapPriorityQueue
 */
public class LinkedHeapPriorityQueue 
  extends HeapPriorityQueue implements PriorityQueue
{
    /**[c]
     * costruttore base, heap vuoto
     * @param c  comparatore per i confronti
     */
    public LinkedHeapPriorityQueue(Comparator c) 
    {
        if ((comp = c) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        T = new LinkedHeapTree();
    } //[c] LinkedHeapPriorityQueue

    /**[c]
     * costruttore da sequenza di elementi-chiave
     * @param s  sequenza
     * @param c  comparatore per i confronti
     */
    public LinkedHeapPriorityQueue(Sequence s, Comparator c) 
    {
        if ((comp = c) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        T = new LinkedHeapTree(); 
        bottomUpHeap(s, T.root(), s.size(), true);
    } //[m] LinkedHeapPriorityQueue
  
      /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new LinkedHeapPriorityQueue(comp);
    } //[m] newContainer

    /**[m][s] !!
     * ordinamento di una sequenza mediante coda
     * a priorita` -
     * gli elementi della coda debbono essere anche
     * interpetabili come chiavi ordinabili secondo
     * il comparatore fornito
     * adotta il metodo di costruzione bottomUpHeap
     * per ottenere lo heap dalla sequenza
     * usando il comparatore fornito
     * @param s  sequenza da ordinare
     * @param c  comparatore per i confronti
     */
    public static void sort(Sequence s, Comparator c)
    {
        Object e;
        LinkedHeapPriorityQueue pq = 
          new LinkedHeapPriorityQueue(s, c);
        while(!pq.isEmpty())
        {
            e = pq.removeMin();
            s.insertLast(e);
        } //while
    } //[m][s] sort

    /**[m] !!
     * costruzione ricorsiva dello heap
     * a partire da una sequenza di elementi
     * @param s  sequenza degli elementi
     * @param pos  posizione di espansione
     * @param num  numero elementi della sottosequenza
     * @param isLast  true se il sottoalbero conterra` 
     *                l'ultimo nodo, false altrimenti
     * @return il sottoalbero costruito
     */
    public void bottomUpHeap(Sequence s, Position pos, 
      int num, boolean isLast)
    {
        if (num == 0)
            return;

        Object e = s.removeFirst();
        int subNum[] = HeapDistrib.eval(num);
        boolean leftLast = (subNum[2] == 0);
          // l'ultimo nodo dello heap 
        Position v = ((LinkedBinaryTree)T)
              .expandInternal(pos, true);
        bottomUpHeap(s, pos, subNum[0], 
          leftLast && isLast);
        bottomUpHeap(s, T.rightChild(v), 
          subNum[1], !leftLast && isLast);
        T.replaceElement(v, new Item(e, e));
        if (num == 1 && isLast)
            // e` l'ultimo nodo dell'albero
             ((LinkedHeapTree)T).setLast(v);

        Position p;
        // down-heap bubbling
        while (T.isInternal(T.leftChild(v))) 
        {
            // r ha un figlio sinistro interno
            if (T.isExternal(T.rightChild(v)) ||
              // r ha un figlio destro esterno
              comp.isLessThanOrEqualTo(
              key(T.leftChild(v)),
              key(T.rightChild(v)))
              // r ha due figli entrambi interni e 
              // il sinistro e` <= del destro
              )
                p = T.leftChild(v);
            else
                p = T.rightChild(v);
            // p e` il figlio minore 

            if (comp.isLessThan(
              key(p), key(v))) 
            {
                // ordine errato, scambio
                T.swapElements(v, p);
                v = p;
            }
            else 
                // l'ordinamento e` ripristinato
                break;
        } // while(isInternal)
    } //[m] bottomUpHeap

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return "LinkedHeapPriorityQueue [size=" + size() +
          "]:\n" + T.toString();
    } //[m] toString()

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        Comparator comp = new IntegerComparator();

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "LinkedHeapPriorityQueue: " + 
              "Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        LinkedHeapPriorityQueue myHSPQ = 
          new LinkedHeapPriorityQueue(comp);
        Sequence mySeq = new ArraySequence();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;
            String str = "";

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    mySeq.insertLast(next);
                    str = "Valore=" + next.toString();
                    myHSPQ.insertItem(next, str);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (BoundaryViolationException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(BoundaryViolationException)

        FI2Sys.out.println("Priority Queue creato");
        FI2Sys.out.println(myHSPQ);

        FI2Sys.out.println("Lista chiavi");
        ObjectIterator keyIt = myHSPQ.keys();
        while (keyIt.hasNext())
            FI2Sys.out.println(keyIt.nextObject());
        
        FI2Sys.out.println("size=" + myHSPQ.size());
        FI2Sys.out.println("T.size=" + myHSPQ.T.size());
        ObjectIterator it = myHSPQ.elements();
        int pos = 0;
        while (it.hasNext())
            FI2Sys.out.println("elemento pos=" + (pos++) +
              " elem: " + it.nextObject());
        while (true)
        {
            try
            {
                FI2Sys.out.println("chiave minima: " +
                  ((Integer)myHSPQ.minKey()).intValue());
                FI2Sys.out.println(
                  "oggetto con chiave minima: " +
                  myHSPQ.minElement());
                FI2Sys.out.println("oggetto rimosso: " +
                  myHSPQ.removeMin());
            }
            catch (EmptyContainerException e)
            {
                FI2Sys.out.println("fine elementi ->" + e);
                break;
            }
        } //while

        FI2Sys.out.println("Sequenza non ordinata");
        FI2Sys.out.println(mySeq);
        LinkedHeapPriorityQueue.sort(mySeq, comp);
        FI2Sys.out.println("Sequenza ordinata");
        FI2Sys.out.println(mySeq);
    } //[m][s] main

} //{c} LinkedHeapPriorityQueue

