package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture Lineari
 */

/**{f}
 * LinkedQueue.java
 * Implementazione con lista concatenata per una coda FIFO
 *
 * @version 1.00 98/0/19
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.QueueEmptyException;
import FI2.Util.FI2Sys;

/**{c}
 * coda FIFO generica realizzata con lista 
 * concatenata semplice
 * @see Queue
 */
public class LinkedQueue implements Queue
{
    protected SLNode head;  // testa, primo elemento
    protected SLNode tail;  // coda, ultimo elemento
    protected int size;  // numero elementi
      // estensibile

    /**[c]
     * costruttore di default
     */
    public LinkedQueue()
    {
        head = tail = null;
        size = 0;
    } //[c] LinkedQueue

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * mantiene l'ordine originario -
     * gli oggetti non vengono clonati - O(n)
     * @param val  array da copiare
     */
    public LinkedQueue(Object val[])
    {
        this();
        head = tail = null;
        for (int i=0; i<val.length; i++)
            enqueue(val[i]);
    } //[c] LinkedQueue

    /**[m]
     * O(1)
     * @see Queue
     */
    public int size()
    {
        return size;
    } //[m] size

    /**[m]
     * O(1)
     * @see Queue
     */
    public boolean isEmpty()
    {
        return size == 0;
    } //[m] isEmpty

    /**[m]
     * O(1)
     * @see Queue
     */
    public Object front()
      throws QueueEmptyException
    {
        if (isEmpty())
            throw new QueueEmptyException(
              "In LinkedQueue.front");
        return head.getElement();
    } //[m] front

    /**[m]
     * O(1)
     * @see Queue
     */
    public void enqueue(Object obj)
    {
        SLNode node = new SLNode();
        node.setElement(obj);
        node.setNext(null);
        if (size++==0)
            head = node;
        else
            tail.setNext(node);
        tail = node;
    } //[m] enqueue

    /**[m]
     * O(1)
     * @see Queue
     */
    public Object dequeue()
      throws QueueEmptyException
    {
        if (size==0)
            throw new QueueEmptyException(
              "In LinkedQueue.dequeue");
        Object obj = head.getElement();
        head = head.getNext();
        if (--size == 0)
            tail = null;
        return obj;
    } //[m] dequeue

    /**[m] !!
     * conversione a stringa - O(n)
     * @return stringa con il contenuto
     */
    public String toString()
    {
        String str =
          "Coda su lista concatenata [size=" + size() +
          "] (dalla testa):\n";
        int count=0;
        SLNode tmp = head;
output:
        while (true)
        {
            do
            {
                if (tmp == null)
                    // finita la lista
                    break output;
                str += count + ": " +
                  tmp.getElement().toString() + " | ";
                tmp=tmp.getNext();
                count++;
            }
            while (count%6 != 0);
            if (tmp != null)
                str += "\n";
        } //while
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "LinkedQueue: Battere almeno 6 interi (fine=Ctrl-Z):");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        LinkedQueue myQueue = new LinkedQueue();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myQueue.enqueue(next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        }
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        // la lista ha ora i numeri forniti nell'ordine

        FI2Sys.out.println("Coda creata");
        FI2Sys.out.println(myQueue);

        FI2Sys.out.println("size=" + myQueue.size());

        myQueue.dequeue();
        myQueue.dequeue();
        FI2Sys.out.println("La coda dopo due estrazioni e`:");
        FI2Sys.out.println("size=" + myQueue.size());
        FI2Sys.out.println(myQueue);

        // prova della circolarita`
        for (int i=0; i<1000; i++)
        {
            myQueue.enqueue(new Integer(i));
            myQueue.dequeue();
        }
        FI2Sys.out.println("Dopo 1000 enq-deq la coda e`:");
        FI2Sys.out.println("size=" + myQueue.size());
        FI2Sys.out.println(myQueue);
    } //[m][s] main

} //{c} LinkedQueue
