package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * LinkedStack.java
 * Implementazione con lista concatenata 
 * semplice di uno stack
 *
 * @version 1.01 98/10/31
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.StackEmptyException;
import FI2.Util.FI2Sys;

/**{c}
 * stack generico realizzato come lista concatenata semplice
 * @see Stack
 */
public class LinkedStack implements Stack
{
    private SLNode top;  // cima dello stack
    private int size;  // numero elementi

    /**[c]
     * costruttore di default
     */
    public LinkedStack()
    {
        top = null;
        size = 0;
    } //[c] LinkedStack

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua un push nell'ordine degli elementi -
     * gli oggetti non vengono clonati 
     * @param val  array da copiare
     */
    public LinkedStack(Object val[])
    {
        this();
        for (int i=0; i<val.length; i++)
            push(val[i]);
    } //[c] LinkedStack

    /**[m]
     * O(1)
     * @see Stack
     */
    public int size()
    {
        return size;
    } //[m] size

    /**[m]
     * O(1)
     * @see Stack
     */
    public boolean isEmpty()
    {
        return size == 0;
        // return top == null;
    } //[m] isEmpty

    /**[m]
     * O(1)
     * @see Stack
     */
    public Object top()
      throws StackEmptyException
    {
        if (isEmpty())
            throw new StackEmptyException(
              "LinkedStack, in top.");
        return top.getElement();
    } //[m] top

    /**[m]
     * O(1)
     * @see Stack
     */
    public void push(Object obj)
    {
        top = new SLNode(obj, top);
        size++;
    } //[m] push

    /**[m]
     * O(1)
     * @see Stack
     */
    public Object pop()
      throws StackEmptyException
    {
        if (isEmpty())
            throw new StackEmptyException(
              "In LinkedStack.pop");
        Object temp = top.getElement();
        top = top.getNext();
        size--;
        return temp;
    } //[m] pop

    /**[m] !!
     * conversione a stringa - O(n)
     * @return stringa con il contenuto
     */
    public String toString()
    {
        if (size == 0)
            // stack vuoto
            return "Stack su lista concatenata vuoto!\r\n";
        String str =
          "Stack su lista concatenata [size=" + size() +
          "] (top all'inizio):\n";
        int count=0;
        SLNode tmp = top;
output:
        while (true)
        {
            do
            {
                if (tmp == null)
                    // finita la lista
                    break output;
                str += count + ": " +
                  tmp.getElement().toString() + " | ";
                tmp=tmp.getNext();
                count++;
            }
            while (count%6 != 0);
            if (tmp != null)
                str += "\n";
        } //while
        return str;
    } //[m] toString


    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "LinkedStack: Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        LinkedStack myStack = new LinkedStack();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myStack.push(next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)

        FI2Sys.out.println("Stack creato");
        FI2Sys.out.println(myStack);

        FI2Sys.out.println("size=" + myStack.size());
        FI2Sys.out.println("top=" +
          ((Integer)myStack.top()).intValue());

        myStack.pop();
        myStack.pop();
        FI2Sys.out.println("Lo stack dopo due pop e`:"+
          myStack);
        FI2Sys.out.println(
          "Svuota lo stack [ora genera eccezione]");
        for (; myStack.size()>=0; myStack.pop());
            // genera eccezione di stack vuoto
    } //[m][s] main

} //{c} LinkedStack
