package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * List.java
 * interfaccia per una sequenza con posizioni
 * e metodi read-write -
 * derivata da PositionalSequence della v 1.x 
 *
 * @version 2.00 00/09/20
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Set.Position;
import FI2.Set.PositionalContainer;

/**{i}
 * generica sequenza con accesso via posizionatore
 * e metodi di modifica
 * @see PositionalContainer
 */
public interface List 
  extends InspectableList, PositionalContainer
{
    /**[m][a]
     * inserisce come nuovo primo l'elemento 
     * fornito
     * @param e  nuovo oggetto
     * @return nuova prima posizione
     */
    Position insertFirst(Object e);

    /**[m][a]
     * inserisce come nuovo ultimo l'elemento 
     * fornito
     * @param e  nuovo oggetto
     * @return nuova ultima posizione
     */
    Position insertLast(Object e);

    /**[m][a]
     * inserisce l'elemento fornito come nuova
     * posizione immediatamente prima di quella fornita
     * @param p  posizionatore
     * @param e  nuovo oggetto
     * @return posizione dell'elemento inserito
     * @exception InvalidPositionException
     * posizionatore illegale
     */
    Position insertBefore(Position p, Object e)
      throws InvalidPositionException;

    /**[m][a]
     * inserisce l'elemento fornito come nuova
     * posizione immediatamente dopo quella fornita
     * @param p  posizionatore
     * @param e  nuovo oggetto
     * @return posizione dell'elemento inserito
     * @exception InvalidPositionException
     * posizionatore illegale
     */
    Position insertAfter(Position p, Object e)
      throws InvalidPositionException;

    /**[m][a]
     * rimuove l'elemento alla posizione fornita
     * @param p  posizionatore
     * @return l'elemento rimosso
     * @exception InvalidPositionException
     * posizionatore illegale
     */
    Object remove(Position p)
      throws InvalidPositionException;

    /**[m][a]
     * rimuove l'elemento immediatamente prima della
     * posizione fornita
     * @param p  posizionatore
     * @return  l'elemento rimosso
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception BoundaryViolationException
     * posizionatore p in prima posizione
     */
    Object removeBefore(Position p)
      throws InvalidPositionException, 
        BoundaryViolationException;

    /**[m][a]
     * rimuove l'elemento immediatamente dopo la
     * posizione fornita
     * @param p  posizionatore
     * @return  l'elemento rimosso
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception BoundaryViolationException
     * posizionatore p in ultima posizione
     */
    Object removeAfter(Position p)
      throws InvalidPositionException, 
        BoundaryViolationException;

    /**[m][a]
     * rimuove l'elemento in prima posizione
     * @return  l'elemento rimosso
     * @exception EmptyContainerException  sequenza vuota
     */
    Object removeFirst() throws EmptyContainerException;

    /**[m][a]
     * rimuove l'elemento in ultima posizione
     * @return  l'elemento rimosso
     * @exception EmptyContainerException  sequenza vuota
     */
    Object removeLast() throws EmptyContainerException;

} //{i} List
