package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * SeqHeapPriorityQueue.java
 * Implementazione con heap su sequenza con array
 * di coda a priorita` -
 * derivata da SeqHeapSPriorityQueue della v 1.x 
 *
 * @version 2.00 00/09/24
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Set.Container;
import FI2.Set.ObjectIterator;
import FI2.Set.Position;
import FI2.Set.Item;
import FI2.Util.FI2Sys;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Exceptions.EmptyContainerException;
import FI2.Util.Comparator;
import FI2.Util.IntegerComparator;
import FI2.Tree.SequenceBinaryTree;
import FI2.Tree.SequenceHeapTree;

/**{c}
 * coda a priorita` (non supporta Locator) 
 * realizzata mediante un heap - 
 * lo heap si appoggia su un albero binario
 * realizzato in forma di sequenza con array
 * @see HeapPriorityQueue
 */
public class SeqHeapPriorityQueue 
  extends HeapPriorityQueue 
{
    /**[c]
     * costruttore base, heap vuoto
     * @param c  comparatore per i confronti
     */
    public SeqHeapPriorityQueue(Comparator c) 
    {
        if ((comp = c) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        T = new SequenceHeapTree();
    } //[m] SeqHeapPriorityQueue
  
    /**[c]
     * costruttore da sequenza -
     * viene utilizzato per il metodo sort() e
     * assume la conoscenza della rappresentazione
     * di un albero mediante una sequenza -
     * crea i nodi dell'albero, assunto completo
     * come dev'essere un heap, collocando gli
     * elementi della sequenza fornita 
     * (indici 0..size-1) nei successivi nodi, che
     * risultano essere nodi interni dell'albero
     * (indici da ROOTRANK=1 a size). Indi chiama
     * buildHeap() che trasforma l'albero in un heap
     * 'rovescio' cioe` con la chiave maggiore nella radice -
     * scambiato questo elemento con l'ultimo nodo, si
     * ripete la trasformazione in heap trascurando
     * ad ogni ciclo l'ultimo elemento della sottosequenza
     * (che si sa gia` essere il maggiore della sottosequenza
     * del ciclo precedente): alla fine (l'ultimo ciclo 
     * coinvolge 2 elementi) la sequenza risulta ordinata -
     * O(n*log n)
     * @param s  sequenza da ordinare
     * @param c  comparatore per i confronti
     */
    public SeqHeapPriorityQueue(Sequence s, Comparator c) 
    {
        if ((comp = c) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        T = new SequenceHeapTree(); 
        // copia la sequenza fornita in quella che
        // rappresenta l'albero
        int size = s.size();
        Position curPos = 
          ((SequenceBinaryTree)T).
          atRank(SequenceBinaryTree.ROOTRANK);

        for (int i=1; i<=size; )
        {
            Object e = s.removeFirst();
            ((SequenceBinaryTree)T).expandExternal(curPos);
            T.replaceElement(curPos, new Item(e, e));
            curPos = ((SequenceBinaryTree)T).atRank(++i);
        } //for i=1
        buildHeap(size);
        for (int i=size; i>1; )
        {
            T.swapElements(((SequenceBinaryTree)T).
              atRank(SequenceBinaryTree.ROOTRANK), 
              ((SequenceBinaryTree)T).atRank(i));
              // scambia l'ultimo con il massimo
            heapify(--i, 
              ((SequenceBinaryTree)T).
              atRank(SequenceBinaryTree.ROOTRANK));
              // l'ultimo elemento viene trascurato
        } //for i=size
    } //[c] SeqHeapPriorityQueue

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new SeqHeapPriorityQueue(comp);
    } //[m] newContainer

    /**[m][s] !!
     * ordinamento di una sequenza mediante coda
     * a priorita` (rappresentata da this) -
     * gli elementi della coda debbono essere anche
     * interpetabili come chiavi ordinabili
     * usando il comparatore associato a this -
     * essendo lo heap realizzato con una sequenza,
     * e` possibile adottare la tecnica di costruzione
     * incrementale dello heap mediante bottomUpHeap()
     * @param s  sequenza da ordinare
     */
    public static void sort(Sequence s, Comparator c)
    {
        Object e;
        int size = s.size();
        SeqHeapPriorityQueue pq = 
          new SeqHeapPriorityQueue(s, c);
        // copia sequenza riordinata in quella
        // originaria
        for (int i=1; i<=size; i++)
            s.insertLast( ((Item)
              ((SequenceBinaryTree)pq.T).atRank(i).element())
              .element());
    } //[m][s] sort

    /**[m] !!
     * costruzione ricorsiva dello heap sulla sequenza che
     * rappresenta l'albero-heap di size elementi (quindi
     * vengono presi in considerazione solo i nodi da
     * ROOTRANK=1 a size) - assume come heap 'rovesci'
     * sia left(pos) che right(pos) ed effettua ricorsivamente 
     * un down-heap bubbling se necessario per far diventare
     * heap tutto l'albero di radice pos - 
     * alla fine il sottoalbero di radice pos 
     * contiene la chiave massima nella radice -
     * detto k il numero di elementi del sottoalbero in pos,
     * si ha O(log k), ovvero h=height(pos), O(h)
     * @param size  numero degli elementi
     * @param pos  radice del sottoalbero
     */
    private void heapify(int size, Position pos)
    {
        ArrayListPosition left = 
          (ArrayListPosition) T.leftChild(pos);
        ArrayListPosition right = 
          (ArrayListPosition) T.rightChild(pos);
        Position largest = (left.index() <= size &&
          comp.isGreaterThan(key(left), 
          key(pos))) ?
          left : pos;

        if (right.index() <= size &&
          comp.isGreaterThan(key(right), 
          key(largest)))
            largest = right;
        if (largest != pos)
        {
            // scambio
            T.swapElements(largest, pos);
            heapify(size, largest);
        }
    } //[m] heapify

    /**[m] !!
     * trasforma in heap 'rovescio' la sequenza che 
     * rappresenta l'albero -
     * la costruzione avviene bottom-up tenendo conto
     * della proprieta` di completezza e della 
     * rappresentazione nella sequenza - 
     * O(n)
     * @param size  numero degli elementi
     */
    private void buildHeap(int size)
    {
        for (int i=size>>1; i>=1; 
          heapify(size, ((SequenceBinaryTree)T).atRank(i--)));
    } //[m] buildHeap

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return "SeqHeapPriorityQueue [size=" + size() +
          "]:\n" + T.toString();
    } //[m] toString()

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
     
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        Comparator comp = new IntegerComparator();

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "SeqHeapPriorityQueue: " + 
              "Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        SeqHeapPriorityQueue myHSPQ = 
          new SeqHeapPriorityQueue(comp);
        Sequence mySeq = new ArraySequence();

        sIn.parseNumbers();
        try
        {
            Integer next;
            String str = "";

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    mySeq.insertLast(next);
                    str = "Valore (" + next.toString() + ")";
                    myHSPQ.insertItem(next, str);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (BoundaryViolationException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(BoundaryViolationException)

        FI2Sys.out.println("Priority Queue creato");
        FI2Sys.out.println(myHSPQ);

        FI2Sys.out.println("Lista chiavi");
        ObjectIterator keyIt = myHSPQ.keys();
        while (keyIt.hasNext())
            FI2Sys.out.println(keyIt.nextObject());
        
        FI2Sys.out.println("size=" + myHSPQ.size());
        FI2Sys.out.println("T.size=" + myHSPQ.T.size());
        ObjectIterator it = myHSPQ.elements();
        int pos = 0;
        while (it.hasNext())
            FI2Sys.out.println("elemento pos=" + (pos++) +
              " elem: " + it.nextObject());
        while (true)
        {
            try
            {
                FI2Sys.out.println("chiave minima: " +
                  ((Integer)myHSPQ.minKey()).intValue());
                FI2Sys.out.println(
                  "oggetto con chiave minima: " +
                  myHSPQ.minElement());
                FI2Sys.out.println("oggetto rimosso: " +
                  myHSPQ.removeMin());
            }
            catch (EmptyContainerException e)
            {
                FI2Sys.out.println("fine elementi ->" + e);
                break;
            }
        } //while

        FI2Sys.out.println("Sequenza non ordinata");
        FI2Sys.out.println(mySeq);
        SeqHeapPriorityQueue.sort(mySeq, comp);
        FI2Sys.out.println("Sequenza ordinata");
        FI2Sys.out.println(mySeq);
    } //[m][s] main

} //{c} SeqHeapPriorityQueue
