package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * ArrayLocatorSet.java
 * realizzazione di un insieme con vettore espandibile
 * e con supporto Locator -
 * derivata da ExpArrLocSet della v 1.x 
 *
 * @version 2.00 00/09/29
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidElementException;
import FI2.Exceptions.InvalidLocatorException;
import FI2.Util.FI2Sys;

/**{c}
 * estende ArraySet per implementare un LocatorSet
 * cioe` un insieme con supporto di Locator
 * @see ArraySet
 * @see LocatorSet
 */
public class ArrayLocatorSet extends ArraySet 
  implements LocatorSet
{
    /**[m]
     * inserimento di un LocArrSetElem nel set
     * @param loc  riferimento dell'oggetto da inserire
     * @return il riferimento
     */
    protected Locator locInsert(LocArrSetElem loc) 
    {
        addElem(loc);
          // aggiunge alla fine, quindi gli altri
          // riferimenti (Locator) rimangono validi
        loc.setPosition(size-1);
          // imposta posirione di inserimento
        loc.setContainer(this);
          // imposta contenitore
        return (Locator) loc;
    } //[m] locInsert

    /**[m]
     * rimozione di un LocArrSetElem
     * @param loc  riferimento dell'oggetto da rimuovere
     * @return il riferimento
     */
    protected LocArrSetElem locRemove(Locator loc) 
    {
        LocArrSetElem locAE = checkLocator(loc); 
          // verifica Locator
        int pos = locAE.position(); // posizione da eliminare

        // rimozione
        S.removeAtRank(pos);
        size--;
        for (int i=pos; i<size; i++)
            ((LocArrSetElem)S.elemAtRank(i)).setPosition(i);
              // corregge riferimenti
        locAE.setContainer(null);
          // invalidazione 
        return locAE;
    } //[m] locRemove

    /**[m]
     * verifica di un LocArrSetElem
     * @param loc  riferimento dell'oggetto da verifiacre
     * @return il riferimento come LocArrSetElem
     */
    protected LocArrSetElem checkLocator(Locator loc)
      throws InvalidLocatorException
    {
        // loc.container() verifica se il posizionatore
        // e` legale
        if (loc.container()!=this)
            throw new InvalidLocatorException(
              "Locator riferito ad altro contenitore");
        try
        {
            return (LocArrSetElem)loc;
        }
        catch(ClassCastException e)
        {
            throw new InvalidLocatorException(
              "Locator non valido in questo contesto");
        }
    } //[m] checkLocator

    // metodi della classe

    /**[c]
     * costruttore base 
     */
    public ArrayLocatorSet() 
    { super(); }

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {
        /**{c}{l}
         * classe locale che rappresenta 
         * l'iteratore interno -
         * si appoggia sull'enumeratore locators()
         */
        class Iter implements ObjectIterator
        {
            ObjectIterator locs;

            /**[c]
             * costruttore di default
             */
            Iter()
            {
                locs = locators();
            } //[c] Iter
    
            /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return locs.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return 
                  ((LocArrSetElem)locs.nextObject()).element();
            } //[m] nextObject

        } //{c} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    { return new ArrayLocatorSet(); }

    /**[m]
     * metodo ridefinito per inserire un LocArrSetElem
     * @see Set
     */
    public void insertElement(Object obj)
      throws InvalidElementException
    { insert(obj); }

    /**[m]
     * metodo ridefinito per rimuovere un LocArrSetElem
     * @see Set
     */
    public boolean removeElement(Object obj)
      throws InvalidElementException
    {
        int pos; // posizione da cercare

        for (pos=0; pos<size; pos++)
            if (obj.equals
              (((LocArrSetElem)S.elemAtRank(pos)).element()))
                break;
        if (pos >= size)
            return false;
        remove((LocArrSetElem)S.elemAtRank(pos));
        return true;
    } //[m] removeElement

    /**[m]
     * @see LocatorSet
     */
    public Locator insert(Object obj)
      throws InvalidElementException
    {
        LocArrSetElem loc = new LocArrSetElem(obj, null, -1);
        return locInsert(loc);
    }

    /**[m]
     * @see LocatorSet
     */
    public Locator insert(Locator loc)
      throws InvalidLocatorException, InvalidElementException
    { return locInsert((LocArrSetElem)loc); }

    /**[m]
     * @see LocatorSet
     */
    public void remove(Locator loc)
      throws InvalidLocatorException
    { locRemove(loc); }

    /**[m]
     * @see LocatorSet
     */
    public LocatorIterator locators()
    { 
        /**{c}{l}
         * classe locale che rappresenta 
         * l'iteratore interno
         */
        class LocIter implements LocatorIterator
        {
            ObjectIterator iter;

            /**[c]
             * costruttore di default
             */
            LocIter()
            {
                iter = ArrayLocatorSet.super.elements();
                  //!! super della classe associata
            } //[c] LocIter
    
            /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return iter.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return 
                  iter.nextObject();
            } //[m] nextObject

            /**[m]
             * @see LocatorIterator
             */
            public Locator nextLocator()
            {
                return 
                  (Locator)iter.nextObject();
            } //[m] nextLocator

        } //{c} LocIter

        return new LocIter();
    } //[m] locators

    /**[m][s]
     * metodo di collaudo
     * @param args[]  non usato
     */
    public static void main(String[] args)
    {
        LocatorSet set1 = new ArrayLocatorSet();
        LocatorSet set2 = new ArrayLocatorSet();
        LocatorSet dom = new ArrayLocatorSet();
        FI2Sys.out.println("Collaudo ArrayLocatorSet");
        FI2Sys.out.println("set1 inizialmente e`" +
          " [" + (set1.isEmpty() ? "" : "non ") + "vuoto]" );


        Object val;

        for (int i=1; i<=15; i++)
        {
            do
                val =
                  new Integer((int)(Math.random() * 20));
            while (set1.contains(val));
                // elementi distinti
            set1.insertElement(val);
        } //for

        // l'insieme puo` essere disuniforme
        Locator loc1 = set1.insert("Stringa 1");
        set1.insertElement("Stringa 2");

        for (int i=1; i<=7; i++)
        {
            do
                val =
                  new Integer((int)(Math.random() * 20));
            while (set2.contains(val));
                // elementi distinti
            set2.insertElement(val);
        } //for

        // l'insieme puo` essere disuniforme
        set2.insertElement("Stringa 1");
        set2.insertElement("Stringa 3");

        for (int i=0; i<=21; i++)
        {
            val = new Integer(i);
            dom.insertElement(val);
                // dominio 0..21
        } //for

        // l'insieme puo` essere disuniforme
        dom.insertElement("Stringa 1");
        dom.insertElement("Stringa 2");
        dom.insertElement("Stringa 3");

        Set sTmp;
        FI2Sys.out.println("\nset1: " + set1);
        FI2Sys.out.println("\nL'elemento " + loc1.element() +
          " e` in posizione " + 
          ((LocArrSetElem)loc1).position());
        FI2Sys.out.println("\nset2: " + set2);
        FI2Sys.out.println("\nset1 inizialmente e` " +
          (set1.isEqual(set2) ? "uguale " : "diverso ") 
          + "da set2" );
        FI2Sys.out.println("\n~set1: " +
          set1.complement(dom));
        FI2Sys.out.println("\n~set2: " +
          set2.complement(dom));
        FI2Sys.out.println("\nset1 + set2: " +
          (sTmp = set1.union(set2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nset1 & set2: " +
          (sTmp=set1.intersect(set2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nset1 - set2: " +
          (sTmp = set1.subtract(set2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        try
        {
            ((GenericSet)set1).insertElement(
              new Integer(2000), dom);
            // per provocare l'eccezione
        }
        catch (InvalidElementException ex)
        {
            // cattura
            FI2Sys.out.println(ex + "<<-- !!!");
        } //catch

        FI2Sys.out.println("\nIl dominio era (size=" + 
          dom.size() + "): " + dom);
        dom.removeElement("Stringa 2");
        FI2Sys.out.println(
          "\nIl dominio dopo la rimozione (size=" + 
          dom.size() + "): " + dom);

        FI2Sys.out.println("\nset1 era (size=" + 
          set1.size() + "): " + set1);
        FI2Sys.out.println("\nL'elemento " + loc1.element() +
          " e` in posizione " + 
          ((LocArrSetElem)loc1).position());
        set1.remove(loc1);
        FI2Sys.out.println("\nset1 dopo la rimozione (size=" + 
          set1.size() + "): ");
        LocatorIterator locs = set1.locators();
        while (locs.hasNext())
        {
            Locator loc = locs.nextLocator();
            FI2Sys.out.println("elemento: " + loc.element() +
             " posizione=" + ((LocArrSetElem)loc).position());
        } // while
        FI2Sys.out.println("\nIl dominio era (size=" + 
          dom.size() + "): " + dom);
        loc1 = dom.insert(loc1);
        FI2Sys.out.println(
          "\nIl dominio dopo l'inserzione (size=" + 
          dom.size() + "): " + dom);
        FI2Sys.out.println("\nL'elemento " + loc1.element() +
          " e` in posizione " + 
          ((LocArrSetElem)loc1).position());
    } //[m][s] main

} //{c} ArrayLocatorSet
