package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * GenericPosition.java
 * posizionatore generico
 *
 * @version 1.00 98/08/19
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidPositionException;

/**{c}
 * indicatore di posizione (posizionatore) su
 * contenitore generico -
 * sono dichiarati protected i metodi che debbono essere
 * visibili solo ai metodi dei contenitori che supportano
 * (a livello del package) i posizionatori e non alle
 * applicazioni esterne
 * 
 */
public class GenericPosition implements Position
{
    protected Container c; 
      // insieme su cui il posizionatore agisce
    protected Object e_;  // oggetto 'indicato'

    /**[c]
     * costruttore di default, posizionatore non valido
     */
    public GenericPosition()
    { this(null, null); }

    /**[c]
     * costruttore effettivo
     * @param cont  
     * contenitore su cui il posizionatore agisce
     * @param elem  oggetto associato
     */
    protected GenericPosition(Container cont, Object elem)
    {
        c = cont;
        e_ = elem;
    } //[c] GenericPosition

    /**[m]
     * per completezza, esistendo il costruttore nullo,
     * un posizionatore puo` effettivamente essere illegale
     * @see Position
     */
    public Object element() throws InvalidPositionException
    {
        if (c==null)
            throw new InvalidPositionException(
              "In GenericPosition.element");
        return e_;
    } //[m] element

    /**[m]
     * per completezza, esistendo il costruttore nullo,
     * un posizionatore puo` effettivamente essere illegale
     * @see Position
     */
    public Container container() 
      throws InvalidPositionException
    {
        if (c==null)
            throw new InvalidPositionException(
              "In GenericPosition.container");
        return c;
    } //[m] container

    /**[m]
     * fornisce l'elemento associato assumendo 
     * il posizionatore corretto
     * @return l'elemento associato
     */
    protected Object getElement()
    { return e_; }

    /**[m]
     * aggiorna l'elemento 'indicato' dal posizionatore
     * @param elem  elemento da assegnare
     */
    protected void setElement(Object elem) 
    { e_ = elem; }

    /**[m]
     * aggiorna l'insieme associato al posizionatore
     * @param cont  nuovo insieme
     * @exception InvalidPositionException  
     * posizionatore illegale
     */
    protected void setContainer(Container cont) 
      throws InvalidPositionException
    { c = cont; }

    /**[m]
     * rende non piu` valido il posizionatore
     */
    protected void setInvalid()
    { c = null; }

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        if (c==null)
            throw new InvalidPositionException(
              "In GenericPosition.toString");
        return "Generic Position cont.size=" + c.size() +
          " elem=" + e_;
    } //[m] toString

} //{i} GenericPosition
