package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * GenericSet.java
 * realizzazione astratta di un insieme generico
 *
 * @version 2.00 00/09/27
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidElementException;
import FI2.Set.ObjectIterator;

/**{c}{a}
 * realizzazione di un insieme generico di Object -
 * non puo` contenere ripetizioni dello stesso oggetto -
 * questa realizzazione fa vedere una semplice applicazione
 * del metodo template: i metodi non generalizzabili
 * debbono essere implementati dalle classi 'concrete'
 * che si potranno derivare da questa
 * @see Set
 */
public abstract class GenericSet 
  implements Set
{
    protected int size;  // numero di elementi dell'insieme

    /**[c]
     * O(1)
     * costruttore base, set vuoto
     */
    public GenericSet()
    {
        size=0;
    } //[c] GenericSet

    /**[m]
     * O(1)
     * @see Container
     */
    public int size()
    {
        return size;
    } //[m] size

    /**[m]
     * O(1)
     * @see Container
     */
    public boolean isEmpty()
    {
        return size == 0;
    } //[m] isEmpty

    /**[m][a]
     * aggiunge un elemento all'insieme che si
     * sa non gia` presente -
     * dipendente dalla realizzazione -
     * qualche realizzazione puo` sollevare
     * un'eccezione di tipo "full"
     * @param obj  oggetto da aggiungere
     */
    protected abstract void addElem(Object obj);

    /**[m][a]
     * rimuove un elemento all'insieme che si
     * sa gia` presente -
     * dipendente dalla realizzazione -
     * @param obj  oggetto da aggiungere
     */
    protected abstract void remElem(Object obj);

    /**[m]
     * @see Set
     */
    public void insertElement(Object obj)
      throws InvalidElementException
    {
        if (contains(obj))
            // tentativo di aggiunta di un elemento
            // gia` presente
            throw new InvalidElementException(
              "In GenericSet.insertElement [" + obj + 
              "] gia` presente");
        addElem(obj);
    } //[m] insertElement

    /**[m] !!
     * aggiunge un elemento all'insieme
     * supponendo l'elemento appartenente
     * al dominio fornito -
     * @param obj  oggetto da aggiungere
     * @param dom  insieme dominio
     * @exception InvalidElementException
     * se l'elemento non e` presente nel dominio
     */
    public void insertElement(Object obj, Set dom)
      throws InvalidElementException
    {
        if (!dom.contains(obj))
            // tentativo di aggiunta di un elemento
            // non nel dominio
            throw new InvalidElementException(
              "In Set.insertElement [" + obj + 
              "] fuori dominio");
        insertElement(obj);
    } //[m] insertElement
    
    /**[m]
     * @see Set
     */
    public boolean removeElement(Object obj)
      throws InvalidElementException
    {
        if (!contains(obj))
            // tentativo di rimozione di un elemento
            // non presente
            return false;
        remElem(obj);
        return true;
    } //[m] removeElement

    /**[m] !!
     * O(n)
     * @see Set
     */
    public boolean contains(Object obj)
    {
        ObjectIterator it1 = elements();
            // iteratore su this

        while (it1.hasNext())
            if (obj.equals(it1.nextObject()))
                // trovato
                return true;
        return false;
    } //[m] contains

    /**[m] !!
     * @see Set
     */
    public boolean isMember(Object obj)
    { return contains(obj); }

    /**[m]
     * O(n1*n2)
     * @see Set
     */
    public boolean isEqual(Set snd)
    {
        ObjectIterator it1 = elements();
            // iteratore su this

        while (it1.hasNext())
            if (!snd.contains(it1.nextObject()))
                // uno non contenuto
                return false;
        return true;
    } //[m] isEqual

    /**[m]
     * O(n1+n2)
     * @see Set
     */
    public Set union(Set snd)
    {
        ObjectIterator it1 = elements();
            // iteratore su this
        ObjectIterator it2 = snd.elements();
            // iteratore su Snd
        Set union = (Set) newContainer();
            // Set risultato
        Object tmp;
            // temporaneo

        while (it1.hasNext())
            union.insertElement(it1.nextObject());
                // copia di this
                // (si poteva definire in alternatica
                // il metodo clone())

        while (it2.hasNext())
            if (! union.contains(tmp = it2.nextObject()))
                // l'elemento del secondo Set non appartiene
                // gia` al primo Set
                union.insertElement(tmp);
        return union;
    } //[m] union

    /**[m]
     * O(n1+n2)
     * @see Set
     */
    public Set intersect(Set snd)
    {
        ObjectIterator it1 = elements();
            // iteratore su this
        Set inters = (Set) newContainer();
            // Bag risultato
        Object tmp;

        while (it1.hasNext())
            if (snd.contains(tmp = it1.nextObject()))
                // presente in entrambi
                inters.insertElement(tmp);
        return inters;
    } //[m] intersect

    /**[m]
     * O(n1+n2)
     * @see Set
     */
    public Set subtract(Set snd)
    {
        ObjectIterator it1 = elements();
            // iteratore su this
        Set diff = (Set) newContainer();
            // set risultato
        Object tmp;

        while (it1.hasNext())
            if (!snd.contains(tmp = it1.nextObject()))
                // presente in this ma non in Snd
                diff.insertElement(tmp);
        return diff;
    } //[m] subtract

    /**[m]
     * @see Set
     */
    public Set complement(Set dom)
    {
        return dom.subtract(this);
    } //[m] complement

    /**[m]
     * conversione a stringa
     * @return descrizione del contenuto
     */
    public String toString()
    {
        ObjectIterator it = elements();
        String str = "{GenericSet} "; 

output:
        for (int j=0; ; j++)
        {
            int pos = 0;
            for (int i=0; i<6; i++)
            {
                // 6 valori per riga
                pos = j*6+i;
                if (!it.hasNext())
                    // finita la lista
                    break output;
                str += pos + ": " +
                  it.nextObject().toString() + " | ";
            } //for(i)
            if (it.hasNext())
                str += "\n";
        } //for(j)
        return str;
    } //[m] toString

} //{c}{a} GenericSet
