package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * IntegerSet.java
 * realizzazione di un Set di valori interi
 *
 * @version 2.00 00/09/29
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidElementException;
import FI2.Util.FI2Sys;
import FI2.Util.SimpleComparable;

/**{c}
 * realizzazione di un insieme di interi
 * senza ripetizione di valori e supporto
 * con vettore espandibile -
 * si tratta di una specializzazione
 * di Set in quanto gli elementi non
 * sono piu` oggetti generici ma valori
 * interi (oltretutto NON sono proprio
 * oggetti) -
 * la realizzazione e` in forma di
 * 'adattatore' da ArraySet -
 * vedere [Good] pag 93
 * @see ArraySet
 */
public class IntegerSet implements Container
{
    private ArraySet Is;  // il set generico di supporto

    /**[c]
     * costruttore base, set vuoto
     */
    public IntegerSet()
    {
        Is = new ArraySet();
    } //[c] IntegerSet

    /**[c]
     * costruttore con un set generico gia` costruito
     * @param set  set generico 
     */
    public IntegerSet(ArraySet set)
    {
        Is = set;
    } //[c] IntegerSet

    /**[m]
     * @see InspectableContainer
     */
    public int size()
    {
        return Is.size();
    } //[m] size

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty()
    {
        return Is.isEmpty();
    } //[m] isEmpty

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {
        return Is.elements();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new IntegerSet();
    } //[m] newContainer

    /**[m]
     * aggiunge un intero all'insieme
     * @see Set
     * @param i  intero da aggiungere
     */
    public void insertElement(int i)
    {
        Is.insertElement(new Integer(i));
    } //[m] insertElement

    /**[m]
     * aggiunge un intero all'insieme
     * supponendo l'intero appartenente
     * al dominio fornito
     * @see Set
     * @param i  intero da aggiungere
     * @param dom  Set dominio
     * @exception InvalidElementException
     * se l'elemento e` gia` presente
         */
    public void insertElement(int i, IntegerSet dom)
      throws InvalidElementException
    {
        Is.insertElement(new Integer(i), dom.Is);
    } //[m] insertElement

    /**[m]
     * rimuove un intero dall'insieme se presente
     * @see Set
     * @param i  intero da aggiungere
     * @return true se rimosso; 
     *         false altrimenti
     */
    public boolean removeElement(int i)
    {
        return Is.removeElement(new Integer(i));
    } //[m] removeElement

    /**[m]
     * determina se un intero appartiene all'insieme -
     * per il confronto usa equals di Integer che
     * compara i valori rappresentati
     * @see Set
     * @param i  intero da cercare
     * @return true se l'intero e` un elemento
     *         dell'insieme;
     *         false altrimenti
     */
    public boolean contains(int i)
    {
        return Is.contains(new Integer(i));
    } //[m] contains

    /**[m]
     * per mantenere allineata la documentazione -
     * vedere [Good] pag. 314
     * @param obj  oggetto da cercare
     * @return true se l'oggetto e` un elemento
     *         dell'insieme;
     *         false altrimenti
     */
    public boolean isMember(int i)
    {
        return Is.contains(new Integer(i));
    } //[m] isMember

    /**[m]
     * verifica se i due set contengono gli stessi 
     * elementi
     * @param snd  secondo insieme da confrontare
     * @return true se contengono gli stessi elementi;
     *         false altrimenti
     * @see SimpleComparable
     */
    public boolean isEqual(IntegerSet snd)
    {
        return Is.equals(snd.Is);
    } //[m] isEqual

    /**[m]
     * restituisce il Set di interi unione
     * @see Set
     * @param snd  secondo set
     * @return Set unione
     */
    public IntegerSet union(IntegerSet snd)
    {
        return new IntegerSet((ArraySet)Is.union(snd.Is));
    } //[m] union

    /**[m]
     * restituisce il Set intersezione
     * @see Set
     * @param snd  secondo set
     * @return Set intersezione
     */
    public IntegerSet intersect(IntegerSet snd)
    {
        return 
          new IntegerSet((ArraySet)Is.intersect(snd.Is));
    } //[m] intersect

    /**[m]
     * restituisce il Set differenza
     * @see Set
     * @param snd  secondo set
     * @return Set differenza (this - Snd)
     */
    public IntegerSet subtract(IntegerSet snd)
    {
        return new IntegerSet((ArraySet)Is.subtract(snd.Is));
    } //[m] subtract

    /**[m]
     * restituisce il Set complemento
     * rispetto al dominio fornito
     * @see Set
     * @param dom  dominio
     * @return Set complemento (Dom - this)
     */
    public IntegerSet complement(IntegerSet dom)
    {
        return new 
          IntegerSet((ArraySet)Is.complement(dom.Is));
    } //[m] complement

    /**[m]
     * conversione a stringa
     * @return descrizione del contenuto
     */
    public String toString()
    {
        return Is.toString();
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[]  non usato
     */
    public static void main(String[] args)
    {
        IntegerSet set1 = new IntegerSet();
        IntegerSet set2 = new IntegerSet();
        IntegerSet dom = new IntegerSet();
        FI2Sys.out.println("Collaudo IntegerSet");
        FI2Sys.out.println("set1 inizialmente e`" +
          " [" + (set1.isEmpty() ? "" : "non ") + "vuoto]" );

        int val;

        for (int i=1; i<=15; i++)
        {
            do
                val = (int)(Math.random() * 20);
            while (set1.contains(val));
                // elementi distinti
            set1.insertElement(val);
        } //for

        for (int i=1; i<=7; i++)
        {
            do
                val = (int)(Math.random() * 20);
            while (set2.contains(val));
                // elementi distinti
            set2.insertElement(val);
        } //for

        for (int i=0; i<=21; dom.insertElement(i++));
                // dominio 0..21

        IntegerSet sTmp;
        FI2Sys.out.println("\nset1: " + set1);
        FI2Sys.out.println("\nset2: " + set2);
        FI2Sys.out.println("\nset1 inizialmente e`" +
          (set1.isEqual(set2) ? "uguale " : "diverso ") 
          + "da set2" );
        FI2Sys.out.println("\n~set1: " +
          set1.complement(dom));
        FI2Sys.out.println("\n~set2: " +
          set2.complement(dom));
        FI2Sys.out.println("\nset1 + set2: " +
          (sTmp = set1.union(set2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nset1 & set2: " +
          (sTmp=set1.intersect(set2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nset1 - set2: " +
          (sTmp = set1.subtract(set2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nIl numero 3 " + 
          (set1.contains(3) ? "" : "non ") + 
        "e` presente in set1" );
        FI2Sys.out.println("\nIl numero 40 " + 
          (set1.contains(40) ? "" : "non ") + 
        "e` presente in set1" );
        try
        {
            set1.insertElement(2000, dom);
            // per provocare l'eccezione
        }
        catch (InvalidElementException ex)
        {
            // cattura
            FI2Sys.out.println(ex + "<<-- !!!");
        } //catch

        FI2Sys.out.println("\nIl dominio era: " + dom);
    } //[m][s] main

} //{c} IntegerSet

