package FI2.Sorting;
/**{p}
 * -- FI2 --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Sorting
 */

/**{f}
 * ArraySort.java
 * Raccolta dei metodi di sorting su array
 *
 * @version 2.00 98/09/23
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.Comparable;
import FI2.Util.ComparableInt;
import FI2.Util.Metrics;
import FI2.Util.FI2Sys;

/**{c}
 * include tutti i metodi (statici) d'interesse per il sorting
 * su un array di oggetti SimpleComaparable -
 * l'array e` sempre passato via parametro 
 */
public class ArraySort {

    private static Comparable[] aux;
        // vettore ausiliario per merge sort

    private static Comparable[] locVec;
    private static Metrics locMet;
        // riferimenti interni per evitare il passaggio di
        // parametro nelle ricorsive
    private static final int CUTOFF = 10;
        // parametro per full-blown quick sort

    // indici-codifica del metodo di sort scelto
    public static final int NUL_SSEL = 0; 
    public static final int INSERTION_SSEL = NUL_SSEL+1;
    public static final int 
      BININSERTION_SSEL = INSERTION_SSEL+1;
    public static final int SHELL_SSEL = BININSERTION_SSEL+1;
    public static final int SELECTION_SSEL = SHELL_SSEL+1;
    public static final int BUBBLE_SSEL = SELECTION_SSEL+1;
    public static final int SHAKER_SSEL = BUBBLE_SSEL+1;
    public static final int MERGE_SSEL = SHAKER_SSEL+1;
    public static final int ITERMERGE_SSEL = MERGE_SSEL+1;
    public static final int HEAP_SSEL = ITERMERGE_SSEL+1;
    public static final int QUICK_SSEL = HEAP_SSEL+1;
    public static final int RANDQUICK_SSEL = QUICK_SSEL+1;
    public static final int 
      FULLBQUICK_SSEL = RANDQUICK_SSEL+1;
    public static final int LASTNUL_SSEL = FULLBQUICK_SSEL+1;

    public static final String methTab[] =
    {
        "nullSort", // indice 0
        "insertionSort",
        "binInsertionSort",
        "shellSort",
        "selectionSort",
        "bubbleSort",
        "shakerSort",
        "mergeSort",
        "iterMergeSort",                                        
        "heapSort",
        "quickSort",
        "randQuickSort",
        "fullbQuickSort"
    }; //methTab

    private int select = NUL_SSEL;  // sort scelto

    /**[m][s]
     * insertion sort (vedi carte da gioco):
     * ad ogni ciclo si posiziona un nuovo elemento
     * preso all'inizio della seconda parte (non ordinata) e
     * posto nel giusto rodine nella prima (ordinata) -
     * Read 2(n-1) = #(n) -
     * best case: Cmp n-1 = #(n) -
     *            Move 0 = #(1) -
     * worst case: Cmp Move 0.5(n**2-n) = #(n**2) -
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void insertionSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        met.init();
        for (int j=1; j<numEl; j++)
        {
            met.readCnt++;
            Comparable key = vec[j];
            int i = j-1;
            while (i >= 0 &&
              vec[i].greater(key) )
            {
                met.cmpCnt++;
                met.swapCnt++;
                vec[i+1] = vec[i];
                i--;
            } //while
            if (i >= 0)
                // un confronto finale in piu`
                met.cmpCnt++;
            met.readCnt++;
            vec[i+1] = key;
        } // for(j)
        met.end();
    } //[m][s] insertionSort

    /**[m][s]
     * binary insertion sort:
     * metodo divide-et-impera, simile a insertion sort,
     * solo che per la ricerca della posizione di inserimento
     * si sfrutta il metodo di ricerca binaria, che
     * ad ogni passo dimezza l'incertezza sciegliendo
     * in quale delle due meta` della porzione corrente
     * proseguire la ricerca
     * Read 2(n-1) = #(n) -
     * best case: Cmp SUMi [1..n] (ceil(log2 i)] = 
     *            #(n*log2 n) -
     *            Move 0 = #(1) -
     * worst case: Cmp SUMi [1..n] (ceil(log2 i)] = 
     *             #(n*log2 n) -
     *             Move 0.5(n**2-n) = #(n**2)
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void binInsertionSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        met.init();
        for (int j=1; j<numEl; j++)
        {
            met.readCnt++;
            Comparable key = vec[j];
            int low = 0, high = j-1;
            while (low <= high)
            {
                int med = (low+high)>>1;
                met.cmpCnt++;
                if (vec[med].greater(key))
                    high = med-1;
                else
                    low = med+1;
            } //while
            for (int i=j-1; i>=low; i--)
            {
                met.swapCnt++;
                vec[i+1] = vec[i];
            } //for(i)
            met.readCnt++;
            vec[low] = key;
        } // for(j)
        met.end();
    } //[m][s] binInsertionSort

    /**[m][s]
     * shell sort:
     * variante di insertion sort, l'ordinamento
     * viene eseguito su sub-array i cui elementi
     * sono a distanza fissa h >= 1, con h decrescente -
     * l'analisi di questo algoritmo e` difficoltosa,
     * si congettura una complessita` #(n * (log2 n)**2)
     * e #(n ** 1.25) -
     * il ciclo while(h>0) viene eseguito un numero di volte
     * pari a K=floor(log3 (floor(n/9)*2+1))+1
     * Read 2 * SUMi [1..K] (n-hi) = 2 * (K*n - SUMi hi) =
     * 2 * (K*n - (3**(K+1)-2K-3)/4)
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void shellSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        met.init();
        int h=1;
        for (; h <= numEl/9; h = 3*h+1);
            // calcola lo h di partenza
            // la sequenza, in ordine crescente, per h e`:
            // 1, 4, 13, 40, 121 ...

        while (h > 0)
        {
            // ordinamento con inserzione di ordine h
            // h e` la distanza degli elementi del sub-array

            for (int j=h; j<numEl; j++)
            {
                met.readCnt++;
                Comparable key = vec[j];
                int i = j-h;
                while (i >= 0 &&
                  vec[i].greater(key) )
                {
                    met.cmpCnt++;
                    met.swapCnt++;
                    vec[i+h] = vec[i];
                    i -= h;
                } //while
                if (i >= 0)
                    // un confronto finale in piu`
                    met.cmpCnt++;
                met.readCnt++;
                vec[i+h] = key;
            } // for(j)

            h /= 3;
                // aggiorna h
        } //while(h)
        met.end();
    } //[m][s] shellSort

    /**[m][s]
     * selection sort:
     * ad ogni ciclo trova il minimo nella 
     * seconda parte (non ordinata) e
     * lo pone alla fine della prima (ordinata) -
     * Cmp 0.5(n**2-n) = #(n**2) -
     * best case: Move 0 = #(1) -
     * worst case: Move n-1 = #(n) -
     * (quando sono inizialmente ordinati ma sfasati di uno) -
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void selectionSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        met.init();
        for (int j=0; j<numEl-1; j++)
        {
            int k = j;
            for (int i=j+1; i<numEl; i++)
            {
                met.cmpCnt++;
                if (vec[k].greater(vec[i]))
                    k = i;
            } //for(i)

            if (k != j)
            {
                // scambio
                Comparable tmp = vec[k];
                vec[k] = vec[j];
                vec[j] = tmp;
                met.swapCnt++;
                    // metrica di scambio
            } //if
        } // for(j)
        met.end();
    } //[m][s] selectionSort

    /**[m][s]
     * bubble sort:
     * quando due oggetti adiacienti non
     * rispettano l'ordinamento, vengono scambiati
     * attraverso lo scambio dei riferimenti nell'array -
     * Cmp 0.5(n**2-n) = #(n**2) -
     * best case: Move 0 = #(1) -
     * worst case: Move 0.5(n**2-n) = #(n**2) -
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void bubbleSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        met.init();
        for (int i=0; i<numEl-1; i++)
            for (int j=i+1; j<numEl; j++)
            {
                met.cmpCnt++;
                    // metrica di confronto
                if (vec[i].greater(vec[j]))
                {
                    // scambio
                    Comparable tmp = vec[i];
                    vec[i] = vec[j];
                    vec[j] = tmp;
                    met.swapCnt++;
                        // metrica di scambio
                } //if
            } //for(j)
        met.end();
    } //[m][s] bubbleSort

    /**[m][s]
     * shaker sort:
     * variante di bubble sort che alterna il verso
     * delle passate successive e tiene conto delle
     * sottoparti gia` ordinate evitando confronti
     * inutili -
     * best case: Cmp n-1 = #(n) -
     *            Move 0 <-> #(1) -
     * worst case: Cmp Move 0.5(n**2-n) = #(n**2)
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void shakerSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        met.init();
        int last = numEl-1;
        for (int low=1, high=numEl-1; low<=high; )
        {
            for (int j=high; j>=low; j--)
            {
                met.cmpCnt++;
                    // metrica di confronto
                if (vec[j-1].greater(vec[j]))
                {
                    last = j;
                        // Da last indietro forse e` ordinato
                    // scambio
                    Comparable tmp = vec[j-1];
                    vec[j-1] = vec[j];
                    vec[j] = tmp;
                    met.swapCnt++;
                        // metrica di scambio
                } //if
            } //for(j)
            low = last+1;
                // Da last indietro E` ordinato

            for (int j=low; j<=high; j++)
            {
                met.cmpCnt++;
                    // metrica di confronto
                if (vec[j-1].greater(vec[j]))
                {
                    last = j;
                        // Da last in avanti forse e` ordinato
                    // scambio
                    Comparable tmp = vec[j-1];
                    vec[j-1] = vec[j];
                    vec[j] = tmp;
                    met.swapCnt++;
                        // metrica di scambio
                } //if
            } //for(j)
            high = last-1;
                // Da last in avanti E` ordinato
        } //for(Low)
        met.end();
    } //[m][s] shakerSort

    /**[m][s]
     * metodo base per merge sort
     * fonde in modo ordinato la parte di vettore compresa
     * tra Low e Mid con quella tra Mid+1 e High
     * supposte entrambe ordinate -
     * k = High-Low+1 (numero elementi sub-array) -  
     * Read 2k = #(k)
     * worst case: Cmp k-1 = #(k)
     * @param low  indice inferiore
     * @param mid  indice intermedio
     * @param high  indice superiore
     */
    private static void merge
      (int low, int mid, int high)
    {
        int p1 = low, p2 = mid+1;
            // p1 scorre sulla prima meta`, p2 sulla seconda
/*
realizzazione inefficiente

        for (int p=low; p<=high; p++)
        {
            locMet.readCnt++;
                // fa comunque un accesso in lettura
                // per ogni ciclo
            if (p1 > mid)
                // prima meta` esaurita: copia della seconda
                aux[p] = locVec[p2++];
            else if (p2 > high)
                // seconda meta` esaurita, copia della prima
                aux[p] = locVec[p1++];
            else
            {
                locMet.cmpCnt++;
                if (! locVec[p1].greater(locVec[p2]) )
                    // l'oggetto nella prima meta` e`
                    // minore o uguale a quello nella seconda
                    aux[p] = locVec[p1++];
                else
                    // l'oggetto nella prima meta` e`
                    // maggiore a quello nella seconda
                    aux[p] = locVec[p2++];
            } // else
        } // for (p)
*/

/* realizzazione alternativa */

        int p = low;
        while (p1 <= mid && p2 <= high)
        {
            locMet.readCnt++;
            locMet.cmpCnt++;
            aux[p++] = (locVec[p1].greater(locVec[p2]) ?
              locVec[p2++] : locVec[p1++]);
                // copia il minore,
                // se eguali prende dalla prima
        } //while
        if (p1 > mid)
            // prima meta` esaurita: copia della seconda
            for (; p2<=high; locMet.readCnt++,
              aux[p++] = locVec[p2++]);
        else
            // seconda meta` esaurita: copia della prima
            for (; p1<=mid; locMet.readCnt++,
              aux[p++] = locVec[p1++]);

        // copia vettore ausiliario aggiornando 
        // quello originario
        for (p=low; p<=high; p++)
        {
            locVec[p] = aux[p];
            locMet.readCnt++;
        } //for(p)
    } //[m][s] merge

    /**[m][s]
     * metodo ricorsivo per merge sort
     * mette in ordine la porzione tra Low e High -
     * k = High-Low+1 (numero elementi sub-array) -
     * Read 0 = #(1)  se k=1;
     *      2*Read(k/2) + 2k = 2*Read(k/2) + #(k)  se k>1 -
     *      risulta #(k*log2 k) - 
     * analogo per Cmp in worst case
     * @param low  indice inferiore
     * @param high  indice superiore
     */
    private static void mSort
      (int low, int high)
    {
        if (low < high)
        {
            mSort(low, (low+high)>>1);
                // ordina la prima meta`
            mSort(((low+high)>>1)+1, high);
                // ordina la seconda meta`
            merge(low, (low+high)>>1, high);
                // fonde le due meta` in ordine
        } //if
    } //[m][s] mSort

    /**[m][s]
     * merge sort: metodo divide-et-impera con
     * vettore ausiliario -
     * complessita` #(n*log2 n)
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void mergeSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        aux = new Comparable[numEl];
        locVec = vec;
        locMet = met;
            // per evitare il passaggio come parametro
        locMet.init();
        mSort(0, numEl-1);
        locMet.end();
    } //[m][s] mergeSort

    /**[m][s]
     * iterative merge sort: versione iterativa
     * di merge sort
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void iterMergeSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        int i, n;
        aux = new Comparable[numEl];
        locVec = vec;
        locMet = met;
            // per evitare il passaggio come parametro
        /* i rappresenta il numero di elementi nel subarray
        corrente, che raddoppia ad ogni ciclo
          n = i*2 mantenuto in una variabile per efficienza */
        locMet.init();
        for (i=1, n=2; n < (numEl<<1); i <<= 1, n <<= 1)
            for (int j=0; j < numEl; j += n)
                // test per i casi in cui numEl non e`
                // potenza di 2
                if (j+i < numEl)
                    merge(j, i+j-1, ((j+n < numEl) ? j+n-1 :
                      numEl-1));
        locMet.end();
    } //[m][s] iterMergeSort

    /**[m][s]
     * metodo ricorsivo base per heap sort
     * considera LEFT(Pos) e Right(Pos) come heap
     * e sistema le cose in modo che Pos sia la radice
     * di un heap -
     * tener presente che gli indici partono da 0 -
     * k numero di nodi dell'albero di radice LEFT(Pos) +
     * numero di nodi dell'albero di radice RIGHT(Pos) + 1 -
     * complessita` O(log2 k) -
     * h altezza del nodo Pos -
     * complessita` O(h)
     * @param size  heap-size
     * @param pos  posizione di riferimento
     */
    private static void heapify
      (int size, int pos)
    {
        int left = (pos<<1)|1;
            // LEFT(Pos)
            // es. LEFT(0) == 1, LEFT(8) == 17
        int right = (pos+1)<<1;
            // RIGHT(Pos)
            // es. RIGHT(0) == 2, RIGHTT(8) == 18

        if (left < size)
            locMet.cmpCnt++;
        int largest =
          (left < size &&
          locVec[left].greater(locVec[pos]) ) ?
          left :
          pos;

        if (right < size)
            locMet.cmpCnt++;
        if (right < size &&
          locVec[right].greater(locVec[largest]))
            largest = right;
        if (largest != pos)
        {
            // scambio
            Comparable tmp = locVec[pos];
            locVec[pos] = locVec[largest];
            locVec[largest] = tmp;
            locMet.swapCnt++;
                // metrica di scambio
            heapify(size, largest);
        } //if
    } //[m][s] heapify

    /**[m][s]
     * rende il vettore fornito un heap di size pari
     * alla sua ampiezza -
     * complessita` O(n)
     * @param numEl  numero di elementi presenti
     */
    private static void buildHeap
      (int numEl)
    {
        for (int i=(numEl>>1)-1; i>=0;
          heapify(numEl, i--));
    } //[m][s] buildHeap

    /**[m][s]
     * heap sort: metodo divide-et-impera basato
     * sulla costruzione di un heap -
     * complessita` O(n*log2 n)
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void heapSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        locVec = vec;
        locMet = met;
            // per evitare il passaggio come parametro
        locMet.init();
        int size = numEl;
        buildHeap(numEl);
        for (int i=numEl-1; i>0; i--)
        {
            // scambio
            Comparable tmp = locVec[0];
            locVec[0] = locVec[i];
            locVec[i] = tmp;
            locMet.swapCnt++;
                // metrica di scambio
            heapify(--size, 0);
                // ad ogni ciclo decrementa lo heap-size

        } //for
        locMet.end();
    } //[m][s] heapSort

    /**[m][s]
     * metodo base per quicksort,
     * suddivide e riordina la porzione p..r in due
     * parti, p..q e q+1..r, tali per cui
     * vec[i] <= x per p..i..q,
     * vec[j] >= x per p+1..j..r -
     * n = r-p+1 -
     * complessita` #(n)
     * @param p  indice inferiore
     * @param r  indice superiore
     */
    private static int partition
      (int p, int r)
    {
        Comparable x = locVec[p];
            // pivot
        int i = p-1;
        int j = r+1;
        while (true)
        {
            do
            {
                j--;
                locMet.cmpCnt++;
            }
            while ( locVec[j].greater(x) );
                // si ferma quando trova un elemento <= pivot

            do
            {
                i++;
                locMet.cmpCnt++;
            }
            while ( x.greater(locVec[i]) );
                // si ferma quando trova un elemento >= pivot

            if (i < j)
            {
                // c'e` da riordinare, scambio e continua
                Comparable tmp = locVec[i];
                locVec[i] = locVec[j];
                locVec[j] = tmp;
                locMet.swapCnt++;
                    // metrica di scambio
            } //if
            else
                // condizione raggiunta
                return j;
        } // while(true)
    } //[m][s] partition

    /**[m][s]
     * metodo ricorsivo per quicksort,
     * mette in ordine la porzione p..r del vettore  
     * worst case: #(n**2) -
     * best case: #(n*log2 n) -
     * average case: O(n*log2 n)
     * @param p  indice inferiore
     * @param r  indice superiore
     */
    private static void qSort
      (int p, int r)
    {
        if (p < r)
        {
            int q = partition(p, r);
            qSort(p, q);
            qSort(q+1, r);
        } //if
    } //[m][s] qSort

    /**[m][s]
     * quicksort: metodo divide-et-impera ricorsivo -
     * complessita` O(n*log2 n)
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void quickSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        locVec = vec;
        locMet = met;
            // per evitare il passaggio come parametro
        locMet.init();
        qSort(0, numEl-1);
        locMet.end();
    } //[m][s] quickSort

    /**[m][s]
     * partizione per randomized quicksort,
     * esegue lo scambio casuale prima della partizione
     * @param p  indice inferiore
     * @param r  indice superiore
     */
    private static int randPartition
      (int p, int r)
    {
        int i = (int)((r-p)*Math.random()+p);

        // scambio preventivo
        Comparable tmp = locVec[p];
        locVec[p] = locVec[i];
        locVec[i] = tmp;
        locMet.swapCnt++;
            // metrica di scambio
        return partition(p, r);
    } //[m][s] randPartition

    /**[m][s]
     * metodo ricorsivo per randomized quicksort,
     * mette in ordine la porzione p..r del vettore  
     * worst case: #(n**2) -
     * best case: #(n*log2 n) -
     * average case: O(n*log2 n)
     * @param p  indice inferiore
     * @param r  indice superiore
     */
    private static void randQSort
      (int p, int r)
    {
        if (p < r)
        {
            int q = randPartition(p, r);
            randQSort(p, q);
            randQSort(q+1, r);
        } //if
    } //[m][s] randQSort

    /**[m][s]
     * randomized quicksort: metodo divide-et-impera ricorsivo
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void randQuickSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        locVec = vec;
        locMet = met;
            // per evitare il passaggio come parametro
        locMet.init();
        randQSort(0, numEl-1);
        locMet.end();
    } //[m][s] randQuickSort

    /**[m][s]
     * determina un pivot piu` ragionevole
     * posizionando in vec[p] il valore medio
     * delle chiavi di vec[p], vec[(p+r)/2], vec[r]
     * @param p  indice inferiore
     * @param r  indice superiore
     */
    private static void medianOfThree
      (int p, int r)
    {
        if (r-p >= 2)
        {
            locMet.readCnt += 3;
            Comparable beg = locVec[p];
            Comparable mid = locVec[(p+r)>>1];
            Comparable end = locVec[r];

            locMet.cmpCnt++;
            if (mid.greater(beg))
            {
                locMet.cmpCnt++;
                if (end.greater(mid))
                    // mid e` mediano
                {
                    locVec[p] = mid;
                    locVec[(p+r)>>1] = beg;
                    locMet.swapCnt++;
                        // metrica di scambio
                } //if end>mid
                else
                {
                    locMet.cmpCnt++;
                    if (end.greater(beg))
                        // end e` mediano
                    {
                        locVec[p] = end;
                        locVec[r] = beg;
                        locMet.swapCnt++;
                            // metrica di scambio
                    } //if end>beg
                    // else beg e` gia mediano
                } //else end !> mid
            }
            else
            {
                locMet.cmpCnt++;
                if (end.greater(mid))
                {
                    locMet.cmpCnt++;
                    if (end.greater(beg))
                        // beg e` gia` mediano
                    {
                    } //if end>beg
                    else
                        // end e` mediano
                    {
                        locVec[p] = end;
                        locVec[r] = beg;
                        locMet.swapCnt++;
                            // metrica di scambio
                    } //else end !> beg
                } //if end>mid
                else
                    // mid e` mediano
                {
                    locVec[p] = mid;
                    locVec[(p+r)>>1] = beg;
                    locMet.swapCnt++;
                        // metrica di scambio
                } //else end !> mid
            } //else mid !> beg
        } //if (r-p)
    } //[m][s] medianOfThree

    /**[m][s]
     * metodo ricorsivo per full-blown quick sort,
     * si ferma a CUTOFF elementi  
     * @param p  indice inferiore
     * @param r  indice superiore
     */
    private static void fullbQSort
      (int p, int r)
    {
        if (r-p >= CUTOFF)
        {
            while (p < r)
            {
                medianOfThree(p, r);
                int q = partition(p, r);
                fullbQSort(p, q);
                p = q+1;
            } //while
        } //if
    } //[m][s] fullbQSort

    /**[m][s]
     * full-blown quicksort: variante di quicksort
     * con partizionamento media-di-tre
     * ed eliminazione del tail-recursion
     * migliora il tutto fermando
     * l'ordinamento a CUTOFF elementi
     * e completandolo con insertion sort -
     * e` ritenuta la variante piu` efficiente 
     * @param vec  array da ordinare
     * @param numEl  numero di elementi presenti
     * @param met  metriche
     */
    public static void fullbQuickSort
      (Comparable[] vec, int numEl, Metrics met)
    {
        locVec = vec;
        locMet = met;
            // per evitare il passaggio come parametro
        locMet.init();
        fullbQSort(0, numEl-1);
        locMet.end();
        Metrics met2 = new Metrics();
        insertionSort(vec, numEl, met2);
        met.add(met2);
    } //[m][s] fullbquickSort

    /**[c]
     * consente di creare un'istanza che sceglie uno dei
     * metodi proposti
     * @param meth  indice-codifica del metodo proposto
     */
    public ArraySort(int meth)
    {
        select = (meth < 0 || meth >= LASTNUL_SSEL) ?
        NUL_SSEL : meth;
    } //[c] ArraySort

    /**[c]
     * consente di creare un'istanza che sceglie uno dei
     * metodi proposti
     * @param meth  stringa identificativo del metodo:
     *              coincide con il nome del metodo statico
     */
    public ArraySort(String meth)
    {
        if (meth.length() != 0)
        {
            // stringa nonnulla
            for (int i=0; i<LASTNUL_SSEL; i++)
                if (methTab[i].equals(meth))
                {
                    select = i;
                    return;
                } //if
            select = NUL_SSEL;
        } //if
        else
            // default
            select = NUL_SSEL+1;
                // il primo
    } //[c] ArraySort

    /**[m]
     * verifica se un metodo di sort e` selezionato
     */
    public boolean isSortable()
    {
        return select != NUL_SSEL;
    } //[m] isSortable

    /**[m]
     * indice-codifica del metodo scelto
     */
    public int methCode()
    {
        return select;
    } //[m] methCode

    /**[m]
     * stringa corrispondente al metodo scelto
     */
    public String method()
    {
        return methTab[select];
    } //[m] method

    /**[m]
     * attiva il metodo di sort scelto
     */
    public void sort(Comparable[] vec, int numEl,
      Metrics met)
    {
        switch(select)
        {
          case NUL_SSEL:
              // nop
              return;
          case INSERTION_SSEL:
              insertionSort(vec, numEl, met);
              return;
          case BININSERTION_SSEL:
              binInsertionSort(vec, numEl, met);
              return;
          case SHELL_SSEL:
              shellSort(vec, numEl, met);
              return;
          case SELECTION_SSEL:
              selectionSort(vec, numEl, met);
              return;
          case BUBBLE_SSEL:
              bubbleSort(vec, numEl, met);
              return;
          case SHAKER_SSEL:
              shakerSort(vec, numEl, met);
              return;
          case MERGE_SSEL:
              mergeSort(vec, numEl, met);
              return;
          case ITERMERGE_SSEL:
              iterMergeSort(vec, numEl, met);
              return;
          case HEAP_SSEL:
              heapSort(vec, numEl, met);
              return;
          case QUICK_SSEL:
              quickSort(vec, numEl, met);
              return;
          case RANDQUICK_SSEL:
              randQuickSort(vec, numEl, met);
              return;
          case FULLBQUICK_SSEL:
              fullbQuickSort(vec, numEl, met);
              return;
        } //switch(Select)
    } //[m] sort

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: metodo scelto
     * @param args[1]  opzionale: nome file di input
     * @exception IOException  se file non trovato
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length <= 1)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "Battere il numero di elementi:");
        }
        else
            in = new FileInputStream(args[1]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader
      (new InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);

        sIn.parseNumbers();

        sIn.nextToken();
        int len = (int)sIn.nval;
        FI2Sys.out.println("Numero elementi: " + len);
        ComparableInt[] vec = new ComparableInt[len];
        ComparableInt[] vecWork = new ComparableInt[len];

        for(int i=0; i<len; )
        {
            if (args.length == 0)
                FI2Sys.out.println("Battere elemento "
                  + (i+1) + ":");
            sIn.nextToken();
            if (sIn.ttype == StreamTokenizer.TT_EOF)
                break;
            if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                vec[i++] = new ComparableInt((int)sIn.nval);
            else
                FI2Sys.out.println(sIn.sval +
                  " non numero: non inserito");
        }


        ArraySort myAs = (
          args.length == 0  || args[0].equals("Any") ?
          new ArraySort("") : // primo metodo
          new ArraySort(args[0]));
        Metrics myMet = new Metrics();
        for(int i=0; i<len; vecWork[i] = vec[i], i++);
            // copia il vettore di riferimenti per
            // poter ripetere il sorting piu` volte

        myAs.sort(vecWork, len, myMet);
            // il vettore ha ora i numeri riordinati

        do
        {
            FI2Sys.out.println(myAs.method() +
          ": vettore creato");
            FI2Sys.out.println("Primi 8 (o len) elementi");
            for (int i=0; i< ((len >= 8) ? 8 : len); i++)
                FI2Sys.out.println(i + ": " + vecWork[i]);
            if (len >= 8)
            {
                FI2Sys.out.println("Ultimi 8 elementi");
                for (int i=0; i<8; i++)
                    FI2Sys.out.println(len-8+i + ": " +
                      vecWork[len-8+i]);
            }//if
            FI2Sys.out.println("Metriche:" +
          myMet);
            if (args.length != 0 && args[0].equals("Any"))
                // si passa al successivo
                myAs = new ArraySort(myAs.methCode()+1);
            else
                myAs = new ArraySort(ArraySort.NUL_SSEL);
            for(int i=0; i<len; vecWork[i] = vec[i], i++);
                // copia il vettore di riferimenti per
                // poter ripetere il sorting piu` volte
            myAs.sort(vecWork, len, myMet);
                // il vettore ha ora i numeri riordinati
        }
        while(myAs.isSortable());
    } //[m][s] main

} //{c} ArraySort

