package FI2.String;
/**{p}
 * -- FI2.String --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Stringhe
 */

/**{f}
 * Cl0TokenDesc.java
 * descrittore di token con semantica
 *
 * @version 2.00 00/09/27
 * @author M.Moro DEI UPD
 */


import java.util.*;
import FI2.Linear.*;
import FI2.Set.ObjectIterator;

/**{c}
 * descrittore di token: aggiunge al token base
 * un attributo semantico (indice per i simboli,
 * valore per i numeri, ecc.) 
 * e una lista di righe di comparsa
 */
public class Cl0TokenDesc extends Token
{
    // codifiche di tipo (public)
    public static final int GENKEYTOK = -1;  // token generico
    public static final int NULTOK = 0;
    public static final int NATTOK = NULTOK+1;  // numero naturale
    public static final int IDENTOK = NATTOK+1;  // identificatore
    public static final int KINTOK = IDENTOK+1;  // keyword in
    public static final int KOUTTOK = KINTOK+1;  // keyword out
    public static final int KENDTOK = KOUTTOK+1;  // keyword end
    public static final int STRTOK = KENDTOK+1;  // stringa
    public static final int SEMICTOK = STRTOK+1;  // operatore ;
    public static final int ASSTOK = SEMICTOK+1;  // operatore =
    public static final int LEFPTOK = ASSTOK+1;  // operatore (
    public static final int RIGPTOK = LEFPTOK+1;  // operatore )
    public static final int MULTOK = RIGPTOK+1;  // operatore *
    public static final int PLUSTOK = MULTOK+1;  // operatore +
    public static final int MINUSTOK = PLUSTOK+1;  // operatore -
    public static final int DIVTOK = MINUSTOK+1;  // operatore /
    public static final int COMMATOK = DIVTOK+1;  // operatore ,
    public static final int REFTOK = COMMATOK+1;  // operatore ^
    public static final int EOFTOK = REFTOK+1;  // EOF
    public static final int LEXERR = EOFTOK+1;  // errore lessicale

    // descrizioni (package)
    static final String NATSCODE = "NATV";
    static final String IDENSCODE = "IDEN";
    static final String KEYWSCODE = "KEYW";
    static final String STRSCODE = "STRI";
    static final String OPERSCODE = "OPER";
    static final String EOFSCODE = "EOFI";
    static final String ERRSCODE = "ERRI";

    private static final int MAXROWS = 20;  // capacita` lista righe

    private Object attr;  // attributo semantico
    private Sequence rows;  // lista delle linee di comparsa

    /**[m]
     * imposta la stringa descrittiva del tipo
     * (operatore, identificatore, ecc)
     * @param s  stringa
     * @param t  codifica di tipo
     * @return descrizione del tipo
     */
    private static String toTypeDesc(String s, int type)
    {
        String scode;

        switch(type)
        {
          case NATTOK:  // naturale
            scode = NATSCODE;
            break;
          case IDENTOK:  // identificatore
            scode = IDENSCODE;
            break;
          case KINTOK:  // keyword in
          case KOUTTOK:  // keyword out
          case KENDTOK:  // keyword end
            scode = KEYWSCODE;
            break;
          case STRTOK:  // stringa
            scode = STRSCODE;
            break;
          case EOFTOK:  // eof
            scode = EOFSCODE;
            break;
          case LEXERR:  // errore lessicale
            scode = ERRSCODE;
            break;
          default:  // operatore
            scode = s;
        } //switch
        return scode;
    } //[m] toTypeDesc

    /**[c]
     * costruttore base
     * @param s  stringa
     * @param t  codifica di tipo
     * @param r  riga
     * @param p  posizione
     * @param v  attributo semantico
     */
    public Cl0TokenDesc(String s, int t, int r, int p, Object v)
    {
        super(s, t, toTypeDesc(s, t), r, p);
        attr = v;
        rows = new ArraySequence();
        rows.insertLast(new Integer(r));
    } //[c] Cl0TokenDesc (String s ...)

    /**[m]
     * conversione a stringa estesa
     * @return descrizione estesa
     */
    public String toString(int dummy)
    {  return super.toString(dummy)+" [attr = <"+attr+">]"; }

    /**[m]
     * iteratore delle righe
     * @return enumeratore
     */
    public ObjectIterator rows()
    {
        return rows.elements();
    } //[m] rows

    /**[m]
     * imposta valore semantico
     * @param v  valore semantico
     */
    public void setAttr(Object v)
    {
        attr = v;
    } //[m] setAttr
    
    /**[m]
     * legge valore semantico
     * @return valore semantico
     */
    public Object attr()
    {  return attr;  }
    
    /**[m]
     * aggiunge una riga alla lista
     * @param row  riga da aggiungere
     */
    public void addRow(int row)
    {
        rows.insertLast(new Integer(row));
    } //[m] addRow

} //{c} Cl0TokenDesc
