package FI2.String;
/**{p}
 * -- FI2.String --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Stringhe
 */

/**{f}
 * KMPMatch.java
 * ricerca di sottostringa con l'algoritmo
 * Knuth-Morris-Pratt
 *
 * @version 1.00 98/09/25
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.FI2Sys;

/**{c}
 * realizza il metodo indexOf per la ricerca
 * della prima occorrenza di una sottostringa
 * con il metodo efficiente KMP
 */
public class KMPMatch
{
    /**[m][s]
     * ricerca la seconda stringa (m car.) 
     * nella prima (n car.) con metodo KMP -
     * O(n+m)
     * @param t  stringa in cui ricercare
     * @param p  sottostringa (pattern) da cercare
     * @return l'indice (0..n-m) della prima 
     *         occorrenza di p, se trovata;
     *         -1 altrimenti
     */
    public static int indexOf(String t, String p)
    {
        int n = t.length();  // lunghezze
        int m = p.length();
        if (n == 0)
            // stringa nulla
            return -1;
        int fail[] = new int[m];
        computeFailure(fail, p);
        int j = 0;
        for (int i=0; i<n; i++)
        {
            while (j>0 && p.charAt(j) != t.charAt(i))
                j = fail[j-1];
            if (p.charAt(j) == t.charAt(i))
                j++;
            if (j == m)
                // trovata
                return i-m+1;
        } //for
        // non trovata
        return -1;
    } //[m][s] indexOf

    /**[m][s]
     * calcolo della funzione di failure per KMP
     * O(m)
     * @param p  sottostringa (pattern) da cercare
     * @param fail  vettore di failure da impostare
     */
    private static void 
      computeFailure(int fail[], String p)
    {
        int j = 0;
        fail[0] = 0;
        for (int i=1; i<p.length(); i++)
        {
            while (j>0 && p.charAt(j) != p.charAt(i))
                j = fail[j-1];
            if (p.charAt(j) == p.charAt(i))
                j++;
            fail[i] = j;
        } //for
    } //[m][s] computeFailure

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  non usato
     */
    public static void main(String[] args)
    {
        String t = "0123456789abcdefgefg";
        String p1 = "9abaef";
        String p2 = "9abc";
        String p3 = "efgefg";
        FI2Sys.out.println(p1 + " in " + t + " = " +
          indexOf(t, p1));
        FI2Sys.out.println(p2 + " in " + t + " = " +
          indexOf(t, p2));
        FI2Sys.out.println(p3 + " in " + t + " = " +
          indexOf(t, p3));
    } //[m][s] main

} //{c} KMPMatch
