package FI2.Temi;
/**{p}
 * -- FI2.Temi --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Temi d'esame
 */

/**{f}
 * StringVal.java
 * Scritto del 20/2/99
 * implementazione dell'interfaccia NatAssocMem
 *
 * @version 2.00
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Dictionary.Dictionary;
import FI2.Dictionary.UnOrderedSLList;
import FI2.Util.FI2Sys;
import FI2.Util.StringComparator;
import FI2.Exceptions.*;

/**{c}
 * metodo dell'adattatore su UnOrderedSListSeq
 * @see NatAssocMem
 * @see UnOrderedSLList
 */
public class StringVal implements NatAssocMem
{
    private UnOrderedSLList S;  // dizionario di supporto
    private static final int NOTPRESENT = -1;
      // condizione di non presente

    /**[c]
     * costruttore base, sequenza vuota
     */
    public StringVal()  
    {
        S = new UnOrderedSLList(new StringComparator());
    } //[c] StringVal

    /**[m]
     * @see NatAssocMem
     */
    public int lookup (Object obj)
    {
        Object valObj;

        if ((valObj = S.findElement(obj)) == 
          Dictionary.NO_SUCH_KEY)
            return NOTPRESENT;
        return ((Integer)valObj).intValue();
    } //[m] lookup

    /**[m]
     * @see NatAssocMem
     */
    public void insert(Object obj, int val)
      throws AlreadyPresentException
    {
        if (lookup(obj) != NOTPRESENT)
            throw new AlreadyPresentException(obj.toString());
        if (val<0 || val>500)
            throw new InvalidValueException(
              new Integer(val).toString());
        S.insertItem(obj, new Integer(val));
    } //[m] insert

    /**[m]
     * @see NatAssocMem
     */
    public int delete(Object obj)
      throws NotPresentException
    {
        int val;

        if ((val = lookup(obj)) == NOTPRESENT)
            throw new NotPresentException(obj.toString());
        S.removeElement(obj);
        return val;
    } //[m] insert

    /**[m]
     * metodo di supporto pe acquisire un valore numerico
     * @param gets  stringa di invito
     * @param st  stream di input
     * @return valore acquisito
     */
    private static int getVal(String gets, 
      StreamTokenizer st)
      throws IOException
    {
        do
        {
            FI2Sys.out.println(gets);
            st.nextToken();
            if (st.ttype == StreamTokenizer.TT_EOF)
                throw new IOException("EOF");
        }
        while (st.ttype != StreamTokenizer.TT_NUMBER);
        return (int)st.nval;
    } //[m] getVal

    /**[m][s]
     * metodo di collaudo
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in = System.in;

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);

        sIn.parseNumbers();

        StringVal myDictio = new StringVal();

        int N; // numero di oggetti

        do
        {
            N = getVal("Battere numero oggetti (N):", sIn);
        }
        while (N<=0 || N>200);

        for (int i=1; i<=N; i++)
        {
            do
            {
                FI2Sys.out.println("Battere stringa da inserire:");
                sIn.nextToken();
                if (sIn.ttype == StreamTokenizer.TT_EOF)
                    throw new IOException("EOF");
            }
            while (sIn.ttype != StreamTokenizer.TT_WORD);
            String obj = new String(sIn.sval);

            for (; ; )
            {
                try
                {
                    int val = getVal("Battere valore associato (0<=val<=500):",
                          sIn);
                    myDictio.insert(obj, val);
                    break;
                }
                catch (InvalidValueException e)
                {
                    FI2Sys.out.println(e);
                }
                catch (IOException e)
                {
                    break;
                }
            } //for(;;)
        } //for i

        for (int i=1; i<=5; i++)
        {
            do
            {
                FI2Sys.out.println("Battere stringa da cercare:");
                sIn.nextToken();
                if (sIn.ttype == StreamTokenizer.TT_EOF)
                    throw new IOException("EOF");
            }
            while (sIn.ttype != StreamTokenizer.TT_WORD);
            String obj = new String(sIn.sval);

            int val = myDictio.lookup(obj);
            if (val == NOTPRESENT)
                FI2Sys.out.println("Stringa ->" + obj + "<- non presente");
            else
                FI2Sys.out.println("La stringa ->" + obj + 
                  "<- ha associato il valore: " + val);
        } //for i

        do
        {
            FI2Sys.out.println("Battere stringa da rimuovere:");
            sIn.nextToken();
            if (sIn.ttype == StreamTokenizer.TT_EOF)
                throw new IOException("EOF");
        }
        while (sIn.ttype != StreamTokenizer.TT_WORD);
        String obj = new String(sIn.sval);
        try
        {
            myDictio.delete(obj);
            FI2Sys.out.println("Stringa ->" + obj + "<- rimossa");
        }
        catch (NotPresentException e)
        {
            FI2Sys.out.println("Stringa ->" + obj + "<- non presente");
        }
    } //[m][s] main

} //{c} NatAssocMem
