package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * BracketedExpressionTour.java
 * ricostruzione con parentesi dell'espressione da albero
 * astratto
 *
 * @version 2.00 00/09/24
 * @author M.Moro DEI UPD
 */

import FI2.Set.Position;
import FI2.Util.VariableInfo;
import FI2.Util.IntegerVariableInfo;
import FI2.Util.DoubleVariableInfo;
import FI2.Util.OperatorInfo;
import FI2.Util.DoubleAdditionOperator;
import FI2.Util.DoubleSubtractionOperator;
import FI2.Util.DoubleOppositeOperator;
import FI2.Util.DoubleMultiplicationOperator;
import FI2.Util.DoubleDivisionOperator;
import FI2.Util.IntegerAdditionOperator;
import FI2.Util.IntegerSubtractionOperator;
import FI2.Util.IntegerOppositeOperator;
import FI2.Util.IntegerMultiplicationOperator;
import FI2.Util.IntegerDivisionOperator;
import FI2.Util.FI2Sys;

/**{c}
 * assume di ricevere un albero binario che esprime l'albero
 * astratto di una espressione algebrica e ne effettua la
 * ricostruzione dell'espressione con le parentesi
 * attraverso una visita ricorsiva con il tour di
 * Eulero -
 * l'elemento memorizzato in cascuno dei nodi foglia 
 * e` un descrittore di operando (VariableIndo) 
 * mentre quello dei nodi interni e` un descrittore 
 * di operatore (OperatorInfo)
 * @see EulerTour
 * @see VariableInfo
 * @see OperatorInfo 
 */
public class BracketedExpressionTour 
  extends EulerTour 
{
    /**[m]
     * @see EulerTour
     */
    public Object execute(InspectableBinaryTree T) 
    {
        super.execute(T);
        FI2Sys.out.println("L'espressione ricostruita: ");
        eulerTour(tree.root());
        FI2Sys.out.println(" ");
        return null;
    } //[m] execute

    /**[m]
     * @see EulerTour
     */
    protected void visitExternal(Position p, TraversalResult r)
    {
        VariableInfo v = (VariableInfo)p.element();
        if (v != null)
            FI2Sys.out.print(v.variable().toString() + " ");
          //in questo modo sono gestiti anche operatori unari
          // per i quali il figlio di destra 
          // e` una foglia nulla
    } //[m] external

    private int priority(Position p)
    {
        if (p.element() instanceof DoubleVariableInfo)
            return 6;
        // operatore
        if (p.element() instanceof DoubleAdditionOperator)
            // +
            return 1;
        if (p.element() instanceof DoubleSubtractionOperator)
            // - binario
            return 2;
        if (p.element() 
          instanceof DoubleMultiplicationOperator)
            // *
            return 3;
        if (p.element() instanceof DoubleDivisionOperator)
            // /
            return 4;
        if (p.element() instanceof DoubleOppositeOperator)
            // /
            return 5;
        // altro??
        return 6;
    } //[m] priority

    /**[m]
     * @see EulerTour
     */
    protected void visitLeft(Position p, TraversalResult r)
    { 
        if (tree.isExternal(p))
            // operando, niente da fare
            return;
        if (priority(p) >= priority(tree.leftChild(p)))
            // non e` giustificata: ci vuole la parentesi
            // la relazione risulta falsa se leftSon
            // e` operando
            FI2Sys.out.print("( ");
    } //[m] left

    /**[m]
     * @see EulerTour
     */
    protected void visitBelow(Position p, TraversalResult r)
    { 
        if (tree.isExternal(p))
        {
            // operando, solo il nodo
            FI2Sys.out.print(p.element() + " ");
            return;
        }
        if (priority(p) >= priority(tree.leftChild(p)))
            // non e` giustificata: ci vuole la parentesi
            // la relazione risulta falsa se leftSon
            // e` operando
            FI2Sys.out.print(") ");
        FI2Sys.out.print(p.element() + " ");
        if (priority(p) >= priority(tree.rightChild(p)))
            // non e` giustificata: ci vuole la parentesi
            // la relazione risulta falsa se rightSon
            // e` operando
            FI2Sys.out.print("( ");
    } //[m] left

    /**[m]
     * @see EulerTour
     */
    protected void visitRight(Position p, TraversalResult r)
    { 
        if (tree.isExternal(p))
            // operando, niente da fare
            return;
        if (priority(p) >= priority(tree.rightChild(p)))
            // non e` giustificata: ci vuole la parentesi
            // la relazione risulta falsa se rightSon
            // e` operando
            FI2Sys.out.print(") ");
    } //[m] right

} //{c} BracketedExpressionTour
