package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * InspectableTree.java
 * interfaccia per albero generico
 * con metodi read-only -
 * derivata da SimpleTree della v 1.x
 *
 * @version 2.00 00/09/22
 * @author M.Moro DEI UPD
 */

import FI2.Set.Position;
import FI2.Set.PositionIterator;
import FI2.Set.InspectablePositionalContainer;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.InvalidContainerException;
import FI2.Exceptions.BoundaryViolationException;

/**{i}
 * albero generico con metodi di accesso 
 * di sola lettura -
 * quasi tutte le operazioni agiscono sul nodo indicato da
 * un posizionatore
 * @see InspectablePositionalContainer
 */
public interface InspectableTree 
  extends InspectablePositionalContainer
{
    /**[m][a]
     * il nodo radice dell'albero
     * @return un posizionatore alla radice
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    Position root()
      throws InvalidContainerException;

    /**[m][a]
     * il nodo padre di un nodo dell'albero
     * @param v  nodo di cui restituire il padre
     * @return nodo padre di v
     * @exception BoundaryViolationException
     * v e` la radice dell'albero contenitore
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    Position parent(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException;

    /**[m][a]
     * l'insieme dei nodi figlio di un nodo dell'albero
     * @param v  nodo di cui restituire i figli
     * @return un iteratore dei figli
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    PositionIterator children(Position v)
      throws InvalidPositionException, 
      InvalidContainerException;

    /**[m][a]
     * l'insieme dei nodi fratello, da sinistra a destra,
     * diversi dal nodo fornito dell'albero -
     * non e` documentato in [Good] pag. 234 ma e` 
     * presente nella libreria JDSL
     * @param v  nodo di cui restituire i figli
     * @return un iteratore dei figli
     * @exception BoundaryViolationException
     * v e` la radice dell'albero contenitore
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    PositionIterator siblings(Position v)
      throws InvalidPositionException, 
      InvalidContainerException;

    // metodi di interrogazione

    /**[m][a]
     * il nodo fornito e` nodo interno dell'albero?
     * @param v  nodo da valutare
     * @return true se nodo interno, false altrimenti
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    boolean isInternal(Position v)
      throws InvalidPositionException, 
      InvalidContainerException;

    /**[m][a]
     * il nodo fornito e` nodo foglia dell'albero?
     * @param v  nodo da valutare
     * @return true se nodo foglia, false altrimenti
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    boolean isExternal(Position v)
      throws InvalidPositionException, 
      InvalidContainerException;

    /**[m][a]
     * il nodo fornito e` nodo radice dell'albero?
     * @param v  nodo da valutare
     * @return true se nodo radice, false altrimenti
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    boolean isRoot(Position v)
      throws InvalidPositionException, 
      InvalidContainerException;

} //{i} InspectableTree
