package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * LinkedBinaryTree.java
 * implementazione di albero binario mediante
 * nodi concatenati
 *
 * @version 2.00 00/09/23
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.Position;
import FI2.Set.ObjectIterator;
import FI2.Set.PositionIterator;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.PositionalContainer;
import FI2.Set.InspectablePositionalContainer;
import FI2.Util.FI2Sys;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.InvalidContainerException;
import FI2.Exceptions.BoundaryViolationException;

/**{c}
 * albero binario - la radice e i nodi sono BTNode -
 * un albero viene costruito per default come 
 * costituito da un unico nodo nullo
 * (definito come nodo con campo element == null): 
 * poiche' il conteggio include gli eventuali nodi nulli, 
 * un albero 'vuoto di elementi' ha pero` size == 1 -
 * non e` necessario un metodo insert(): la costruzione
 * procede con l'uso del metodo expandExternal() a 
 * partire dall'albero iniziale -
 * un nodo foglia ha i riferimenti left e right == null -
 * per semplicita` di trattamento, si assume che un nodo 
 * interno abbia ENTRAMBI i riferimenti nonnulli
 * (se necessario, uno dei due puo` riferire un nodo nullo) -
 * quando un albero perde tutti i nodi (si veda ad esempio
 * replaceSubTree) viene invalidato (root = null) -
 * vi sono alcune differenze con la realizzazione di [Good]
 * pag. 268, in particolare per quanto riguarda gli iteratori
 * che operano direttamente sulla struttura e non su una
 * copia dei suoi nodi: per consistenza, si assume che la
 * navigazione con un iteratore si completi prima che
 * l'albero subisca modifiche
 */
public class LinkedBinaryTree implements ExpandBinaryTree
{
    private static final int LEFTCHILD = 0;
    private static final int RIGHTCHILD = 1;
    private static final int DONE = 2;
      // per gli enumeratori

    private Position root;  // posizionatore al nodo radice
    private int size;  // numero totale di nodi

    /**[c]
     * costruttore di default, albero vuoto
     */
    public LinkedBinaryTree()
    {
        root = new BTNode(null, null, null, null, this);
        size = 1;
    } //[c] LinkedBinaryTree

    /**[c]
     * costruttore con sottoalbero
     */
    public LinkedBinaryTree(BTNode r, int numNodes)
    {
        root = r;
        size = numNodes;
    } //[c] LinkedBinaryTree

    /**[m]
     * O(1)
     * @see InspectableContainer
     */
    public int size()
    {
        return size;
    } //[m] size

    /**[m]
     * O(1)
     * @see InspectableContainer
     */
    public boolean isEmpty()
    {
        return size == 0;
    } //[m] isEmpty

    /**[m]
     * O(n)
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore
         * interno per gli elementi -
         * sfrutta quello delle posizioni
         * @see PosIter
         */
        class Iter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore di posizioni

            /**[c]
             * costruttore di default
             */
            Iter()
            {
                posIt = positions();
            } //[c] Iter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return posIt.hasNext();
            } //[m] hasNext
            
            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return 
                  ((BTNode)posIt.nextPosition())
                  .getElement();
            } //[m] nextElement

        } //{c} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new LinkedBinaryTree();
    } //[m] newContainer

    /**[m]
     * @see InspectablePositionalContainer
     */
    public PositionIterator positions()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per le posizioni
         */
        class PosIter implements PositionIterator
        {
            private Position p;

            /**[c]
             * costruttore di default
             */
            PosIter()
            {
                p = first(root);
                  // prosizionatore al primo
            } //[c] PosIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                try
                {
                    p.element();
                      // tenta l'accesso all'elemento
                    // ok, c'e`
                    return true;
                }
                catch (InvalidPositionException e)
                {
                    // no, non c'e`
                    return false;
                }
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return nextPosition();
            }

            /**[m]
             * @see PositionIterator
             */
            public Position nextPosition()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"In LinkedBinaryTree.PosIter.nextObject: fine elementi"
                  );
                Position curPos = p;
                try
                {
                    p = after(p);
                      // tenta l'aggiornamento di succ.
                    // ok, c'e`
                }
                catch (BoundaryViolationException e)
                {
                    // no, non c'e`, posizionatore illegale
                    p = new 
                      BTNode(null, null, null, null, null);
                }
                return curPos;
            } //[m] nextPosition
            
        } //{c} PosIter

        return new PosIter();
    } //[m] positions

    /**[m]
     * verifica la validita` del posizionatore -
     * O(1)
     * @param p  posizionatore da verificare
     * @return il posizionatore come BTNode
     * se legale
     * @exception InvalidPositionException  
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore non valido
     */
    protected BTNode checkPosition(Position p)
      throws InvalidPositionException,
      InvalidContainerException
    {
        // p.container() verifica se il posizionatore
        // e` legale
        if (p.container() != this)
            throw new InvalidPositionException(
              "Posizionatore riferito ad altro contenitore");
        if (root == null)
            throw new InvalidContainerException(
              "Contenitore non valido");
        try
        {
            BTNode temp = (BTNode) p;
            return temp;
        }
        catch (ClassCastException e)
        {
            // il posizionatore non e` di tipo BTNode
            throw new InvalidPositionException(
"Posizionatore di tipo non corretto per questo contenitore");
        }
    } //[m] checkPosition

    /**[m]
     * O(1)
     * @see PositionalContainer
     */
    public Object replaceElement(Position p, Object newElem)
      throws InvalidPositionException,
      InvalidContainerException
    {
        BTNode here = checkPosition(p);
        Object oldElem = here.getElement();
        here.setElement(newElem);
        return oldElem;
    } //[m] replaceElement

    /**[m]
     * O(1)
     * @see PositionalContainer
     */
    public void swapElements(Position p, Position q)
      throws InvalidPositionException,
      InvalidContainerException
    {
        BTNode first = checkPosition(p);
        BTNode second = checkPosition(q);
        Object oldFirstElem = first.getElement();
        first.setElement(second.getElement());
        second.setElement(oldFirstElem);
    } //[m] swapElements

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public Position root()
      throws InvalidContainerException
    {
        if (root == null)
            throw new InvalidContainerException(
              "LinkedBinaryTree.root(): Contenitore illegale"
          );
        return root;
    } //[m] root

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public Position parent(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, 
      InvalidContainerException
    {
        if (v == root)
            throw new BoundaryViolationException(
  "LinkedBinaryTree.parent(): il nodo e` la radice");
        BTNode here = checkPosition(v);
        return here.getParent();
    } //[m] parent

    /**[m]
     * O(nc)
     * @see InspectableTree
     */
    public PositionIterator children(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per i figli
         */
        class ChildIter implements PositionIterator
        {
            private BTNode p;  // il nodo padre
            private int order;  // nell'ordine sin-des

            /**[c]
             * costruttore
             * @param v  nodo di cui ricavare i figli
             */
            ChildIter(BTNode v)
            {
                p = v;
                order = isInternal(v) ? LEFTCHILD: DONE;  
                  // parte con il sinistro se non foglia
            } //[c] ChildIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return order != DONE;
            } //[m] hasNext
            
            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return nextPosition();
            }
            
            /**[m]
             * @see PositionIterator
             */
            public Position nextPosition()
            {
                if (order == DONE)
                    throw new NoSuchElementException(
"In LinkedBinaryTree.ChildIter.nextPosition: fine figli"
                  );
                Position curPos;
                if (order == LEFTCHILD)
                {
                    curPos = p.getLeft();
                    order = RIGHTCHILD;
                }
                else
                {
                    curPos = p.getRight();
                    order = DONE;
                }
                return curPos;
            } //[m] nextPosition

        } //{c} ChildIter

        BTNode here = checkPosition(v);
        return new ChildIter(here);
    } //[m] children

    /**[m]
     * O(ns)
     * @see InspectableTree
     */
    public PositionIterator siblings(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per i fratelli
         */
        class SiblingIter implements PositionIterator
        {
            private BTNode p;  // il nodo padre
            private int order;  // nell'ordine sin-des

            /**[c]
             * costruttore
             * @param v  nodo di cui ricavare i figli
             */
            SiblingIter(BTNode v)
            {
                p = v.getParent();
                order = (p.getRight() == v) ? 
                  LEFTCHILD: RIGHTCHILD; 
                  // l'altro fratello
            } //[c] SiblingIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return order != DONE;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return nextPosition();
            }

            /**[m]
             * @see PositionIterator
             */
            public Position nextPosition()
            {
                if (order == DONE)
                    throw new NoSuchElementException(
"In LinkedBinaryTree.SiblingIter.nextObject: fine fratelli"
                  );
                Position curPos;
                if (order == LEFTCHILD)
                    curPos = p.getLeft();
                else
                    curPos = p.getRight();
                order = DONE;
                return curPos;
            } //[m] nextObject

        } //{c}{l} SiblingIter

        if (v == root)
            throw new BoundaryViolationException(
  "In LinkedBinaryTree.siblings: il nodo e` la radice");
        BTNode here = checkPosition(v);
        return new SiblingIter(here);
    } //[m] siblings

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public boolean isInternal(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        BTNode here = checkPosition(v);
        return here.getLeft() != null
          // || here.getRight() != null
          ;
          // si assume che una foglia abbia entrambi 
          // i riferimenti nulli
    } //[m] isInternal

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public boolean isExternal(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        return !isInternal(v);
    } //[m] isExternal

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public boolean isRoot(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        BTNode here = checkPosition(v);
        return here == root;
    } //[m] isRoot

    /**[m]
     * O(1)
     * @see InspectableBinaryTree
     */
    public Position leftChild(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        BTNode here = checkPosition(v);
        if (here.getLeft() == null)
          // foglia
            throw new BoundaryViolationException(
  "In LinkedBinaryTree.leftChild: il nodo e` una foglia");
        return here.getLeft();
    } //[m] leftChild

    /**[m]
     * O(1)
     * @see InspectableBinaryTree
     */
    public Position rightChild(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        BTNode here = checkPosition(v);
        if (here.getRight() == null)
          // foglia
            throw new BoundaryViolationException(
  "In LinkedBinaryTree.rightChild: il nodo e` una foglia");
        return here.getRight();
    } //[m] rightChild

    /**[m]
     * O(1)
     * @see InspectableBinaryTree
     */
    public Position sibling(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        if (v == root)
            throw new BoundaryViolationException(
  "In LinkedBinaryTree.sibling: il nodo e` la radice");
        BTNode here = checkPosition(v);
        BTNode par = here.getParent();
        return (par.getLeft() == v) ? 
          par.getRight() : par.getLeft();
    } //[m] sibling

    /**[m]
     * O(1)
     * @see ExpandBinaryTree
     */
    public void expandExternal(Position vExt)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        BTNode here = checkPosition(vExt);
        if (here.getRight() != null)
          // non foglia
            throw new InvalidPositionException(
"In LinkedBinaryTree.expandExternal: " + 
              "il nodo non e` una foglia");
        here.setLeft(new 
          BTNode(null, here, null, null, this));
        here.setRight(new 
          BTNode(null, here, null, null, this));
        size +=2;
    } //[m] expandExternal
    
    /**[m] !!
     * il nodo fornito diviene figlio 
     * sinistro o destro a scelta di un nuovo
     * nodo interno che diventa figlio (vuoto) del 
     * padre del nodo indicato e ha un altro figlio 
     * (vuoto) -
     * O(1)
     * @param v  nodo da espandere
     * @param expandLeft  true se deve diventare 
     *                    figlio sinistro, false destro
     * @return il nuovo nodo padre di v
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    public Position expandInternal(Position v, 
      boolean expandLeft)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        BTNode here = checkPosition(v);
        BTNode par = here.getParent();
          // il vecchio padre 
        BTNode newExt = new 
          BTNode(null, null, null, null, this);
        BTNode newN = new BTNode(null, par, 
          expandLeft ? here : newExt, 
          expandLeft ? newExt : here, this);
        newExt.setParent(newN);
        here.setParent(newN);
        if (here == root)
            // il nuovo diventa radice
            root = newN;
        else
            // si aggiorna il padre
            if (par.getLeft() == here)
                // era figlio di sinistra
                par.setLeft(newN);
            else
                // era figlio di destra
                par.setRight(newN);
        size +=2;
        return newN;
    } //[m] expandInternal

    /**[m]
     * O(1)
     * @see ExpandBinaryTree
     */
    public Object removeAboveExternal(Position vExt)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        if (vExt == root)
            throw new BoundaryViolationException(
  "In LinkedBinaryTree.removeAboveExternal: " + 
              "il nodo e` la radice");
        BTNode here = checkPosition(vExt);
        if (here.getRight() != null)
          // non foglia
            throw new InvalidPositionException(
"In LinkedBinaryTree.removeAboveExternal: " + 
              "il nodo non e` una foglia");
        BTNode par = here.getParent();
          // il padre della foglia
        BTNode gpar = par.getParent();
          // il nonno della foglia; null se par==root
        BTNode sib = (par.getLeft() == vExt) ? 
          par.getRight() : par.getLeft();
          // il fratello
        Object obj = par.getElement();
        if (gpar == null)
          // non c'e` il nonno
        {
            // il fratello diventa radice
            root = sib;
            sib.setParent(null);
        }
        else
        {
            // si aggiorna il nonno
            if (gpar.getLeft() == par)
                // nuovo figlio di sinistra
                gpar.setLeft(sib);
            else
                // nuovo figlio di destra
                gpar.setRight(sib);
            sib.setParent(gpar);
        } //else
        size -=2;
        return obj;
    } //[m] removeAboveExternal

    /**[m]
     * @see ExpandBinaryTree
     */
    public ExpandBinaryTree cut(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        return replaceSubtree(v, new LinkedBinaryTree());
          // il taglio puo` essere visto come scambio
          // con un sottoalbero vuoto
    } //[m] cut
    
    /**[m]
     * @see ExpandBinaryTree
     */
    public void link(Position vExt, ExpandBinaryTree subTree)
      throws IllegalArgumentException,
      InvalidPositionException, InvalidContainerException
    {
        if (((BTNode)vExt).getLeft() != null)
          // non foglia
            throw new InvalidPositionException(
"In LinkedBinaryTree.link: il nodo non e` una foglia"
          );
        replaceSubtree(vExt, subTree);
          // viene scartato il sottoalbero restituito
    } //[m] link

    /**[m]
     * O(n) (a causa di updateCont())
     * @see ExpandBinaryTree
     */
    public ExpandBinaryTree 
      replaceSubtree(Position v, ExpandBinaryTree subTree)
      throws IllegalArgumentException,
      InvalidPositionException, InvalidContainerException
    {
        if (subTree == this)
            throw new IllegalArgumentException(
  "In LinkedBinaryTree.replaceSubTree: " + 
              "il sottoalbero coincide con l'albero");
        LinkedBinaryTree sTree;
        try
        {
            sTree = (LinkedBinaryTree) subTree;
        }
        catch (ClassCastException e)
        {
            // l'albero non e` di tipo LinkedBinaryTree
            throw new InvalidPositionException(
"In LinkedBinaryTree.replaceSubTree: " + 
              "sottoalbero di tipo incompatibile");
        }
        BTNode here = checkPosition(v);

        BTNode par = here.getParent();
          // il padre del nodo
        // si aggiorna il padre, se c'e`
        if (par == null)
            root = sTree.root;
              // il nodo e` radice
        else 
            if (par.getLeft() == here)
                // nuovo figlio di sinistra
                par.setLeft(sTree.root);
            else
                // nuovo figlio di destra
                par.setRight(sTree.root);
        ((BTNode)sTree.root).setParent(par);
        int hereSize = count(here);
          // numero nodi rimossi
        size += (sTree.size-hereSize);
        // tutti i nodi del sottoalbero attaccato vanno 
        // sistemati come contenitore
        updateCont(sTree.root);

        // subTree diventa vuoto
        sTree.root = new BTNode(null, null, null, null, 
          subTree);
        sTree.size = 1;

        LinkedBinaryTree newTree = new 
          LinkedBinaryTree(here, hereSize);
        // tutti i nodi del nuovo albero vanno sistemati come
        // contenitore
        newTree.updateCont(here);
        return newTree;
    } //[m] replaceSubTree

    /**[m]
     * fornisce l'estremita` sinistra del sottoalbero -
     * il contenitore e` assunto corretto -
     * O(log n) se bilanciato
     * @param v  posizionatore al sottoalbero
     * @return il nodo estremita` sinistra
     */
    private BTNode first(Position v)
    {

        BTNode here = (BTNode) v;
        while (here.getLeft() != null)
            here = here.getLeft();
        // curNode e` l'estremita` sinistra
        return here;
    } //[m] first

    /**[m]
     * nodo successore secondo la visita in ordine -
     * il contenitore e` assunto corretto -
     * O(log n) se bilanciato
     * @param v  nodo corrente
     * @return il nodo successore se this != last
     * @exception BoundaryViolationException
     * v e` l'ultimo nodo della visita
     */
    private BTNode after(Position v)
      throws BoundaryViolationException
    {
        if (((BTNode)v).getRight() != null)
            // se c'e` il figlio di destra, il successore
            // e` l'estremo sinistro del sottoalbero di destra
            return first(((BTNode)v).getRight());
        // cerca all'indietro il primo padre 'attaccato'
        // a sinistra 
        BTNode curPar = ((BTNode)v).getParent(), 
          curNode = ((BTNode)v);
        while (curPar!=null && curNode == curPar.getRight())
        {
            curNode = curPar;
            curPar = curPar.getParent();
        } //while
        if (curPar == null)
            throw new BoundaryViolationException(
  "In LinkedBinaryTree.after: il nodo e` l'ultimo");
        return curPar;
    } //[m] after

    /**[m]
     * conta ricorsivamente i nodi di un sottoalbero -
     * il contenitore e` assunto corretto - 
     * O(n)
     * @param v  radice sottoalbero
     * @return il conteggio
     */
    private int count(Position v)
    {
        int sum = 1;
        if (((BTNode)v).getLeft() != null)
            // se c'e` il figlio di sinistra, 
            // aggiungi conteggio
            sum += count(((BTNode)v).getLeft());
        if (((BTNode)v).getRight() != null)
            // se c'e` il figlio di destra, aggiungi conteggio
            sum += count(((BTNode)v).getRight());
        return sum;
    } //[m] count

    /**[m]
     * aggiorna ricorsivamente il riferimento al 
     * contenitore nei nodi di un sottoalbero -
     * il contenitore e` assunto corretto - 
     * O(n)
     * @param v  radice sottoalbero
     */
    private void updateCont(Position v)
    {
        ((BTNode)v).setContainer(this);
        if (((BTNode)v).getLeft() != null)
            updateCont(((BTNode)v).getLeft());
        if (((BTNode)v).getRight() != null)
            updateCont(((BTNode)v).getRight());
    } //[m] updateCont

    private StreamTokenizer sBuildIn;
      // stream di input

    /**[m] !!
     * costruisce ricorsivamente un albero
     * con valori interi
     * @return il precedente oggetto
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    protected boolean buildInteger(Position v, 
      int nodes)
      throws IOException
    {
        Integer next = new Integer(0);
        boolean ret = true; // ritorno
        FI2Sys.out.println("Crea albero di " + nodes +
          " elementi");
        try
        {
            while(sBuildIn.nextToken() != 
              StreamTokenizer.TT_EOF)
            {
                if (sBuildIn.ttype == 
                  StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sBuildIn.nval);
                    break;
                } //if
                else  
                    FI2Sys.out.println(sBuildIn.sval +
                      " non numero: non inserito");
            } //while
        } //try
        catch (EOFException e)
        {
            // termina il ciclo
            return false;
        } //catch
        if (nodes > 1)
        {
            expandExternal(v);
            int leftSize = nodes>>1;
              // se del caso, ne mette di piu` a sinistra
              // perche' i nodi ancora da sistemare sono
              // nodes-1. Ad esempio con nodes=4, a sinistra
              // ne mette 2 e a destra 1
            if (leftSize > 0)
                 ret = ret && 
                   buildInteger(leftChild(v), leftSize);
            int rightSize = nodes-1-leftSize;
            if (rightSize > 0)
                 ret = ret && 
                   buildInteger(rightChild(v), rightSize);
        } //if
        replaceElement(v, next);
        return ret;
    } //[m] build

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return BinTreeBasicAlgo.toString(this, root());
    } //[m] toString()

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "BinTree: Battere numero di interi e interi (fine=Ctrl-Z):"
              );
        } //if
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(
          new InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        LinkedBinaryTree myTree = new LinkedBinaryTree();

        sIn.parseNumbers();
        int numEl = 0;
        try
        {
            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    numEl = (int)sIn.nval;
                    break;
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero");
            } //while
        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch

        myTree.sBuildIn = sIn;
          // per le chiamate ricorsive di buildInteger
        myTree.buildInteger(myTree.root(), numEl);
        FI2Sys.out.println("Albero creato");
        FI2Sys.out.println("size=" + myTree.size());
        FI2Sys.out.println(myTree);
        Position root;
        FI2Sys.out.println("\nLa radice e` " + 
          (root=myTree.root()).element());
        FI2Sys.out.println(
          "\nIl figlio sinistro della radice e` " + 
          myTree.leftChild(root).element() + 
          "\n  e suo padre e` " +
          myTree.parent(myTree.leftChild(root)).element());
        FI2Sys.out.println("\nI figli della radice sono:");
        PositionIterator it = myTree.children(root);
        while (it.hasNext())
            FI2Sys.out.println(it.nextPosition().
              element());
        FI2Sys.out.println(
          "\nI fratelli del figlio destro della radice sono:"
          );
        it = myTree.siblings(myTree.rightChild(root));
        while (it.hasNext())
            FI2Sys.out.println(it.nextPosition().
              element());
        FI2Sys.out.println(
"\nIl singolo fratello del figlio destro della radice e` " +
          myTree.sibling(myTree.rightChild(root)).element());
        FI2Sys.out.println("\nVisita in preordine");
        BinTreeBasicAlgo.preorderPrint(myTree, myTree.root());
        FI2Sys.out.println("\nVisita parentetica");
        BinTreeBasicAlgo.parentheticRepresentation(myTree, 
          myTree.root());
        FI2Sys.out.println("\nVisita in ordine interno");
        BinTreeBasicAlgo.inorderPrint(myTree, myTree.root());
        FI2Sys.out.println("\nTour di Eulero");
        BinTreeBasicAlgo.eulerTour(myTree, myTree.root());
        FI2Sys.out.println("\nVisita in postordine");
        BinTreeBasicAlgo.postorderPrint(myTree, 
          myTree.root());
        FI2Sys.out.println("\nAltezza dell'albero=" + 
          BinTreeBasicAlgo.height1(myTree));
        FI2Sys.out.println(
          "\nProfondita` e altezza dei nodi");
        it = myTree.positions();
        int pos = 0;
        while (it.hasNext())
        {
            Position p = it.nextPosition();
            FI2Sys.out.println("Nodo " + (pos++) + " el=" +
              p.element() + " prof.=" +
              BinTreeBasicAlgo.depth(myTree, p) +
              " alt.=" +
              BinTreeBasicAlgo.height2(myTree, p));
        } //while
        LinkedBinaryTree myTree2 = new LinkedBinaryTree();
        FI2Sys.out.println("\nCopia del sottoalbero destro");
        BinTreeBasicAlgo.cloneSubTree(myTree, myTree2, 
          myTree.rightChild(myTree.root()), myTree2.root());
        FI2Sys.out.println(
          "\nVisita parentetica della copia");
        BinTreeBasicAlgo.parentheticRepresentation(myTree2, 
          myTree2.root());
        LinkedBinaryTree isACut = 
          (LinkedBinaryTree)myTree.cut(
          myTree.leftChild(myTree.root()));
        FI2Sys.out.println(
          "\nVisita parentetica dell'albero dopo il cut");
        BinTreeBasicAlgo.parentheticRepresentation(myTree, 
          myTree.root());
        FI2Sys.out.println(
          "\nVisita parentetica del cut");
        BinTreeBasicAlgo.parentheticRepresentation(isACut, 
          isACut.root());
    } //[m][s] main

} //{c} LinkedBinaryTree

