package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * SequenceHeapTree.java
 * implementazione di albero binario 
 * specializzato per heap mediante
 * sequenza
 *
 * @version 1.00 00/09/24
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidContainerException;
import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Set.Position;

/**{c}
 * albero binario per heap - 
 * estende l'implementazione su sequenza
 * di un albero binario in modo da trattarlo
 * come heap
 */
public class SequenceHeapTree
  extends SequenceBinaryTree 
  implements HeapTree
{
    private Position last;  
      // ultima posizione di inserimento

    /**[c]
     * costruttore di default, albero vuoto
     */
    public SequenceHeapTree()
    {
        super();
        last = null;
    } //[c] SequenceHeapTree

    /**[m]
     * @see HeapTree
     */
    public Position add(Object elem)
      throws InvalidContainerException
    {
        Position z = findInsPosition(); 
          // posizione di inserimento
        // z e` la posizione di inserimento ovvero
        // la foglia piu` a sinistra
        expandExternal(z);
        replaceElement(z, elem);
        return (last = z);
    } //[m] add

    /**[m]
     * @see HeapTree
     */
    public Object remove()
      throws EmptyContainerException, 
      InvalidContainerException
    {
        Position tmp = leftChild(last); 
          // questo e` il nodo che sostituisce l'ultimo
        Object t = last.element();
          // attuale elemento
        removeAboveExternal(rightChild(last));
          // rimuove due nodi
        if (size()==1)
            // rimane una coda vuota
            last = null;
        else {
            while (!isRoot(tmp) && isLeftChild(tmp))
                // cerca in su un figlio di destra
                // eventualmente fino alla radice
                tmp = parent(tmp);
            if (!isRoot(tmp))
                tmp = leftChild(parent(tmp));
                // trovato, si va al fratello
            while (!isExternal(tmp))
                // cerca in giu` sul ramo destro la foglia
                tmp = rightChild(tmp);
            last = parent(tmp);
              // nuova posizione di inserimento
        }
        return t;
    }

    /**[m]
     * determina la posizione di inserimento
     * che e` il successore di last
     * @return la posizione del successore di last
     */
    private Position findInsPosition()
    {
        return atRank((size()-1)/2+1);
    } //[m] findInsPosition

    /**[m]
     * verifica figlio di sinistra 
     * @param p  posizionatore 
     * @return true se this e` figlio di sinistra del nodo
     * il cui posizionatore e` fornito
     * @exception InvalidPositionException  
     * posizionatore illegale
     */
    private boolean isLeftChild(Position p) 
      throws InvalidPositionException 
    {
        try 
        {
            return leftChild(parent(p)).equals(p);
        } 
        catch (BoundaryViolationException e) {
            return false; 
              // se p e` la radice
        }
    } //[m] isLeftChild

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return "SequenceHeapTree [size=" + size() +
          "]:\n" + super.toString();
    } //[m] toString()

} //{c} SequenceHeapTree

